package im.zego.chathouse.model;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.annotations.Expose;

import java.util.ArrayList;
import java.util.HashMap;

/**
 * 房间属性
 */
public class RoomAttributes {

    public static final Gson gson = new GsonBuilder().excludeFieldsWithoutExposeAnnotation().create();

    public static final String SEAT_0_TAG = "seat_0";
    public static final String SEAT_1_TAG = "seat_1";
    public static final String SEAT_2_TAG = "seat_2";
    public static final String SEAT_3_TAG = "seat_3";
    public static final String SEAT_4_TAG = "seat_4";
    public static final String INFO_TAG = "info";
    public static final String[] SEAT_LIST = new String[]{SEAT_0_TAG,SEAT_1_TAG,SEAT_2_TAG,SEAT_3_TAG,SEAT_4_TAG};

    /**
     * 麦位0，为房主
     */
    private Seat seat0 = new Seat();
    /**
     * 麦位1，为上麦观众
     */
    private Seat seat1 = new Seat();
    /**
     * 麦位2，为上麦观众
     */
    private Seat seat2 = new Seat();
    /**
     * 麦位3，为上麦观众
     */
    private Seat seat3 = new Seat();
    /**
     * 麦位4，为上麦观众
     */
    private Seat seat4 = new Seat();
    /**
     * 房间属性版本信息
     */
    private final Info info = new Info();

    public RoomAttributes(){
        seat0.setIdx(0);
        seat1.setIdx(1);
        seat2.setIdx(2);
        seat3.setIdx(3);
        seat4.setIdx(4);

        seat0.setStatus(Seat.SeatStatus.OCCUPIED.status);
    }


    /**
     * 获取对应麦位信息
     * @param position 麦位
     * @return 麦位
     */
    public Seat getSeat(int position) {
        switch (position){
            case 0:
                return seat0;
            case 1:
                return seat1;
            case 2:
                return seat2;
            case 3:
                return seat3;
            case 4:
                return seat4;
            default:
                return null;
        }
    }

    public void setSeat(Seat seat,int position) {
        switch (position){
            case 0:
                seat0 = seat;
                break;
            case 1:
                seat1 = seat;
                break;
            case 2:
                seat2 = seat;
                break;
            case 3:
                seat3 = seat;
                break;
            case 4:
                seat4 = seat;
                break;
            default:

        }
    }

    /**
     * 获取房间属性序列化对应的map
     * @return map
     */
    public HashMap<String, String> getSerializeMap(){

        HashMap<String, String> serializeMap = new HashMap<>();
        serializeMap.put(SEAT_0_TAG, gson.toJson(seat0));
        serializeMap.put(SEAT_1_TAG, gson.toJson(seat1));
        serializeMap.put(SEAT_2_TAG, gson.toJson(seat2));
        serializeMap.put(SEAT_3_TAG, gson.toJson(seat3));
        serializeMap.put(SEAT_4_TAG, gson.toJson(seat4));
        serializeMap.put(INFO_TAG, gson.toJson(info));
        return serializeMap;
    }

    /**
     * 获取房间属性序列化对应的map
     * @return map
     */
    public HashMap<String, String> getSerializeMap(String seatTag,Seat seat,Info info){

        HashMap<String, String> serializeMap = new HashMap<>();
        serializeMap.put(seatTag, gson.toJson(seat));

        serializeMap.put(INFO_TAG, gson.toJson(info));
        return serializeMap;
    }

    /**
     * 获取麦位变更记录信息
     * @return 麦位变更记录信息
     */
    public Info getInfo() {
        return info;
    }

//    public void setInfo(Info info) {
//        this.info = info;
//    }

    public static class Info{
//        /**
//         * 版本号
//         */
//        @Expose
//        int seq;
        /**
         * 修改者
         */
        @Expose
        String author;

//        public int getSeq() {
//            return seq;
//        }
//
//        public void setSeq(int seq) {
//            this.seq = seq;
//        }

        public String getAuthor() {
            return author;
        }

        public void setAuthor(String author) {
            this.author = author;
        }
    }

    public static Gson getGson() {
        return gson;
    }
}
