package im.zego.chathouse.model;

import static im.zego.chathouse.model.User.UserRole.AUDIENCE;

import com.google.gson.annotations.Expose;
import com.google.gson.annotations.SerializedName;

import im.zego.chathouse.protocol.APIBase;

/**
 * 用户信息
 */
public class User implements Comparable<User>{
    /**
     * 用户ID
     */
    @Expose
    @SerializedName("id")
    private String id;
    /**
     * 用户昵称
     */
    @Expose
    @SerializedName("name")
    private String name;
    /**
     * 用户头像
     */
    @Expose
    @SerializedName("icon")
    private String userIcon;
    /**
     * 用户麦位
     */
    @Expose
    @SerializedName("idx")
    private int idx = -1;
    @Expose
    @SerializedName("mic")
    private boolean micOn = true; //0是关，1是开
    /**
     * 用户是否允许开麦
     */
    @Expose
    @SerializedName("speak")
    private boolean canSpeak = true;
    /**
     * 用户身份
     */
    @Expose
    @SerializedName("role")
    private int role = AUDIENCE.role;
    @Expose (serialize = false, deserialize = false)
    private long loginTime = 0;
    @Expose (serialize = false, deserialize = false)
    private long inviteTime = 0;
    @Expose(serialize = false, deserialize = false)
    private int status = 0;

    public User(){
    }

    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getUserIcon() {
        return userIcon;
    }

    public void setUserIcon(String userIcon) {
        this.userIcon = userIcon;
    }

    public int getIdx() {
        return idx;
    }

    public void setIdx(int idx) {
        this.idx = idx;
    }

    public boolean getMicOn() {
        return micOn;
    }

    public void setMicOn(boolean micOn) {
        this.micOn = micOn;
    }

    public boolean isCanSpeak() {
        return canSpeak;
    }

    public void setCanSpeak(boolean canSpeak) {
        this.canSpeak = canSpeak;
    }

    public int getRole() {
        return role;
    }

    public void setRole(int role) {
        this.role = role;
    }



    public long getLoginTime() {
        return loginTime;
    }

    public void setLoginTime(long loginTime) {
        this.loginTime = loginTime;
    }

    public long getInviteTime() {
        return inviteTime;
    }

    public void setInviteTime(long inviteTime) {
        this.inviteTime = inviteTime;
    }

    public int getStatus() {
        return status;
    }

    public void setStatus(int status) {
        this.status = status;
    }


    public enum UserRole{
        HOST(0),
        GUEST(1),
        AUDIENCE(2);

        public int role;

        UserRole(int role){
            this.role = role;
        }

        public static UserRole getUserRole(int value) {
            try {
                if (HOST.role == value) {
                    return HOST;
                } else if (GUEST.role == value) {
                    return GUEST;
                }  else {
                    return AUDIENCE.role == value ? AUDIENCE : null;
                }
            } catch (Exception var2) {
                throw new RuntimeException("The enumeration cannot be found");
            }
        }
    }

    public enum Status{
        /**
         * 空闲状态
         */
        VACANT(0),
        /**
         * 邀请状态
         */
        INVITING(1),
        /**
         * 占据状态
         */
        OCCUPIED(2);

        public int status;

        Status(int status){
            this.status = status;
        }

        public static Status getStatus(int value) {
            try {
                if (Status.VACANT.status == value) {
                    return VACANT;
                } else if (Status.INVITING.status == value) {
                    return INVITING;
                } else {
                    return OCCUPIED.status == value ? OCCUPIED : null;
                }
            } catch (Exception var2) {
                throw new RuntimeException("The enumeration cannot be found");
            }
        }
    }


    /**
     * 排序顺序为，房主第一，麦上的第二，邀请中的三，其余的第四，相同类别根据时间顺序排序
     */
    @Override
    public int compareTo(User o) {

        if(this.role == UserRole.HOST.role){
            return 1;
        }else if(o.role == UserRole.HOST.role){
            return -1;
        }else {

            int status = this.status - o.status;
            if (status == 0) {
                if (this.status == Status.INVITING.status || this.status == Status.OCCUPIED.status) {
                    return (int) -((this.inviteTime - o.inviteTime) % 10000000);
                }

                return (int) -((this.loginTime - o.loginTime) % 10000000);
            } else {
                return status;
            }
        }
    }
}
