package im.zego.chathouse.protocol;



import static im.zego.chathouse.ErrorCode.ERROR_FAIL_NETWORK;
import static im.zego.chathouse.ErrorCode.ERROR_JSON_FORMAT_INVALID;

import android.os.Handler;
import android.os.Looper;
import android.util.Log;

import com.google.gson.Gson;

import org.jetbrains.annotations.NotNull;
import org.json.JSONObject;

import java.io.IOException;
import java.util.List;
import java.util.concurrent.TimeUnit;

import im.zego.chathouse.KeyCenter;
import im.zego.enjoycommon.log.L;

import okhttp3.Call;
import okhttp3.Callback;
import okhttp3.MediaType;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.RequestBody;
import okhttp3.Response;

/**
 * 网络请求服务封装类
 */
public class APIBase {
    private static final String TAG = "APIBase";

    private static final Gson mGson = new Gson();
    private static final Handler okHandler = new Handler(Looper.getMainLooper());

    public static JsonBuilder builder() {
        return new TokenJsonBuilder();
    }

    public static Gson getGson() {
        return mGson;
    }

    public static Handler getMainHandler(){
        return okHandler;
    }

    public static class OkHttpInstance{

        private volatile static OkHttpInstance instance;
        private final OkHttpClient mOkHttpClient;

        private OkHttpInstance() {
            OkHttpClient.Builder builder = new OkHttpClient.Builder()
                    .connectTimeout(15, TimeUnit.SECONDS)
                    .writeTimeout(20, TimeUnit.SECONDS)
                    .readTimeout(20, TimeUnit.SECONDS);
            mOkHttpClient = builder.build();
        }

        public static OkHttpClient getInstance()
        {
            if (instance == null) {
                synchronized (OkHttpInstance.class) {
                    if (instance == null) {
                        instance = new OkHttpInstance();
                    }
                }
            }
            return instance.mOkHttpClient;
        }

    }

    public static <T> void asyncPost(@NotNull String api, @NotNull final String reqJson, final Class<T> classType, final IAsyncPostCallback<T> reqCallback) {
        asyncPost(null,api,reqJson,classType,reqCallback);
    }
    public static <T> void asyncPost(Object tag,@NotNull String api, @NotNull final String reqJson, final Class<T> classType, final IAsyncPostCallback<T> reqCallback) {
        MediaType mediaType = MediaType.parse("application/json; charset=utf-8");
        RequestBody stringBody = RequestBody.Companion.create(reqJson, mediaType);

        String backendApiUrl = KeyCenter.BACKEND_API_URL;

        Request request = new Request.Builder()
                .tag(tag)
                .url(backendApiUrl + api)
                .post(stringBody)
                .build();
        Log.d(TAG, "request start : api: " + api);
        Log.d(TAG, "request start : reqJson: " + stringBody);
        OkHttpInstance.getInstance().newCall(request).enqueue(new Callback() {
            @Override
            public void onResponse(@NotNull Call call, @NotNull final Response response) {
                String str = "";
                try {
                    str = response.body().string();
                } catch (Exception e) {
                    e.printStackTrace();
                }

                final String finalStr = str;
                Log.d(TAG, "onResponse: api: " + api);
                Log.d(TAG, "onResponse: reqJson: " + reqJson);
                Log.d(TAG, "onResponse: respJson: " + finalStr);

                try {
                    JSONObject jsonObject = new JSONObject(finalStr);
                    final JSONObject retJsonObj = jsonObject.getJSONObject("ret");
                    final int retCode = retJsonObj.getInt("code");
                    final String retMsg = retJsonObj.getString("message");
                    String dataJson = "";
                    try {
                        JSONObject json = jsonObject.optJSONObject("data");
                        if(json != null) {
                            dataJson = json.toString();
                        }
                    }catch (Exception e)
                    {
//                        e.printStackTrace();
                    }

                    if (reqCallback != null) {
                        final T bean = mGson.fromJson(dataJson, classType);

                            okHandler.post(new Runnable() {
                                @Override
                                public void run() {
                                    reqCallback.onResponse(retCode, retMsg, bean);
                                }
                            });

                    }
                } catch (Exception jsonException) {
                    jsonException.printStackTrace();
                    if (reqCallback != null) {
                        okHandler.post(new Runnable() {
                            @Override
                            public void run() {
                                reqCallback.onResponse(ERROR_JSON_FORMAT_INVALID, "", null);
                            }
                        });
                    }
                }


            }

            @Override
            public void onFailure(@NotNull Call call, @NotNull IOException e) {
                Log.d(TAG, "onFailure: " + e.getMessage());

                if (reqCallback != null) {
                    okHandler.post(new Runnable() {
                        @Override
                        public void run() {
                            reqCallback.onResponse(ERROR_FAIL_NETWORK, "", null);
                        }
                    });
                }

            }
        });

    }

    /**
     * 取消请求
     * @param tag 标志
     */
    public static void cancelRequest(Object tag){

        if(tag == null){
            L.e(TAG,"cancelRequest tag is NULL ");
            return;
        }

        List<Call> callList = OkHttpInstance.getInstance().dispatcher().runningCalls();
        for(Call call : callList){
            if(call.request().tag() != null){
                if(call.request().tag()  == tag){
                    L.e(TAG,"cancel request:" + call.request().url());
                    call.cancel();
                }
            }
        }
    }
}
