package im.zego.chathouse.protocol;

import android.os.SystemClock;
import android.text.TextUtils;

import org.jetbrains.annotations.NotNull;

import im.zego.chathouse.KeyCenter;
import im.zego.chathouse.manager.UserManager;
import im.zego.chathouse.model.TokenInfo;
import im.zego.enjoycommon.log.L;

/**
 * 鉴权管理和请求封装
 */
public class TokenManager {

    private static final String TAG = "TokenManager";

    private String mToken;

    /**
     * token刷新阀值，单位毫秒
     */
    private static final long tokenThresholdValue = 300 * 1000;
    private static volatile TokenManager singleton = null;

    private long tokenTimeout;

    private TokenManager() {
    }

    public static TokenManager getInstance() {
        if (singleton == null) {
            synchronized (TokenManager.class) {
                if (singleton == null) {
                    singleton = new TokenManager();
                }
            }
        }
        return singleton;
    }

    /**
     * 返回本地记录的token
     * @return token
     */
    public String getToken() {

        /*
         * 超时时间不为零 且超时时间小于当前时间+token阀值时间，则更新token
         */
        if(tokenTimeout != 0 && tokenTimeout <= (tokenThresholdValue + SystemClock.elapsedRealtime()) ){
            getToken(null);
        }

        return mToken;
    }

    private void setToken(String token) {

        if(TextUtils.isEmpty(token)){
            L.e(TAG,"setToken value is NULL");
            return;
        }
        this.mToken = token;
    }

    /**
     * 从服务器获取token，并记录
     * @param callback 请求回调
     */
    public void getToken(final ICommonResponseCallback callback){
        String reqJson = new JsonBuilder()
                .addParams("app_id", KeyCenter.APP_ID)
                .addParams("app_sign",KeyCenter.APP_SIGN.substring(0,32))
                .addParams("uid", UserManager.getInstance().getUserID())
                .build();

        APIBase.asyncPost("/chat_house/v1/get_token_04", reqJson, TokenInfo.class, new IAsyncPostCallback<TokenInfo>() {
            @Override
            public void onResponse(int code, @NotNull String message, TokenInfo responseJsonBean) {

                if(code == 0){
                    setToken(responseJsonBean.getToken());
                    tokenTimeout = responseJsonBean.getTokenDuration() * 1000 + SystemClock.elapsedRealtime();
                }

                if (callback != null) {
                    callback.onResponse(code);
                }
            }
        });
    }
}
