package im.zego.chathouse.room;

import static im.zego.chathouse.ErrorCode.ERROR_FAIL_NETWORK;

import android.content.DialogInterface;
import android.content.Intent;
import android.os.Build;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.View;
import android.view.WindowManager;
import android.widget.FrameLayout;

import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;

import java.util.ArrayList;

import im.zego.chathouse.R;
import im.zego.chathouse.UserProfile;
import im.zego.chathouse.manager.UserManager;

import im.zego.chathouse.model.RoomInfo;
import im.zego.chathouse.protocol.ICommonResponseCallback;
import im.zego.chathouse.room.custom.RoomBottomMenu;
import im.zego.chathouse.room.custom.RoomMicListView;
import im.zego.chathouse.room.custom.RoomTopInfoView;
import im.zego.chathouse.sdk.rtc.RTCSDKManager;
import im.zego.chathouse.sdk.rtc.callbacks.IRTCCommonCallback;
import im.zego.chathouse.sdk.zim.ZIMSDKManager;
import im.zego.chathouse.sdk.zim.callback.IZIMQueryRoomMemberBaseInfoListCallback;
import im.zego.chathouse.utils.ToastUtil;
import im.zego.enjoycommon.util.MiscUtils;
import im.zego.enjoycommon.util.ScreenAdaptationUtil;
import im.zego.zim.entity.ZIMUserInfo;

public class RoomActivity extends AppCompatActivity {

    public static final String ROOM_ID_INTENT_TAG = "room_id";

    private String mRoomID;

    private FrameLayout mFlRoomTopInfo;
    private FrameLayout mFlRoomMicList;
    private FrameLayout mFlRoomBottomMenu;


    private RoomPresenter mRoomPresenter;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        ScreenAdaptationUtil.init(this.getApplicationContext(), 375);
        ScreenAdaptationUtil.setCustomDensity(this);
        getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        setContentView(R.layout.chathouse_activity_room);
        initWindowBar();
        initData();
        initFrameworkView();
        mRoomPresenter.initRoom();
        startRoomService();
    }


    private void initWindowBar() {
        setViewPaddingTop(findViewById(R.id.cl_room), MiscUtils.getStatusBarHeight(this));
        WindowManager.LayoutParams lp = getWindow().getAttributes();
        lp.flags = lp.flags | WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS;
        getWindow().setAttributes(lp);
    }

    private void initData(){

        mRoomPresenter = new RoomPresenter(this);
        mRoomID = getIntent().getStringExtra(ROOM_ID_INTENT_TAG);

        if(TextUtils.isEmpty(UserManager.getInstance().getUserID())) {
            UserProfile.reset();
            String uid = UserProfile.USER_ID;
            String name = UserProfile.USER_NAME;
            String userIcon = UserProfile.USER_ICON;
            //保存用户信息
            UserManager.getInstance().setUserInfo(uid, name, userIcon);
        }

    }

    /**
     * 初始化框架视图
     */
    private void initFrameworkView(){
        mFlRoomTopInfo = findViewById(R.id.fl_room_top_info);
        mFlRoomMicList = findViewById(R.id.fl_room_mic_list);
        mFlRoomBottomMenu = findViewById(R.id.fl_room_bottom_menu);
    }

    /**
     * 初始化语聊房视图
     */
    public void initChatHouseView(){
        mFlRoomTopInfo.removeAllViews();
        mFlRoomMicList.removeAllViews();
        mFlRoomBottomMenu.removeAllViews();

        RoomTopInfoView roomTopInfoView = new RoomTopInfoView(RoomActivity.this);
        mFlRoomTopInfo.addView(roomTopInfoView);

        RoomManager.getInstance().applyRoomInfoServices(
                services -> {
                    RoomInfo roomInfo = services.getRoomInfo();
                    if(roomInfo != null) {
                        roomTopInfoView.update(roomInfo.getSubject(),roomInfo.getCoverImg());
                    }
                }
        );

        roomTopInfoView.setCloseListener(v -> onBackPressed());



    }

    /**
     * 房间属性获取或设置成功后初始化依赖房间属性的视图
     */
    public void initRoomAllAttributesSuccess(){
        if (!RoomActivity.this.isFinishing() && !RoomActivity.this.isDestroyed()) {

            //查询一遍房间内所有用户的基础信息，并设置用户基本信息
            ZIMSDKManager.getInstance().queryRoomMemberListForBaseInfo(mRoomID, new IZIMQueryRoomMemberBaseInfoListCallback() {
                @Override
                public void onRoomMemberQueried(ArrayList<ZIMUserInfo> userList, String nextFlag, int errorCode) {

                    if(errorCode == 0) {
                        RoomManager.getInstance().applyUserService(services -> {

                            for(ZIMUserInfo zimUserFullInfo : userList){
                                services.setUserBaseInfo(zimUserFullInfo.userID,zimUserFullInfo.userName);
                            }


                        });
                    }


                    RoomMicListView roomMicListView = new RoomMicListView(RoomActivity.this,mRoomID);
                    mFlRoomMicList.addView(roomMicListView);
                    roomMicListView.updateMicList();

                    RoomBottomMenu roomBottomMenu = new RoomBottomMenu(RoomActivity.this);
                    mFlRoomBottomMenu.addView(roomBottomMenu);


                }
            });



        }
    }


    /**
     * 重设 view 的宽高
     */
    private void setViewPaddingTop(View view, int paddingTop) {
        view.setPadding(view.getPaddingLeft(),paddingTop,view.getPaddingRight(),view.getPaddingBottom());
    }

    @Override
    protected void onStop() {
        super.onStop();
        mRoomPresenter.onStop();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        stopRoomService();
    }

    @Override
    public void onBackPressed() {

        if(RoomManager.getInstance().getUserService() == null){
            finish();
            return;
        }

        RoomManager.getInstance().applyRoomDialogService(services -> {
            services.showCloseRoomDialog(RoomActivity.this, null, new DialogInterface.OnClickListener() {
                @Override
                public void onClick(DialogInterface dialog, int which) {

                    if(RoomManager.getInstance().isHost()) {
                        RoomManager.getInstance().closeRoom(new ICommonResponseCallback() {
                            @Override
                            public void onResponse(int errorCode) {
                                if (errorCode != ERROR_FAIL_NETWORK) {
                                    if (dialog != null) {
                                        dialog.dismiss();
                                    }

                                    RTCSDKManager.getInstance().stopMixStream(new IRTCCommonCallback() {
                                        @Override
                                        public void onRTCCallback(int errorCode) {
                                            finish();
                                        }
                                    });



                                } else {
                                    ToastUtil.showWarnToast(R.string.fail_network);
                                }
                            }
                        });
                    }else{
                        RoomManager.getInstance().quitRoom(new ICommonResponseCallback() {
                            @Override
                            public void onResponse(int errorCode) {
                                finish();
                                if (dialog != null) {
                                    dialog.dismiss();
                                }
                            }
                        });

                    }
                }
            });
        });




    }

    /**
     * 获取房间ID
     * @return
     */
    public String getRoomID() {
        return mRoomID;
    }

    /**
     * 开启前台服务
     */
    private void startRoomService() {
        Intent intent = new Intent(this, RoomService.class);
        if (Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.O) {
            startForegroundService(intent);
        } else {
            startService(intent);
        }
    }

    /**
     * 停止前台服务
     */
    private void stopRoomService() {
        Intent intent = new Intent(this, RoomService.class);
        stopService(intent);
    }

}
