package im.zego.chathouse.room;

import static im.zego.chathouse.Constants.MESSAGE_CMD_CALL_INVITE_EXTENDED_DATA;
import static im.zego.chathouse.ErrorCode.ERROR_USER_OFF_LINE;
import static im.zego.chathouse.model.RoomAttributes.SEAT_LIST;
import static im.zego.chathouse.sdk.zim.ZIMEventHandlerWrap.UI_PRIORITY;

import android.content.DialogInterface;
import android.text.TextUtils;

import org.json.JSONObject;

import java.lang.ref.WeakReference;
import java.util.ArrayList;

import im.zego.chathouse.R;
import im.zego.chathouse.manager.UserManager;
import im.zego.chathouse.model.DialogTask;
import im.zego.chathouse.model.InviteExtendedData;
import im.zego.chathouse.model.Seat;
import im.zego.chathouse.protocol.APIBase;
import im.zego.chathouse.protocol.ICommonResponseCallback;
import im.zego.chathouse.protocol.RequestErrorCodeManager;
import im.zego.chathouse.protocol.RoomAPI;
import im.zego.chathouse.room.callback.IActivityNotNullCallback;
import im.zego.chathouse.room.callback.IRoomInitStateCallback;
import im.zego.chathouse.room.help.TimerTaskHelp;
import im.zego.chathouse.room.service.DialogQueueService;
import im.zego.chathouse.room.service.RoomDialogService;
import im.zego.chathouse.sdk.constants.ConnectionState;
import im.zego.chathouse.sdk.rtc.RTCSDKManager;
import im.zego.chathouse.sdk.rtc.callbacks.IConnectionStateChangeListener;
import im.zego.chathouse.sdk.zim.ZIMEventHandlerWrap;
import im.zego.chathouse.sdk.zim.ZIMSDKManager;
import im.zego.chathouse.sdk.zim.callback.IZIMCommonCallback;
import im.zego.chathouse.sdk.zim.callback.IZIMQueryRoomMemberIDListCallback;
import im.zego.enjoycommon.util.T;
import im.zego.zim.ZIM;
import im.zego.zim.entity.ZIMCallInvitationReceivedInfo;
import im.zego.zim.entity.ZIMUserInfo;
import im.zego.zim.enums.ZIMRoomEvent;
import im.zego.zim.enums.ZIMRoomState;

public class RoomPresenter {


    WeakReference<RoomActivity> mWeakReference;

    public RoomPresenter(RoomActivity activity) {
        mWeakReference = new WeakReference<>(activity);
    }

    private ZIMEventHandlerWrap mZimEventHandlerWrap;

    /**
     * 初始化房间
     */
    public void initRoom() {
        RoomManager.getInstance().setRoomInitStateCallback(new IRoomInitStateCallback() {
            @Override
            public void onRoomInitStateCallback(RoomManager.RoomInitState roomInitState) {
                RoomActivity activity = mWeakReference.get();
                switch (roomInitState) {

                    case UNINITIALIZED:
                        if (activity != null && !activity.isDestroyed() && !activity.isFinishing()) {
                            rejoinRoom();
                        }
                        break;
                    case INITIALIZING:
                        break;
                    case INIT_COMPLETED:
                        activity.initChatHouseView();
                        initRoomAllAttributes();
                        break;
                    case INIT_ERROR:

                        break;
                }
            }
        });
        applyActivity(activity -> {
            mZimEventHandlerWrap = new ZIMEventHandlerWrap(activity.getRoomID(), UI_PRIORITY) {

                @Override
                public void onCallInvitationReceived(ZIM zim, ZIMCallInvitationReceivedInfo info, String callID) {
                    super.onCallInvitationReceived(zim, info, callID);

                    try {
                        JSONObject jsonObject = new JSONObject(info.extendedData);
                        final int cmd = jsonObject.optInt("cmd");
                        final String roomID = jsonObject.optString("room_id");

                        String dataJson = "";
                        // 当data为null时会抛出异常
                        try {
                            dataJson = jsonObject.getJSONObject("data").toString();
                        } catch (Exception ignored) {

                        }

                        if(cmd == MESSAGE_CMD_CALL_INVITE_EXTENDED_DATA && RoomManager.getInstance().getCurrentRoomID().equals(roomID)) {

                            InviteExtendedData data = APIBase.getGson().fromJson(dataJson, InviteExtendedData.class);
                            if (data != null) {
                                data.callID = callID;
                                applyActivityAndActivate(activity1 -> {
                                    if (!RoomManager.getInstance().isHost()) {
                                        RoomManager.getInstance().applyRoomDialogService(services -> services.showRoomInviteDialog(activity1, data, null, null));

                                    }

                                });

                            }
                        }

                    } catch (Exception ignored) {

                    }
                }


                @Override
                public void onCallInvitationTimeout(ZIM zim, String callID) {
                    super.onCallInvitationTimeout(zim, callID);
                    if (!RoomManager.getInstance().isHost()) {
                        RoomManager.getInstance().applyRoomDialogService(RoomDialogService::dismissRoomInviteDialog);

                    }
                }

                @Override
                public void onRoomStateChanged(ZIM zim, ZIMRoomState state, ZIMRoomEvent event, JSONObject extendedData, String roomID) {
                    super.onRoomStateChanged(zim, state, event, extendedData, roomID);
                    if (state == ZIMRoomState.DISCONNECTED && event.value() == ZIMRoomEvent.ROOM_NOT_EXIST.value()) {
                        applyActivityAndActivate(activity1 -> {

                            //非房主弹出房间关闭的弹窗
                            if (!RoomManager.getInstance().isHost()) {

                                sdkRoomLogout();
                                RoomManager.getInstance().applyRoomDialogService(services -> {
                                    services.showRoomAlreadyClosedDialog(activity1, new DialogInterface.OnClickListener() {
                                        @Override
                                        public void onClick(DialogInterface dialog, int which) {
                                            if (dialog != null) {
                                                dialog.dismiss();
                                            }
                                            activity1.finish();
                                        }
                                    });
                                });

                            }


                        });
                    }else if(state == ZIMRoomState.CONNECTED && event.value() == ZIMRoomEvent.SUCCESS.value()  && RoomManager.getInstance().getRoomInitState() == RoomManager.RoomInitState.INIT_COMPLETED){
                        applyActivityAndActivate(activity1 -> {
                            //断网重连，获取房间用户列表更新麦位
                            ZIMSDKManager.getInstance().queryRoomMemberListInRoom(activity1.getRoomID(), new IZIMQueryRoomMemberIDListCallback() {
                                        @Override
                                        public void onRoomMemberQueried(ArrayList<String> userList, String nextFlag, int errorCode) {

                                            if(userList == null ){
                                                return;
                                            }
                                            RoomManager.getInstance().applyRoomAttributesService(services1 ->
                                            {

                                                for(int i = 0; i < SEAT_LIST.length;i++){
                                                    Seat seat = services1.getRoomAttributes().getSeat(i);
                                                    if(seat.getUser() != null && !TextUtils.isEmpty(seat.getUser().getId())){

                                                        if(!userList.contains(seat.getUser().getId())){

                                                            //断网恢复，模拟系统补发退房信息
                                                            ArrayList<ZIMUserInfo> userInfoList = new ArrayList<>();
                                                            ZIMUserInfo zimUserInfo = new ZIMUserInfo();
                                                            zimUserInfo.userID = seat.getUser().getId();
                                                            zimUserInfo.userName = seat.getUser().getName();
                                                            userInfoList.add(zimUserInfo);
                                                            ZIMSDKManager.getInstance().getDispatchService().onRoomMemberLeft(ZIMSDKManager.getInstance().zim,userInfoList,activity1.getRoomID());

                                                        }
                                                    }
                                                }


                                            });

                                        }
                                    });
                        });
                    }
                }
            };

            ZIMSDKManager.getInstance().getDispatchService().registerZIMEventHandlerCallbacks(mZimEventHandlerWrap);
        });

        RoomManager.getInstance().applyDialogQueueService(services -> {
            services.setDialogListener(new DialogQueueService.DialogListener() {
                @Override
                public void onData(DialogTask dialogTask) {
                    applyActivityAndActivate(activity -> RoomManager.getInstance().applyRoomDialogService(services1 -> services1.showRoomApplyDialog(activity, dialogTask, services)));
                }

                @Override
                public void remove(String callID) {
                    applyActivity(activity -> RoomManager.getInstance().applyRoomDialogService(services1 -> services1.removeRoomApplyDialog(callID)));
                }
            });
        });


        RoomManager.getInstance().applyRoomConnectService(services -> {
            services.addConnectionStateChange(new IConnectionStateChangeListener() {
                @Override
                public void onConnectionState(ConnectionState state) {
                    if (state == ConnectionState.CONNECTING) {
                        applyActivityAndActivate(activity -> {
                            RoomManager.getInstance().applyRoomDialogService(services1 -> {

                                services1.showRoomNetConnectingDialog(activity, (dialog, which) -> activity.finish());


                            });
                        });
                    } else if (state == ConnectionState.CONNECTED) {
                        applyActivityAndActivate(activity -> {
                            RoomAPI.heartBeat(activity.getRoomID(),null);

                        });
                        RoomManager.getInstance().applyRoomDialogService(RoomDialogService::dismissRoomNetConnectingDialog);



                    } else if (state == ConnectionState.DISCONNECTED) {
                        showDisconnectedDialog();
                        sdkRoomLogout();
                    }
                }
            });
        });

        RequestErrorCodeManager.getInstance().setRequestErrorCodeListener(new RequestErrorCodeManager.RequestErrorCodeListener() {
            @Override
            public void onRequestErrorCode(Object tag, String requestUrl, int errorCode) {
                if (errorCode == ERROR_USER_OFF_LINE) {
                    showDisconnectedDialog();

                    sdkRoomLogout();
                }
            }
        });

    }

    /**
     * 展示房间断开网络弹窗
     */
    public void showDisconnectedDialog() {
        RoomManager.getInstance().applyRoomDialogService(RoomDialogService::dismissRoomNetConnectingDialog);
        applyActivityAndActivate(activity -> {
            RoomManager.getInstance().applyRoomDialogService(services1 -> services1.showRoomNetDisConnectedDialog(activity, (dialog, which) -> activity.finish()));
        });
    }

    /**
     * 重新加入房间
     */
    public void rejoinRoom() {
        applyActivityAndActivate(
                activity -> {
                    if (!TextUtils.isEmpty(activity.getRoomID())) {
                        RoomManager.getInstance().joinRoom(activity.getRoomID(), UserManager.getInstance().getUserName(),
                                UserManager.getInstance().getUserIcon(), activity.getApplication(), new ICommonResponseCallback() {
                                    @Override
                                    public void onResponse(int errorCode) {
                                        if (errorCode == 0) {
                                            T.show(R.string.chathouse_succeeded_in_joining_the_room);
                                        }
                                    }
                                });
                    } else {
                        T.show(R.string.chathouse_room_info_error);
                    }
                }
        );
    }

    /**
     * 初始化房间属性
     */
    public void initRoomAllAttributes() {

        RTCSDKManager.getInstance().standardQualityAudioConfig();

        RoomManager.getInstance().applyRoomInfoServices(
                services -> {
                    //如果是房主，则同步房间属性
                    if (UserManager.getInstance().getUserID().equals(services.getRoomInfo().getCreatorId())) {
                        RoomManager.getInstance().applyRoomAttributesService(
                                services1 -> {
                                    services1.firstQueryRoomAllAttributes(services.getRoomID(), new IZIMCommonCallback() {
                                        @Override
                                        public void onZIMCallback(int errorCode) {

                                            services1.hostInit();

                                            applyActivityAndActivate(
                                                    activity -> {
                                                        activity.initRoomAllAttributesSuccess();
                                                        updateUserUserIcon();
                                                    }
                                            );
                                        }
                                    });
                                }
                        );
                    } else {
                        //非房主则获取房间属性
                        RoomManager.getInstance().applyRoomAttributesService(
                                services1 -> {
                                    services1.queryRoomAllAttributes(services.getRoomID(), new IZIMCommonCallback() {
                                        @Override
                                        public void onZIMCallback(int errorCode) {
                                            if (errorCode == 0) {
                                                applyActivityAndActivate(
                                                        activity -> {
                                                            activity.initRoomAllAttributesSuccess();
                                                            updateUserUserIcon();
                                                        }
                                                );
                                            }else{
                                                T.show(R.string.chathouse_net_connect_is_error_please_leave_the_room_and_try_again);
                                            }
                                        }
                                    });
                                }
                        );

                    }
                }
        );

    }

    /**
     * 更新用户附加信息（头像）
     */
    public void updateUserUserIcon() {
//        UserExtendedData userExtendedData = new UserExtendedData();
//        userExtendedData.setUserIcon(UserManager.getInstance().getUserIcon());
//        String data = APIBase.getGson().toJson(userExtendedData);
//        ZIMSDKManager.getInstance().updateUserExtendedData(data, new IZIMCommonCallback() {
//            @Override
//            public void onZIMCallback(int errorCode) {
//                if(errorCode != 0){
//                    T.show(R.string.chathouse_set_user_icon_fail);
//                }
//            }
//        });
        ZIMSDKManager.getInstance().updateUserAvatarUrl(UserManager.getInstance().getUserIcon(), new IZIMCommonCallback() {
            @Override
            public void onZIMCallback(int errorCode) {
                if(errorCode != 0){
                    T.show(R.string.chathouse_set_user_icon_fail);
                }
            }
        });
    }

    /**
     * 页面onStop阶段调用
     */
    public void onStop() {
        applyActivity(
                activity -> {
                    if (activity.isFinishing()) {
//                        RoomManager.getInstance().quitRoom(null);
                        RoomManager.getInstance().setRoomInitStateCallback(null);
                        RoomManager.getInstance().logoutRoom();
                        RoomManager.getInstance().unInit();

                        ZIMSDKManager.getInstance().getDispatchService().unregisterZIMEventHandlerCallbacks(mZimEventHandlerWrap);

                        RoomManager.getInstance().applyDialogQueueService(services -> services.setDialogListener(null));
                    }
                }
        );

    }

    /**
     * 获取页面信息，获取成功则回调
     * @param callback
     */
    private void applyActivity(IActivityNotNullCallback callback) {
        RoomActivity activity = mWeakReference.get();
        if (activity != null && callback != null) {
            callback.block(activity);
        }
    }

    /**
     * 获取页面信息，且页面未结束，则回调
     * @param callback
     */
    private void applyActivityAndActivate(IActivityNotNullCallback callback) {
        RoomActivity activity = mWeakReference.get();
        if (activity != null && callback != null && !activity.isFinishing() && !activity.isDestroyed()) {
            callback.block(activity);
        }
    }

    /**
     * 退出SDK房间
     */
    private void sdkRoomLogout(){
        applyActivityAndActivate(new IActivityNotNullCallback() {
            @Override
            public void block(RoomActivity activity) {
                ZIMSDKManager.getInstance().logoutRoom(activity.getRoomID(),null);
            }
        });
        RTCSDKManager.getInstance().logoutRoom();
        TimerTaskHelp.stopQueryOnlineMemberCount();
    }


}
