package im.zego.chathouse.room.custom;

import android.app.Activity;
import android.app.Dialog;
import android.content.Context;
import android.graphics.Color;
import android.os.CountDownTimer;
import android.text.Spannable;
import android.text.SpannableString;
import android.text.style.ForegroundColorSpan;
import android.view.View;
import android.view.Window;
import android.widget.TextView;

import androidx.annotation.NonNull;

import im.zego.chathouse.R;
import im.zego.enjoycommon.util.MiscUtils;


/**
 * 观众申请连麦弹窗
 */
public class RoomApplyDialog extends Dialog {

    private TextView mTitle;
    private TextView mContent;
    private TextView mCancelButton;
    private TextView mAcceptButton;
    private String callID;

    private final boolean mCancelable;

    public RoomApplyDialog(@NonNull Context context, boolean cancelable) {
        this(context, R.layout.chathouse_dialog_apply,cancelable);
    }

    public RoomApplyDialog(@NonNull Context context, int layoutId, boolean cancelable) {
        this(context, R.style.ChatHouseBaseDialogStyle, layoutId,cancelable);
    }

    private RoomApplyDialog(@NonNull Context context, int themeResId, int layoutId, boolean cancelable) {
        super(context, themeResId);
        mCancelable = cancelable;
        init(context, layoutId);
    }

    private void init(Context context, int layoutId) {
        //设置dialog位置
        Window window = getWindow();
        if (window != null) {
            window.setContentView(layoutId);
            mTitle = window.findViewById(R.id.tv_title);
            mContent = window.findViewById(R.id.tv_content);
            mCancelButton = window.findViewById(R.id.tv_cancel);
            mAcceptButton = window.findViewById(R.id.tv_accept);
        }

        setCancelable(mCancelable);
        setCanceledOnTouchOutside(mCancelable);

        if(context instanceof Activity){
           if(((Activity)context).isDestroyed() || ((Activity)context).isFinishing()) {
               return;
           }
        }
        show();



    }

    /**
     * 倒计时，关闭弹窗
     * @param time 倒计时时间
     */
    public void countDown(long time){
        CountDownTimer countDownTimer = new CountDownTimer(time,1000) {
            @Override
            public void onTick(long millisUntilFinished) {
                String str = MiscUtils.getApplication().getString(R.string.chathouse_apply_seat_count_down);
                SpannableString spannableString = new SpannableString(str);
                spannableString.setSpan(new ForegroundColorSpan(Color.parseColor("#6D24F6")), 4, str.length(), Spannable.SPAN_INCLUSIVE_EXCLUSIVE);
                setMsgTitle(spannableString);
            }

            @Override
            public void onFinish() {
                dismiss();
            }
        };
        countDownTimer.start();
    }

    public void setMsgTitle(CharSequence title) {

        if(mTitle == null){
            return;
        }

        mTitle.setText(title);
    }

    public void setMsgContent(String userName) {

        if(mContent == null){
            return;
        }

        String str = String.format(MiscUtils.getApplication().getString(R.string.chathouse_apply_seat_with_you), userName);
        SpannableString spannableString = new SpannableString(str);
        spannableString.setSpan(new ForegroundColorSpan(Color.parseColor("#6D24F6")), 0, userName.length(), Spannable.SPAN_INCLUSIVE_EXCLUSIVE);

        mContent.setText(spannableString);
    }

    public void setAcceptListener(View.OnClickListener listener) {

        if(mAcceptButton == null){
            return;
        }

        mAcceptButton.setOnClickListener(listener);
    }

    public void setCancelListener(View.OnClickListener listener) {

        if(mCancelButton == null){
            return;
        }

        mCancelButton.setOnClickListener(listener);
    }

    public String getCallID() {
        return callID;
    }

    public void setCallID(String callID) {
        this.callID = callID;
    }
}
