package im.zego.chathouse.room.custom;

import static im.zego.chathouse.sdk.zim.ZIMEventHandlerWrap.UI_PRIORITY;

import android.content.Context;
import android.content.DialogInterface;
import android.util.AttributeSet;
import android.view.View;
import android.widget.FrameLayout;
import android.widget.ImageView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.constraintlayout.widget.ConstraintLayout;

import im.zego.chathouse.R;
import im.zego.chathouse.manager.UserManager;
import im.zego.chathouse.model.Seat;
import im.zego.chathouse.model.User;
import im.zego.chathouse.room.RoomManager;
import im.zego.chathouse.room.help.ZIMMessageHelp;
import im.zego.chathouse.sdk.rtc.RTCSDKManager;
import im.zego.chathouse.sdk.zim.ZIMEventHandlerWrap;
import im.zego.chathouse.sdk.zim.ZIMSDKManager;
import im.zego.chathouse.sdk.zim.callback.IZIMCommonCallback;
import im.zego.chathouse.utils.ViewUtil;
import im.zego.enjoycommon.log.L;
import im.zego.enjoycommon.util.MiscUtils;
import im.zego.enjoycommon.util.T;
import im.zego.zim.ZIM;
import im.zego.zim.entity.ZIMRoomAttributesUpdateInfo;

/**
 * 房间底部菜单视图
 */
public class RoomBottomMenu extends FrameLayout {

    private ImageView mIvMic;
    private ConstraintLayout mClConnectionMic;
    private ConstraintLayout mClMic;

    private ZIMEventHandlerWrap mZimEventHandlerWrap;

    public RoomBottomMenu(@NonNull Context context) {
        this(context,null);
    }

    public RoomBottomMenu(@NonNull Context context, @Nullable AttributeSet attrs) {
        this(context, attrs,0);
    }

    public RoomBottomMenu(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        View.inflate(context, R.layout.chathouse_view_bottom_menu,this);
        initView();
    }

    private void initView(){
        mIvMic = findViewById(R.id.iv_mic);
        mClMic = findViewById(R.id.cl_mic);
        mClConnectionMic = findViewById(R.id.cl_connection_mic);

        RoomManager.getInstance().applyRoomInfoServices(services -> {
            mZimEventHandlerWrap = new ZIMEventHandlerWrap(services.getRoomID(),UI_PRIORITY) {
                @Override
                public void onRoomAttributesUpdated(ZIM zim, ZIMRoomAttributesUpdateInfo info, String roomID) {
                    super.onRoomAttributesUpdated(zim, info, roomID);
                    updateView();
                }
            };
        });

        mClConnectionMic.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {

                RoomManager.getInstance().applyRoomDialogService(services -> {
                    services.showRoomEndMicDialog(ViewUtil.getActivity(mClConnectionMic), null, new DialogInterface.OnClickListener() {
                        @Override
                        public void onClick(DialogInterface dialog, int which) {
                                        //观众修改属性进行下麦
                                        RoomManager.getInstance().applyRoomAttributesService(services1 -> {
                                            Seat seat = services1.getSeatByUser(UserManager.getInstance().getUserID());
                                            if(seat != null){
                                                services1.stageGuestDownSeat(seat.getIdx(), new IZIMCommonCallback() {
                                                    @Override
                                                    public void onZIMCallback(int errorCode) {
                                                        if(errorCode == 0){
                                                            RoomManager.getInstance().applyInviteService(
                                                                    services -> services.removeInvite(seat.getIdx(),UserManager.getInstance().getUserID()));
                                                        }else{
                                                            T.show(String.format(MiscUtils.getApplication().getString(R.string.chathouse_down_seat_request_fail_error_code), errorCode));
                                                        }
                                                    }
                                                });

                                            }
                                        });

                        }
                    });
                });

            }
        });

        mClMic.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                RoomManager.getInstance().applyRoomAttributesService(services1 -> {

                    Seat seat = services1.getSeatByUser(UserManager.getInstance().getUserID());
                    if(seat != null && seat.getUser() != null && seat.getStatus() == User.Status.OCCUPIED.status){

                        if(seat.getUser().isCanSpeak()){
                            RoomManager.getInstance().applyUserService(services -> {

                                User user = services.getUser(UserManager.getInstance().getUserID());
                                if(user != null){
                                    if(user.getMicOn()){
                                        RTCSDKManager.getInstance().enableMic(false);
                                        services.setUserMic(user.getId(),false);
                                        RoomManager.getInstance().applyRoomAttributesService(services2 -> {
                                            services2.setMicOnOff(user.getId(),false);
                                        });
                                        mIvMic.setImageResource(R.mipmap.chathouse_mic_off);
                                    }else{
                                        RTCSDKManager.getInstance().enableMic(true);
                                        services.setUserMic(user.getId(),true);
                                        RoomManager.getInstance().applyRoomAttributesService(services2 -> {
                                            services2.setMicOnOff(user.getId(),true);
                                        });
                                        mIvMic.setImageResource(R.mipmap.chathouse_mic_on);
                                    }
                                }

                            });
                        }else{
                            T.show(getContext().getString(R.string.chathouse_you_silenced_by_the_anchor));
                        }
                    }


                });
            }
        });

//        //通过房间属性更新开关麦
//        RoomManager.getInstance().applyRoomAttributesService(services -> {
//
//
//            Seat seat = services.getSeatByUser(UserManager.getInstance().getUserID());
//
//            if(seat != null && seat.getStatus() == Seat.SeatStatus.OCCUPIED.status &&
//                    seat.getUser() != null){
//
//                if(seat.getUser().getMicOn()){
//                    mIvMic.setImageResource(R.mipmap.chathouse_mic_on);
//                    RTCSDKManager.getInstance().enableMic(true);
//                }else{
//                    mIvMic.setImageResource(R.mipmap.chathouse_mic_off);
//                    RTCSDKManager.getInstance().enableMic(false);
//                }
//
//            }
//
//        });

    }


    /**
     * 根据房间属性，更新UI
     */
    private void updateView(){
        RoomManager.getInstance().applyRoomAttributesService(services1 -> {

            Seat seat = services1.getSeatByUser(UserManager.getInstance().getUserID());
            //房主只展示麦克风
            if(RoomManager.getInstance().isHost()){
                mClMic.setVisibility(VISIBLE);
                mClConnectionMic.setVisibility(GONE);
            }else{
                //非房主，判断是否是在麦上
                if(seat != null && seat.getUser() != null && seat.getStatus() == User.Status.OCCUPIED.status){
                    mClMic.setVisibility(VISIBLE);
                    mClConnectionMic.setVisibility(VISIBLE);
                    //是否被禁言，禁言则显示关闭麦克风
                    if(!seat.getUser().isCanSpeak()){
                        RTCSDKManager.getInstance().enableMic(false);
                        mIvMic.setImageResource(R.mipmap.chathouse_mic_off);
                    }else{

                        RoomManager.getInstance().applyUserService(services -> {
                            //根据用户信息，展示是否开关麦克风
                            User user = services.getUser(UserManager.getInstance().getUserID());
                            if(user != null){
                                if(user.getMicOn()){
                                    RTCSDKManager.getInstance().enableMic(true);
                                    mIvMic.setImageResource(R.mipmap.chathouse_mic_on);
                                }else{
                                    RTCSDKManager.getInstance().enableMic(false);
                                    mIvMic.setImageResource(R.mipmap.chathouse_mic_off);

                                }
                            }

                        });
                    }
                }else {
                    //麦下用户，则都隐藏
                    mClMic.setVisibility(GONE);
                    mClConnectionMic.setVisibility(GONE);
                }
            }

        });

    }

    @Override
    protected void onAttachedToWindow() {
        super.onAttachedToWindow();
        updateView();
        ZIMSDKManager.getInstance().getDispatchService().registerZIMEventHandlerCallbacks(mZimEventHandlerWrap);
    }

    @Override
    protected void onDetachedFromWindow() {
        super.onDetachedFromWindow();
        ZIMSDKManager.getInstance().getDispatchService().unregisterZIMEventHandlerCallbacks(mZimEventHandlerWrap);
    }
}
