package im.zego.chathouse.room.custom;

import android.app.Activity;
import android.app.Dialog;
import android.content.Context;
import android.view.View;
import android.view.Window;
import android.widget.TextView;

import androidx.annotation.NonNull;

import im.zego.chathouse.R;


/**
 * 房间通用确认弹窗
 */
public class RoomCommonDialog extends Dialog {

    private TextView mTitle;
    private TextView mContent;
    private TextView mCancelButton;
    private TextView mConfirmButton;

    private final boolean mCancelable;

    public RoomCommonDialog(@NonNull Context context, boolean cancelable) {
        this(context, R.layout.chathouse_dialog_common,cancelable);
    }

    public RoomCommonDialog(@NonNull Context context, int layoutId, boolean cancelable) {
        this(context, R.style.ChatHouseBaseDialogStyle, layoutId,cancelable);
    }

    private RoomCommonDialog(@NonNull Context context, int themeResId, int layoutId, boolean cancelable) {
        super(context, themeResId);
        mCancelable = cancelable;
        init(context, layoutId);
    }

    private void init(Context context, int layoutId) {
        //设置dialog位置
        Window window = getWindow();
        if (window != null) {
            window.setContentView(layoutId);
            mTitle = window.findViewById(R.id.tv_title);
            mContent = window.findViewById(R.id.tv_content);
            mCancelButton = window.findViewById(R.id.tv_cancel);
            mConfirmButton = window.findViewById(R.id.tv_confirm);
        }

        setCancelable(mCancelable);
        setCanceledOnTouchOutside(mCancelable);

        if(context instanceof Activity){
           if(((Activity)context).isDestroyed() || ((Activity)context).isFinishing()) {
               return;
           }
        }
        show();

    }

    public void setMsgTitle(CharSequence title) {

        if(mTitle == null){
            return;
        }

        mTitle.setText(title);
    }

    public void setMsgContent(CharSequence content) {

        if(mContent == null){
            return;
        }
        mContent.setText(content);
    }

    public void setConfirmListener(View.OnClickListener listener) {

        if(mConfirmButton == null){
            return;
        }

        mConfirmButton.setOnClickListener(listener);
    }

    public void setCancelListener(View.OnClickListener listener) {

        if(mCancelButton == null){
            return;
        }

        mCancelButton.setOnClickListener(listener);
    }
}
