package im.zego.chathouse.room.custom;

import static im.zego.chathouse.Constants.INVITE_CALL_TIME_OUT;

import android.app.Activity;
import android.app.Dialog;
import android.content.Context;
import android.graphics.Color;
import android.os.CountDownTimer;
import android.text.Spannable;
import android.text.SpannableString;
import android.text.style.ForegroundColorSpan;
import android.view.View;
import android.view.Window;
import android.widget.TextView;

import androidx.annotation.NonNull;

import im.zego.chathouse.R;
import im.zego.enjoycommon.util.MiscUtils;


/**
 * 房主邀请弹窗
 */
public class RoomInviteDialog extends Dialog {

    private TextView mTitle;
    private TextView mRejectButton;
    private TextView mAgreeButton;

    private final boolean mCancelable;

    public RoomInviteDialog(@NonNull Context context, boolean cancelable) {
        this(context, R.layout.chathouse_dialog_invite,cancelable);
    }

    public RoomInviteDialog(@NonNull Context context, int layoutId, boolean cancelable) {
        this(context, R.style.ChatHouseBaseDialogStyle, layoutId,cancelable);
    }

    private RoomInviteDialog(@NonNull Context context, int themeResId, int layoutId, boolean cancelable) {
        super(context, themeResId);
        mCancelable = cancelable;
        init(context, layoutId);
    }

    private void init(Context context, int layoutId) {
        //设置dialog位置
        Window window = getWindow();
        if (window != null) {
            window.setContentView(layoutId);
            mTitle = window.findViewById(R.id.tv_title);
            mRejectButton = window.findViewById(R.id.tv_reject);
            mAgreeButton = window.findViewById(R.id.tv_agree);
        }

        setCancelable(mCancelable);
        setCanceledOnTouchOutside(mCancelable);

        if(context instanceof Activity){
           if(((Activity)context).isDestroyed() || ((Activity)context).isFinishing()) {
               return;
           }
        }
        show();
        //设置倒计时时间，并到时间关闭弹窗
        CountDownTimer countDownTimer = new CountDownTimer(INVITE_CALL_TIME_OUT*1000,1000) {
            @Override
            public void onTick(long millisUntilFinished) {
                String str = MiscUtils.getApplication().getString(R.string.chathouse_invite_seat_count_down);
                SpannableString spannableString = new SpannableString(str);
                spannableString.setSpan(new ForegroundColorSpan(Color.parseColor("#6D24F6")), 4, str.length(), Spannable.SPAN_INCLUSIVE_EXCLUSIVE);
                setMsgTitle(spannableString);
            }

            @Override
            public void onFinish() {
                dismiss();
            }
        };
        countDownTimer.start();


    }

    public void setMsgTitle(CharSequence title) {

        if(mTitle == null){
            return;
        }

        mTitle.setText(title);
    }

    public void setAgreeListener(View.OnClickListener listener) {

        if(mAgreeButton == null){
            return;
        }

        mAgreeButton.setOnClickListener(listener);
    }

    public void setRejectListener(View.OnClickListener listener) {

        if(mRejectButton == null){
            return;
        }

        mRejectButton.setOnClickListener(listener);
    }

}
