package im.zego.chathouse.room.custom;

import static im.zego.chathouse.Constants.SOUND_LEVEL_MIN_VALUE;
import static im.zego.chathouse.model.Seat.SeatStatus.INVITING;
import static im.zego.chathouse.model.Seat.SeatStatus.OCCUPIED;
import static im.zego.chathouse.model.Seat.SeatStatus.VACANT;
import static im.zego.chathouse.sdk.zim.ZIMEventHandlerWrap.UI_PRIORITY;

import android.content.Context;
import android.util.AttributeSet;
import android.view.View;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.recyclerview.widget.SimpleItemAnimator;

import java.util.HashMap;
import java.util.Map;

import im.zego.chathouse.R;
import im.zego.chathouse.help.PermissionHelp;
import im.zego.chathouse.manager.UserManager;
import im.zego.chathouse.model.RoomAttributes;
import im.zego.chathouse.model.Seat;
import im.zego.chathouse.model.User;
import im.zego.chathouse.room.RoomManager;
import im.zego.chathouse.room.custom.adapter.RoomStageGuestAdapter;
import im.zego.chathouse.sdk.rtc.RTCEventHandlerWrap;
import im.zego.chathouse.sdk.rtc.RTCSDKManager;
import im.zego.chathouse.sdk.zim.ZIMEventHandlerWrap;
import im.zego.chathouse.sdk.zim.ZIMSDKManager;
import im.zego.chathouse.sdk.zim.callback.IZIMCommonCallback;
import im.zego.chathouse.utils.GlideImageLoader;
import im.zego.chathouse.utils.ToastUtil;
import im.zego.chathouse.utils.ViewUtil;
import im.zego.chathouse.view.ChatHouseWaterWaveView;
import im.zego.enjoycommon.log.L;
import im.zego.enjoycommon.util.T;
import im.zego.zim.ZIM;
import im.zego.zim.entity.ZIMRoomAttributesUpdateInfo;

/**
 * 房间麦位展示视图
 */
public class RoomMicListView extends FrameLayout {

    private RecyclerView mStageGuestList;
    private RoomStageGuestAdapter mAdapter;
    private ImageView mHostUserIcon;
    private ImageView mHostMicOnOff;
    private TextView mHostName;

    /**
     * 房主音浪动画视图
     */
    private ChatHouseWaterWaveView mWaterWave;
    /**
     * 房间ID
     */
    private String mRoomID;

    private ZIMEventHandlerWrap mZimEventHandlerWrap;
    private RTCEventHandlerWrap mRTCEventHandlerWrap;

    public RoomMicListView(@NonNull Context context,String roomID) {
        this(context,null,roomID);
    }

    public RoomMicListView(@NonNull Context context, @Nullable AttributeSet attrs,String roomID) {
        this(context, attrs,0,roomID);
    }

    public RoomMicListView(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr,String roomID) {
        super(context, attrs, defStyleAttr);
        mRoomID = roomID;
        View.inflate(context, R.layout.chathouse_view_mic_list,this);
        initView(context);
        mZimEventHandlerWrap = new ZIMEventHandlerWrap(mRoomID,UI_PRIORITY) {
            @Override
            public void onRoomAttributesUpdated(ZIM zim, ZIMRoomAttributesUpdateInfo info, String roomID) {
                super.onRoomAttributesUpdated(zim, info, roomID);
                updateMicList();
            }
        };

        mRTCEventHandlerWrap = new RTCEventHandlerWrap(mRoomID,UI_PRIORITY) {
            @Override
            public void onSoundLevelUpdate(HashMap<String, Float> userVolumeMap) {
                super.onSoundLevelUpdate(userVolumeMap);
                //音浪更新回调返回的map 为  流ID：音浪值
                RoomManager.getInstance().applyRoomStreamService(services -> {

                    RoomManager.getInstance().applyRoomAttributesService(services1 -> {

                        HashMap<Integer, Float> map = new HashMap<>();
                        for(Map.Entry<String, Float> entry: userVolumeMap.entrySet()) {

                            //通过流ID，查找到UID
                            String uid = services.getUserIDByStreamID(entry.getKey());
                            if(uid != null){
                                //通过UID，查找到麦位信息
                                Seat seat = services1.getSeatByUser(uid);
                                if(seat != null && seat.getStatus() == OCCUPIED.status){
                                    //查找到，则记录下来值为 麦位：音浪值
                                    map.put(seat.getIdx(),entry.getValue());
                                }
                            }
                        }
                        //更新音浪效果
                        updateVolumeLevel(map);




                    });
                });
            }

            @Override
            public void onMixSoundLevelUpdate(HashMap<Integer, Float> userVolumeMap) {
                super.onMixSoundLevelUpdate(userVolumeMap);
                //混流音浪更新回调返回的map 为  麦位：音浪值，无需数据处理
                updateVolumeLevel(userVolumeMap);

            }
        };


    }

    private void initView(Context context){

        mHostUserIcon = findViewById(R.id.host_user_icon);
        mHostName = findViewById(R.id.host_name);
        mHostMicOnOff = findViewById(R.id.host_mic_on_off);

        mWaterWave = findViewById(R.id.water_wave);

        mStageGuestList = findViewById(R.id.stage_guest_list);
        mStageGuestList.setLayoutManager(new GridLayoutManager(context,4));
        mAdapter = new RoomStageGuestAdapter();
        mStageGuestList.setAdapter(mAdapter);
        SimpleItemAnimator simpleItemAnimator = ((SimpleItemAnimator)mStageGuestList.getItemAnimator());
        if(simpleItemAnimator != null){
            simpleItemAnimator.setSupportsChangeAnimations(false);
        }

        mAdapter.setOnClickListener(new RoomStageGuestAdapter.OnClickListener() {
            @Override
            public void onClick(View v, int position,Seat seat) {
                PermissionHelp.micPermission(ViewUtil.getActivity(RoomMicListView.this), () -> clickSeat( position, seat));
            }

            @Override
            public void operateSeat(View v, int position, Seat seat, boolean open) {
                RoomManager.getInstance().applyRoomAttributesService(services -> {
                    if(open){
                        services.openSeat(position+1);
                    }else{
                        if (seat.getStatus() == INVITING.status) {
                            if(seat.getUser() != null) {
                                T.show(String.format(getContext().getString(R.string.chathouse_already_invite_up_seat_please_wait_for_his_confirm), seat.getUser().getName()));
                            }else{
                                T.show( getContext().getString(R.string.chathouse_already_invite_user_up_seat_please_wait_for_his_confirm));
                            }
                        } else {
                            services.closeSeat(position + 1);
                        }
                    }
                });
            }
        });
    }

    private void clickSeat(int position,Seat seat){
        if (RoomManager.getInstance().isHost()) {
            if (seat.getStatus() == INVITING.status) {
                //如果是邀请状态，则不能再邀请
                if(seat.getUser() != null) {
                    T.show(String.format(getContext().getString(R.string.chathouse_already_invite_up_seat_please_wait_for_his_confirm), seat.getUser().getName()));
                }else{
                    T.show(getContext().getString(R.string.chathouse_already_invite_user_up_seat_please_wait_for_his_confirm) );
                }
            }else if(seat.getStatus() == OCCUPIED.status){
                //如果已经是麦位上的，房主会弹出操作框
                RoomManager.getInstance().applyRoomDialogService(services -> {
                    RoomManager.getInstance().applyRoomAttributesService(services1 -> {
                        int canSpeak = services1.getCanSpeak(position + 1);
                        if(canSpeak >= 0){
                            services.showOperationDialog(ViewUtil.getActivity(RoomMicListView.this), position + 1, canSpeak != 0);
                        }
                    });

                });


            }else if(seat.getStatus() == VACANT.status){
                //空闲状态，房主会弹出用户列表
                RoomManager.getInstance().applyRoomDialogService(services -> {
                    services.showPersonListDialog(ViewUtil.getActivity(RoomMicListView.this), position + 1);
                });
            }
        }else {
            if (seat.getStatus() == INVITING.status) {
                //非房主点击邀请状态的，则不能再申请
                RoomManager.getInstance().applyRoomAttributesService(services2 -> {
                    Seat mySeat = services2.getSeatByUser(UserManager.getInstance().getUserID());
                    if(mySeat != null){
                        if(mySeat.getStatus() == INVITING.status) {
                            T.show(String.format(getContext().getString(R.string.chathouse_already_apply_seat_please_do_not_click_it_again), mySeat.getIdx()));
                        }
                    }else{
                        T.show(getContext().getString(R.string.chathouse_already_has_user_apply_please_try_again));
                    }
                });

//                }
            }else if(seat.getStatus() == VACANT.status){
                //空闲状态则说明该麦位可申请
                RoomManager.getInstance().applyInviteService(services -> {
                    RoomManager.getInstance().applyRoomInfoServices(services1 -> {
                        //判断自己是否已经在其他麦位被邀请或者申请了，则不能申请
                        if (services.canApplySeat()) {
                            services.audienceApplySeat(position + 1, services1.getRoomInfo().getCreatorId(), new IZIMCommonCallback() {
                                @Override
                                public void onZIMCallback(int errorCode) {
                                    if (errorCode == 0) {
                                        T.show(getContext().getString(R.string.chathouse_already_apply_seat_please_wait_anchor_confirm));
                                    } else {
                                        T.show(String.format(getContext().getString(R.string.chathouse_apply_seat_fail_error_code), errorCode));
                                    }
                                }
                            });
                        } else {
                            RoomManager.getInstance().applyRoomAttributesService(services2 -> {
                                Seat mySeat = services2.getSeatByUser(UserManager.getInstance().getUserID());
                                if(mySeat != null){
                                    if(mySeat.getStatus() == INVITING.status){
                                        T.show(String.format(getContext().getString(R.string.chathouse_already_apply_seat_please_do_not_click_it_again), mySeat.getIdx()) );
                                    }
                                }
                            });

                        }
                    });
                });
            }

        }
    }


    public void updateMicList(){
        RoomManager.getInstance().applyRoomAttributesService(
                services -> {
                    RoomAttributes roomAttributes = services.getRoomAttributes();
                    Seat seat = roomAttributes.getSeat(0);
                    User host = seat.getUser();
                    if(host != null) {
                        GlideImageLoader.getInstance().displayCircleImageByUrl(host.getUserIcon(), mHostUserIcon);
                        mHostName.setText(host.getName());
                        if(host.getMicOn()){
                            mHostMicOnOff.setVisibility(GONE);
                        }else {
                            mHostMicOnOff.setVisibility(VISIBLE);
                            mWaterWave.cancelCountDown();

                        }
                    }

                    if(mAdapter != null){
                        mAdapter.updateRoomAttributes(roomAttributes);
                    }
                }
        );
    }

    /**
     * 更新音浪，音浪值大于1，则进行更新
     * @param map
     */
    public void updateVolumeLevel(HashMap<Integer, Float> map){

        if(map == null){
            return;
        }

        Float value = map.get(0);
        RoomManager.getInstance().applyRoomAttributesService(
                attributesService -> {

                    if(value!= null && mWaterWave != null ) {
                        RoomAttributes roomAttributes = attributesService.getRoomAttributes();
                        Seat hostSeat = roomAttributes.getSeat(0);
                        User host = hostSeat.getUser();
                        if (host != null && host.getMicOn() && value > SOUND_LEVEL_MIN_VALUE) {


                            mWaterWave.startCountDown();


                        } else {
                            mWaterWave.cancelCountDown();
                        }

                    }


                    for (Map.Entry<Integer,Float> entry : map.entrySet()){

                        if(entry.getValue() != null){
                            GridLayoutManager gridLayoutManager = (GridLayoutManager) mStageGuestList.getLayoutManager();

                            if(gridLayoutManager != null) {
                                //查找对应麦位视图
                                View view = gridLayoutManager.findViewByPosition(entry.getKey() - 1);
                                if(view != null){
                                    //查找麦位对应的音浪动画视图
                                    ChatHouseWaterWaveView waterWave = view.findViewById(R.id.water_wave);
                                    if(waterWave != null){
                                        RoomManager.getInstance().applyRoomAttributesService(services -> {
                                            Seat seat = services.getRoomAttributes().getSeat(entry.getKey());
                                            //麦位是占据状态，且音浪超过最低值要求则展示动画
                                            if(seat != null && seat.getStatus() == OCCUPIED.status && entry.getValue() > SOUND_LEVEL_MIN_VALUE
                                                    && seat.getUser() != null && seat.getUser().getMicOn() && seat.getUser().isCanSpeak()) {
                                                waterWave.startCountDown();
                                            }else{
                                                waterWave.cancelCountDown();
                                            }
                                        });
                                    }
                                }
                            }
                        }

                    }

                });




    }


    @Override
    protected void onAttachedToWindow() {
        super.onAttachedToWindow();
        ZIMSDKManager.getInstance().getDispatchService().registerZIMEventHandlerCallbacks(mZimEventHandlerWrap);
        RTCSDKManager.getInstance().getDispatchService().registerRTCEventHandlerCallbacks(mRTCEventHandlerWrap);
    }

    @Override
    protected void onDetachedFromWindow() {
        super.onDetachedFromWindow();
        ZIMSDKManager.getInstance().getDispatchService().unregisterZIMEventHandlerCallbacks(mZimEventHandlerWrap);
        RTCSDKManager.getInstance().getDispatchService().unregisterRTCEventHandlerCallbacks(mRTCEventHandlerWrap);
    }
}
