package im.zego.chathouse.room.custom;

import static im.zego.chathouse.sdk.zim.ZIMEventHandlerWrap.UI_PRIORITY;

import android.content.Context;
import android.util.AttributeSet;
import android.view.View;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.constraintlayout.widget.ConstraintLayout;

import java.util.ArrayList;

import im.zego.chathouse.R;
import im.zego.chathouse.room.RoomManager;
import im.zego.chathouse.room.help.TimerTaskHelp;
import im.zego.chathouse.sdk.zim.ZIMEventHandlerWrap;
import im.zego.chathouse.sdk.zim.ZIMSDKManager;
import im.zego.chathouse.utils.GlideImageLoader;
import im.zego.zim.ZIM;
import im.zego.zim.entity.ZIMUserInfo;

/**
 * 房间顶部信息视图
 */
public class RoomTopInfoView extends FrameLayout {

    private ImageView mAnchorHeadPortrait;
    private TextView mRoomName;
    private TextView mTvOnlineNum;
    private View mCloseRoom;
    private ConstraintLayout mClOnlineNum;

    private ZIMEventHandlerWrap zimEventHandlerWrap;

    public RoomTopInfoView(@NonNull Context context) {
        this(context,null);
    }

    public RoomTopInfoView(@NonNull Context context, @Nullable AttributeSet attrs) {
        this(context, attrs,0);
    }

    public RoomTopInfoView(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        View.inflate(context, R.layout.chathouse_view_room_top_info,this);
        initView();
        RoomManager.getInstance().applyRoomInfoServices(services -> {
            zimEventHandlerWrap = new ZIMEventHandlerWrap(services.getRoomID(),UI_PRIORITY) {
                @Override
                public void onRoomOnlineMemberCountQueried(int count, int errorCode) {
                    super.onRoomOnlineMemberCountQueried(count, errorCode);

                    if(errorCode == 0){
                        if(mTvOnlineNum != null){
                            mTvOnlineNum.setText(String.valueOf(count));
                        }
                    }
                }

                @Override
                public void onRoomMemberLeft(ZIM zim, ArrayList<ZIMUserInfo> memberList, String roomID) {
                    super.onRoomMemberLeft(zim, memberList, roomID);

                    TimerTaskHelp.startQueryOnlineMemberCount();
                }

                @Override
                public void onRoomMemberJoined(ZIM zim, ArrayList<ZIMUserInfo> memberList, String roomID) {
                    super.onRoomMemberJoined(zim, memberList, roomID);

                    TimerTaskHelp.startQueryOnlineMemberCount();
                }
            };
        });


    }

    private void initView(){
        mAnchorHeadPortrait = findViewById(R.id.anchor_head_portrait);
        mRoomName = findViewById(R.id.room_name);
        mTvOnlineNum = findViewById(R.id.tv_online_num);
        mCloseRoom = findViewById(R.id.close_room);
        mClOnlineNum = findViewById(R.id.cl_online_num);
    }

    public void update(String name,String userIcon){
        GlideImageLoader.getInstance().displayCircleImageByUrl(userIcon, mAnchorHeadPortrait);
        mRoomName.setText(name);
    }


    public void setCloseListener(OnClickListener listener){
        mCloseRoom.setOnClickListener(listener);
    }

    public void setOnlineListener(OnClickListener listener){
        mClOnlineNum.setOnClickListener(listener);
    }

    @Override
    protected void onAttachedToWindow() {
        super.onAttachedToWindow();
        //注册ZIM 监听观众人数
        ZIMSDKManager.getInstance().getDispatchService().registerZIMEventHandlerCallbacks(zimEventHandlerWrap);
        //启动定时任务更新房间人数
        TimerTaskHelp.startQueryOnlineMemberCount();
    }

    @Override
    protected void onDetachedFromWindow() {
        super.onDetachedFromWindow();
        ZIMSDKManager.getInstance().getDispatchService().unregisterZIMEventHandlerCallbacks(zimEventHandlerWrap);
    }
}
