package im.zego.chathouse.room.custom.adapter;

import android.graphics.Color;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import im.zego.chathouse.R;
import im.zego.chathouse.manager.UserManager;
import im.zego.chathouse.model.User;
import im.zego.chathouse.room.RoomManager;
import im.zego.chathouse.room.service.callback.UserServiceHandler;
import im.zego.chathouse.utils.GlideImageLoader;

/**
 * 房间用户列表适配器
 */
public class RoomPersonListAdapter extends RecyclerView.Adapter<RoomPersonListAdapter.PersonListViewHolder>{

    private static final String TAG = "RoomPersonListAdapter";

    private List<User> mUserBaseInfoList = new ArrayList<>();

    private final UserServiceHandler mUserServiceHandler = new UserServiceHandler() {
        @Override
        public void onUserStatusChange(User user) {
            super.onUserStatusChange(user);
            notifyUserInfo(user.getId());
        }
    };

    @NonNull
    @Override
    public RoomPersonListAdapter.PersonListViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.chathouse_item_person_list, parent, false);
        return new RoomPersonListAdapter.PersonListViewHolder(view);
    }

    @Override
    public void onBindViewHolder(@NonNull RoomPersonListAdapter.PersonListViewHolder holder, int position) {

        User user = mUserBaseInfoList.get(position);
        //重置点击事件
        holder.listMenu.setOnClickListener(null);
        if(user != null){
            GlideImageLoader.getInstance().displayCircleImageByUrl(user.getUserIcon(), holder.userIcon);
            holder.userName.setText(user.getName());
            holder.listMenu.setVisibility(View.VISIBLE);


            if(UserManager.getInstance().getUserID().equals(user.getId())){
                holder.listMenu.setBackground(null);
                holder.listMenu.setTextColor(Color.parseColor("#FFA7A6B7"));
                holder.listMenu.setText(R.string.chathouse_me);
            }else if(user.getStatus() == User.Status.OCCUPIED.status){
                holder.listMenu.setBackgroundResource(R.drawable.chathouse_shape_list_menu);
                holder.listMenu.setTextColor(Color.parseColor("#FFA7A6B7"));
                holder.listMenu.setText(R.string.chathouse_already_seat_obtained);
            }else if(user.getStatus() == User.Status.INVITING.status){
                holder.listMenu.setBackgroundResource(R.drawable.chathouse_shape_list_menu);
                holder.listMenu.setTextColor(Color.parseColor("#FFA7A6B7"));
                holder.listMenu.setText(R.string.chathouse_has_invited);
            }else {
                holder.listMenu.setBackgroundResource(R.drawable.chathouse_shape_invite);
                holder.listMenu.setTextColor(Color.parseColor("#FFFFFFFF"));
                holder.listMenu.setText(R.string.chathouse_invite_seat);
                holder.listMenu.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        if(mOnClickListener != null){
                            mOnClickListener.invite(v, holder.getAdapterPosition(), user);
                        }
                    }
                });
            }


            holder.listMenu.setVisibility(View.VISIBLE);

        }else{
            GlideImageLoader.getInstance().displayCircleImageByUrl("", holder.userIcon);
            holder.userName.setText("");
            holder.listMenu.setVisibility(View.GONE);
        }

    }

    @Override
    public int getItemCount() {
        return mUserBaseInfoList.size();
    }

    static class PersonListViewHolder extends RecyclerView.ViewHolder{

        private final ImageView userIcon;
        private final TextView userName;
        private final TextView listMenu;

        public PersonListViewHolder(@NonNull View itemView) {
            super(itemView);
            userIcon = itemView.findViewById(R.id.user_icon);
            userName = itemView.findViewById(R.id.user_name);
            listMenu = itemView.findViewById(R.id.list_menu);
        }
    }

    public void setUserBaseInfoList(List<User> mUserBaseInfoList) {
        Collections.sort(mUserBaseInfoList,Collections.reverseOrder());
        this.mUserBaseInfoList = mUserBaseInfoList;
        notifyDataSetChanged();
    }

    public void notifyUserInfo(String userID){
        if(userID == null && mUserBaseInfoList == null){
            return;
        }

        int length = mUserBaseInfoList.size();
        for(int i = 0; i < length;i++){
            User data = mUserBaseInfoList.get(i);
            if(data != null && data.getId().equals(userID)){
                notifyItemChanged(i);
            }
        }
    }

    @Override
    public void onAttachedToRecyclerView(@NonNull RecyclerView recyclerView) {
        super.onAttachedToRecyclerView(recyclerView);
        RoomManager.getInstance().applyUserService(services -> {
            services.registerUserServiceHandlerCallbacks(mUserServiceHandler);
        });
    }

    @Override
    public void onDetachedFromRecyclerView(@NonNull RecyclerView recyclerView) {
        super.onDetachedFromRecyclerView(recyclerView);
        RoomManager.getInstance().applyUserService(services -> {
            services.unregisterUserServiceHandlerCallbacks(mUserServiceHandler);
        });
    }

    private OnClickListener mOnClickListener;

    public void setOnClickListener(OnClickListener onClickListener) {
        this.mOnClickListener = onClickListener;
    }

    public interface OnClickListener {
        void invite(View v, int position,User user);
    }
}
