package im.zego.chathouse.room.custom.adapter;

import static im.zego.chathouse.model.Seat.SeatStatus.*;

import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import java.util.ArrayList;

import im.zego.chathouse.R;
import im.zego.chathouse.model.RoomAttributes;
import im.zego.chathouse.model.Seat;
import im.zego.chathouse.model.User;
import im.zego.chathouse.room.RoomManager;
import im.zego.chathouse.sdk.zim.ZIMSDKManager;
import im.zego.chathouse.sdk.zim.callback.IZIMQueryRoomMemberListCallback;
import im.zego.chathouse.utils.GlideImageLoader;
import im.zego.chathouse.view.ChatHouseWaterWaveView;
import im.zego.zim.entity.ZIMUserFullInfo;

/**
 * 房间麦位适配器
 */
public class RoomStageGuestAdapter extends RecyclerView.Adapter<RoomStageGuestAdapter.StageGuestViewHolder>{

    private RoomAttributes mRoomAttributes = new RoomAttributes();

    @NonNull
    @Override
    public StageGuestViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.chathouse_item_mic_list, parent, false);
        return new StageGuestViewHolder(view);
    }

    @Override
    public void onBindViewHolder(@NonNull StageGuestViewHolder holder, int position) {
        Seat seat = mRoomAttributes.getSeat(position+1);
        if(seat != null){
            User user = seat.getUser();

            boolean isHost = RoomManager.getInstance().isHost();

            if(seat.getStatus() == VACANT.status || seat.getStatus() == INVITING.status){
                if(isHost) {
                    holder.mUserName.setText(R.string.chathouse_invite_up_seat);
                    if(seat.getStatus() == VACANT.status) {
                        holder.mLockMic.setVisibility(View.VISIBLE);
                    }else {
                        holder.mLockMic.setVisibility(View.GONE);
                    }
                    holder.mLockMic.setText(R.string.chathouse_close_seat);
                }else {
                    holder.mUserName.setText(R.string.chathouse_apply_seat);
                    holder.mLockMic.setVisibility(View.GONE);
                }
                GlideImageLoader.getInstance().displayCircleImageByUrl("",holder.mMicUserIcon);
                holder.mMicBgBorder.setBackgroundResource(R.drawable.chathouse_mic_free_bg_border);
                holder.mMicFreeBg.setImageResource(R.drawable.chathouse_view_mic_add);
                holder.mMicOnOff.setVisibility(View.GONE);

            }else if(seat.getStatus() == OCCUPIED.status){
                holder.mLockMic.setVisibility(View.GONE);
                if(user != null) {
                    holder.mUserName.setText(user.getName());

                    //异常情况下的头像展示
                    if(TextUtils.isEmpty(user.getUserIcon())){
                        RoomManager.getInstance().applyUserService(
                                services -> {
                                    User user2 = services.getUser(user.getId());
                                    if(user2 != null){
                                        if(!TextUtils.isEmpty(user2.getUserIcon())){
                                            user.setUserIcon(user2.getUserIcon());
                                            GlideImageLoader.getInstance().displayCircleImageByUrl(user.getUserIcon(), holder.mMicUserIcon);
                                        }else {
                                            ArrayList<String> userList = new ArrayList<>();
                                            userList.add(user.getId());
                                            ZIMSDKManager.getInstance().queryUsersInfo(userList, new IZIMQueryRoomMemberListCallback() {
                                                @Override
                                                public void onRoomMemberQueried(ArrayList<ZIMUserFullInfo> userList, String nextFlag, int errorCode) {
                                                    if(errorCode == 0 && userList != null && userList.size() >= 1){
                                                        ZIMUserFullInfo info = userList.get(0);
                                                        user2.setUserIcon(info.userAvatarUrl);
                                                    }
                                                    notifyDataSetChanged();
                                                }
                                            });
                                        }
                                    }
                                }
                        );
                    }else {
                        GlideImageLoader.getInstance().displayCircleImageByUrl(user.getUserIcon(), holder.mMicUserIcon);
                    }



                    if(user.isCanSpeak() && user.getMicOn()){
                        holder.mMicOnOff.setVisibility(View.GONE);
                    }else {
                        holder.mWaterWave.cancelCountDown();
                        holder.mMicOnOff.setVisibility(View.VISIBLE);
                    }
                }else {
                    holder.mUserName.setText("");
                    GlideImageLoader.getInstance().displayCircleImageByUrl("", holder.mMicUserIcon);
                    holder.mMicOnOff.setVisibility(View.GONE);
                }
//                holder.mMicBgBorder.setBackgroundResource(R.drawable.chathouse_mic_bg_border);
                holder.mMicBgBorder.setBackground(null);
                holder.mMicFreeBg.setImageResource(R.drawable.chathouse_view_mic_add);



            }else {
                if(isHost) {
                    holder.mUserName.setText(R.string.chathouse_invite_up_seat);
                    holder.mLockMic.setVisibility(View.VISIBLE);
                    holder.mLockMic.setText(R.string.chathouse_open_seat);
                }else {
                    holder.mUserName.setText(R.string.chathouse_apply_seat);
                    holder.mLockMic.setVisibility(View.GONE);
                }
                GlideImageLoader.getInstance().displayCircleImageByUrl("",holder.mMicUserIcon);
                holder.mMicBgBorder.setBackgroundResource(R.drawable.chathouse_mic_free_bg_border);
                holder.mMicFreeBg.setImageResource(R.mipmap.chathouse_mic_lock);
                holder.mMicOnOff.setVisibility(View.GONE);
            }

        }

        //非麦上，移除音浪动画
        if(seat == null || seat.getStatus() != OCCUPIED.status){
            holder.mWaterWave.cancelCountDown();
        }

        holder.mMicBgBorder.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                if(mOnClickListener != null && seat != null){
                    if(seat.getStatus() != CLOSED.status) {
                        mOnClickListener.onClick(v, holder.getAdapterPosition(),seat);
                    }
                }
            }
        });

        holder.mLockMic.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if(mOnClickListener != null && seat != null){
                    mOnClickListener.operateSeat(v, holder.getAdapterPosition(),seat, seat.getStatus() == CLOSED.status);
                }
            }
        });
    }

    @Override
    public int getItemCount() {
        return RoomAttributes.SEAT_LIST.length-1;
    }

    /**
     * 更新房间属性，会刷新麦上列表展示
     * @param roomAttributes
     */
    public void updateRoomAttributes(RoomAttributes roomAttributes){
        mRoomAttributes = roomAttributes;
        notifyDataSetChanged();
    }



    static class StageGuestViewHolder extends RecyclerView.ViewHolder{

        public ImageView mMicBgBorder;
        public ImageView mMicFreeBg;
        public ImageView mMicUserIcon;
        public ImageView mMicOnOff;
        public TextView mUserName;
        public TextView mLockMic;
        public ChatHouseWaterWaveView mWaterWave;

        public StageGuestViewHolder(@NonNull View itemView) {
            super(itemView);
            mMicBgBorder = itemView.findViewById(R.id.mic_bg_border);
            mMicFreeBg = itemView.findViewById(R.id.mic_free_bg);
            mMicUserIcon = itemView.findViewById(R.id.mic_user_icon);
            mMicOnOff = itemView.findViewById(R.id.mic_on_off);
            mUserName = itemView.findViewById(R.id.user_name);
            mLockMic = itemView.findViewById(R.id.lock_mic);
            mWaterWave = itemView.findViewById(R.id.water_wave);



        }
    }

    private OnClickListener mOnClickListener;

    public void setOnClickListener(OnClickListener onClickListener) {
        this.mOnClickListener = onClickListener;
    }

    public interface OnClickListener {
        /**
         * 点击麦位
         * @param position 列表position，非麦位
         * @param seat 麦位信息
         */
        void onClick(View v, int position,Seat seat);

        /**
         * 操作麦位开关
         * @param position 列表position，非麦位
         * @param seat 麦位信息
         * @param open 开关
         */
        void operateSeat(View v, int position, Seat seat, boolean open);
    }
}
