package im.zego.chathouse.room.help;

import android.app.Activity;
import android.content.DialogInterface;
import android.view.Gravity;
import android.view.View;
import android.view.Window;
import android.view.inputmethod.InputMethodManager;
import android.widget.TextView;

import androidx.annotation.NonNull;

import com.google.android.material.bottomsheet.BottomSheetBehavior;
import com.google.android.material.bottomsheet.BottomSheetDialog;

import im.zego.chathouse.R;
import im.zego.chathouse.manager.UserManager;
import im.zego.chathouse.model.Seat;
import im.zego.chathouse.room.RoomManager;
import im.zego.chathouse.sdk.zim.callback.IZIMCommonCallback;
import im.zego.enjoycommon.log.L;
import im.zego.enjoycommon.util.T;

/**
 * 操作弹窗辅助类
 */
public class OperationDialogHelp {


    public boolean isShowing = false;

    public BottomSheetDialog showOperationDialog(Activity activity, int seatMicIndex,boolean canSpeak) {

        if(isShowing){
            return null;
        }
        isShowing = true;
        BottomSheetDialog operationDialog = new BottomSheetDialog(activity, R.style.ChatHouseBottomSheetDialog);
        View view = View.inflate(activity, R.layout.chathouse_dailog_operation, null);
        view.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                InputMethodManager inputMethodManager = (InputMethodManager) activity.getSystemService(Activity.INPUT_METHOD_SERVICE);
                inputMethodManager.hideSoftInputFromWindow(view.getWindowToken(), InputMethodManager.HIDE_NOT_ALWAYS);
            }
        });

        TextView banningSpeakingView = view.findViewById(R.id.banning_speaking);
        banningSpeakingView.setText(canSpeak ? activity.getString(R.string.chathouse_prohibit_to_speak):activity.getString(R.string.chathouse_remove_banned));
        banningSpeakingView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                RoomManager.getInstance().applyRoomAttributesService(
                        services -> {
                            Seat seat = services.getRoomAttributes().getSeat(seatMicIndex);


                            if(seat!= null && seat.getStatus() == Seat.SeatStatus.OCCUPIED.status && seat.getUser() != null && seat.getUser().getId()!= null) {
                                String id = seat.getUser().getId();
                                services.canSpeak(seatMicIndex, !canSpeak,id);
                            }else {
                                T.show(activity.getString(R.string.chathouse_current_seat_not_operable));
                            }

                        }

                );

                operationDialog.dismiss();
            }
        });

        view.findViewById(R.id.down_seat).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                // 数据的变更
                // 假如身份是房主则需要同时修改邀请服务，房间属性，用户信息
                L.e("ZIM","down_seat");
                RoomManager.getInstance().applyRoomAttributesService(services -> {

                    Seat seat = services.getRoomAttributes().getSeat(seatMicIndex);


                    if(seat!= null && seat.getStatus() == Seat.SeatStatus.OCCUPIED.status && seat.getUser() != null && seat.getUser().getId()!= null){

                        String id = seat.getUser().getId();

                        services.downSeat(seatMicIndex);

                        RoomManager.getInstance().applyInviteService(services1 -> {
                            services1.removeInvite(seatMicIndex,id);
                        });


                    }else {
                        T.show(activity.getString(R.string.chathouse_current_seat_not_operable));
                    }
                });

                operationDialog.dismiss();
            }
        });


        view.findViewById(R.id.operation_cancel).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                operationDialog.dismiss();
            }
        });

        operationDialog.setContentView(view);
        final BottomSheetBehavior<View> mDialogBehavior = BottomSheetBehavior.from((View) view.getParent());
        mDialogBehavior.addBottomSheetCallback(new BottomSheetBehavior.BottomSheetCallback() {
            @Override
            public void onStateChanged(@NonNull View bottomSheet, int newState) {
                if (newState == BottomSheetBehavior.STATE_HIDDEN) {

                    operationDialog.dismiss();
                    mDialogBehavior.setState(BottomSheetBehavior.STATE_COLLAPSED);
                }
            }

            @Override
            public void onSlide(@NonNull View bottomSheet, float slideOffset) {
            }
        });

        operationDialog.setOnDismissListener(new DialogInterface.OnDismissListener() {
            @Override
            public void onDismiss(DialogInterface dialog) {
                isShowing = false;
            }
        });


        Window localWindow = operationDialog.getWindow();
        localWindow.setGravity(Gravity.BOTTOM);
        localWindow.setWindowAnimations(R.style.BottomSheetDialogAnim);

        operationDialog.show();

        return operationDialog;

    }


}



