package im.zego.chathouse.room.help;

import static im.zego.chathouse.ErrorCode.ERROR_INVITE_MIC_ALREADY_EXIST;
import static im.zego.chathouse.ErrorCode.ERROR_USER_NO_EXIST;
import static im.zego.chathouse.sdk.zim.ZIMEventHandlerWrap.UI_PRIORITY;

import android.app.Activity;
import android.content.Context;
import android.content.DialogInterface;
import android.view.Gravity;
import android.view.View;
import android.view.Window;
import android.view.inputmethod.InputMethodManager;
import android.widget.TextView;


import androidx.annotation.NonNull;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.google.android.material.bottomsheet.BottomSheetBehavior;
import com.google.android.material.bottomsheet.BottomSheetDialog;

import java.util.ArrayList;
import java.util.List;

import im.zego.chathouse.R;
import im.zego.chathouse.model.User;
import im.zego.chathouse.room.RoomManager;
import im.zego.chathouse.room.custom.adapter.RoomPersonListAdapter;
import im.zego.chathouse.sdk.zim.ZIMEventHandlerWrap;
import im.zego.chathouse.sdk.zim.ZIMSDKManager;
import im.zego.chathouse.sdk.zim.callback.IZIMCommonCallback;
import im.zego.chathouse.sdk.zim.callback.IZIMQueryRoomMemberListCallback;
import im.zego.enjoycommon.log.L;
import im.zego.enjoycommon.util.T;
import im.zego.zim.entity.ZIMUserFullInfo;

/**
 * 用户列表弹窗辅助类
 */
public class PersonListDialogHelp {


    public boolean isShowing = false;

    public BottomSheetDialog showPersonListDialog(Activity activity, int seatMicIndex) {

        if(isShowing){
            return null;
        }
        isShowing = true;
        BottomSheetDialog personListDialog = new BottomSheetDialog(activity, R.style.ChatHouseBottomSheetDialog);
        View view = View.inflate(activity, R.layout.chathouse_dialog_person_list, null);
        view.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                InputMethodManager inputMethodManager = (InputMethodManager) activity.getSystemService(Activity.INPUT_METHOD_SERVICE);
                inputMethodManager.hideSoftInputFromWindow(view.getWindowToken(), InputMethodManager.HIDE_NOT_ALWAYS);
            }
        });

        view.findViewById(R.id.v_top_line).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                personListDialog.dismiss();
            }
        });

        personListDialog.setContentView(view);
        final BottomSheetBehavior<View> mDialogBehavior = BottomSheetBehavior.from((View) view.getParent());
        mDialogBehavior.addBottomSheetCallback(new BottomSheetBehavior.BottomSheetCallback() {
            @Override
            public void onStateChanged(@NonNull View bottomSheet, int newState) {
                if (newState == BottomSheetBehavior.STATE_HIDDEN) {

                    personListDialog.dismiss();
                    mDialogBehavior.setState(BottomSheetBehavior.STATE_COLLAPSED);
                }
            }

            @Override
            public void onSlide(@NonNull View bottomSheet, float slideOffset) {
            }
        });

        personListDialog.setOnDismissListener(new DialogInterface.OnDismissListener() {
            @Override
            public void onDismiss(DialogInterface dialog) {
                isShowing = false;
                RecyclerView personList = (RecyclerView)personListDialog.findViewById(R.id.rv_person_list);
                if(personList != null) {
                    personList.setAdapter(null);
                }
            }
        });


        Window localWindow = personListDialog.getWindow();
        localWindow.setGravity(Gravity.BOTTOM);
        localWindow.setWindowAnimations(R.style.BottomSheetDialogAnim);

        personListDialog.show();
        updatePersonList(personListDialog,seatMicIndex);
        return personListDialog;

    }


    /**
     * 更新弹窗信息
     * @param personListDialog 弹窗
     * @param seatMicIndex 麦位
     */
    private static void updatePersonList(BottomSheetDialog personListDialog,int seatMicIndex){

        RoomManager.getInstance().applyRoomInfoServices(

                services -> {

                    //查询房间用户列表，返回所有用户信息
                    ZIMSDKManager.getInstance().queryRoomMemberList(services.getRoomID(), new IZIMQueryRoomMemberListCallback() {
                        @Override
                        public void onRoomMemberQueried(ArrayList<ZIMUserFullInfo> userList, String nextFlag, int errorCode) {


                            RoomPersonListAdapter adapter = new RoomPersonListAdapter();
                            List<User> userInfoList = new ArrayList<>();
                            //更新用户服务中的用户基本信息
                            RoomManager.getInstance().applyUserService(
                                    services1 -> {
                                        if(userList != null){
                                            for(ZIMUserFullInfo userFullInfo : userList){

                                                User user = services1.getUser(userFullInfo.baseInfo.userID);

                                                if(user == null){
                                                    user = new User();
                                                    user.setId(userFullInfo.baseInfo.userID);
                                                    user.setName(userFullInfo.baseInfo.userName);
                                                    services1.putUser(userFullInfo.baseInfo.userID,user);
                                                }
                                                user.setUserIcon(userFullInfo.userAvatarUrl);

                                                userInfoList.add(user);
                                            }
                                        }

                                    }
                            );

                            adapter.setUserBaseInfoList(userInfoList);
                            adapter.setOnClickListener(new RoomPersonListAdapter.OnClickListener() {
                                @Override
                                public void invite(View v, int position, User user) {
                                    inviteUser(personListDialog,seatMicIndex,user.getId(),user.getName());
                                }
                            });

                            if(errorCode == 0){
                                RecyclerView personList = (RecyclerView)personListDialog.findViewById(R.id.rv_person_list);
                                if(personList != null) {
                                    personList.setLayoutManager(new LinearLayoutManager(personList.getContext()));
                                    personList.setAdapter(adapter);
                                }

                            }else{
                                T.show(R.string.chathouse_net_error_please_try_again);
                                personListDialog.dismiss();
                            }


                        }
                    });
                }

        );

    }

    public static void inviteUser(BottomSheetDialog personListDialog,int seatMicIndex,String id,String name){
        RoomManager.getInstance().applyInviteService(services1 -> {
            services1.hostInvite(seatMicIndex, id, new IZIMCommonCallback() {
                @Override
                public void onZIMCallback(int errorCode) {
                    Context context = personListDialog.getContext();
                    if(context != null) {
                        if (errorCode == 0) {
                            T.show(String.format(context.getString(R.string.chathouse_already_invite_up_seat_please_wait_for_his_confirm), name));
                            personListDialog.dismiss();
                        } else if (errorCode == ERROR_INVITE_MIC_ALREADY_EXIST) {
                            T.show(context.getString(R.string.chathouse_already_invite_user_up_seat_please_wait_for_his_confirm));
                            personListDialog.dismiss();
                        } else if (errorCode == ERROR_USER_NO_EXIST) {
                            T.show(context.getString(R.string.chathouse_user_is_not_in_the_room));
                            personListDialog.dismiss();
                        } else {
                            T.show(String.format(context.getString(R.string.chathouse_invite_seat_fail_error_code), errorCode));
                        }
                    }
                }
            });
        });
    }



}



