package im.zego.chathouse.room.help;

import static im.zego.chathouse.Constants.QUERY_ONLINE_MEMBER_COUNT_INTERVAL;
import static im.zego.chathouse.ErrorCode.ERROR_FAIL_NETWORK;

import im.zego.chathouse.protocol.ICommonResponseCallback;
import im.zego.chathouse.protocol.TokenManager;
import im.zego.chathouse.room.RoomManager;
import im.zego.chathouse.sdk.zim.ZIMSDKManager;
import im.zego.chathouse.sdk.zim.callback.IZIMCommonCallback;

/**
 * 定时任务辅助类
 */
public class TimerTaskHelp {

    public static final int REFRESH_TOKEN = 430;
    public static final int ROOM_ATTRIBUTES = 431;
    public static final int ROOM_NUMBER_OF_PEOPLE = 432;

    /**
     * 刷新token。网络导致的失败会重试
     * @param callback 回调
     */
    public static void refreshToken(ICommonResponseCallback callback){

        Runnable runnable = new Runnable() {
            int count = -1;

            @Override
            public void run() {
                TokenManager.getInstance().getToken(new ICommonResponseCallback() {
                    @Override
                    public void onResponse(int errorCode) {

                        if(errorCode == ERROR_FAIL_NETWORK){
                            count++;
                            retryTask();
                        }

                        if(callback != null){
                            callback.onResponse(errorCode);
                        }
                    }
                });
            }


            public void retryTask(){
                RoomManager.getInstance().applyTimerTaskService(
                        services -> services.addTask(REFRESH_TOKEN , this, (long) (Math.pow(2,count) * 1000))
                );
            }
        };
        RoomManager.getInstance().applyTimerTaskService(
                services -> services.addTask(REFRESH_TOKEN,runnable,0)
        );
    }


    /**
     * 更新房间属性，如果失败，则会启动重试直到成功
     */
    public static void pushRoomAttributes(){


        Runnable runnable = new Runnable() {


            @Override
            public void run() {

                RoomManager.getInstance().applyRoomAttributesService(attributesService -> {


                    if(attributesService.isCanUpdateRoomAttributes()){
                        RoomManager.getInstance().applyRoomAttributesService(services ->
                                services.pushRoomAttributes(new IZIMCommonCallback() {
                                    @Override
                                    public void onZIMCallback(int errorCode) {
                                        if (errorCode == 0){
                                            RoomManager.getInstance().applyTimerTaskService(
                                                    services -> services.removeTask(ROOM_ATTRIBUTES)
                                            );
                                        }else {
                                            nextTask();
                                        }
                                    }
                                }));
                    }else {
                        nextTask();
                    }



                });


            }


            public void nextTask(){
                RoomManager.getInstance().applyTimerTaskService(
                        services -> {
                            services.removeTask(ROOM_ATTRIBUTES);
                            services.addTask(ROOM_ATTRIBUTES , this,  1000);
                        }
                );
            }
        };
        RoomManager.getInstance().applyTimerTaskService(
                services -> services.addTask(ROOM_ATTRIBUTES,runnable,0)
        );


    }


    /**
     * 开始查询房间人数的定时任务
     */
    public static void startQueryOnlineMemberCount(){
        RoomManager.getInstance().applyTimerTaskService(
                services -> services.removeTask(ROOM_NUMBER_OF_PEOPLE)
        );
        Runnable runnable = new Runnable() {


            @Override
            public void run() {
                RoomManager.getInstance().applyRoomInfoServices(services -> ZIMSDKManager.getInstance().queryRoomOnlineMemberCount(services.getRoomID()));
                nextTask();
            }


            public void nextTask(){
                RoomManager.getInstance().applyTimerTaskService(
                        services -> services.addTask(ROOM_NUMBER_OF_PEOPLE , this,  QUERY_ONLINE_MEMBER_COUNT_INTERVAL)
                );
            }
        };
        RoomManager.getInstance().applyTimerTaskService(
                services -> services.addTask(ROOM_NUMBER_OF_PEOPLE,runnable,0)
        );

    }

    /**
     * 停止查询房间人数的定时任务
     */
    public static void stopQueryOnlineMemberCount(){
        RoomManager.getInstance().applyTimerTaskService(
                services -> services.removeTask(ROOM_NUMBER_OF_PEOPLE)
        );

    }

}
