package im.zego.chathouse.room.service;

import java.util.LinkedList;
import java.util.Queue;

import im.zego.chathouse.model.DialogTask;

/**
 * 保存弹窗的消息对象队列
 */
public class DialogQueueService {

    private final DialogQueue mDialogQueue = new DialogQueue();
    private DialogListener mDialogListener;

    private DialogTask mDialogTask = null;

    public DialogQueueService() {
    }

    /**
     * 对象入队
     *
     * @param dialog 弹窗任务
     */
    public void addData(DialogTask dialog) {
        if (dialog != null) {
            mDialogQueue.offer(dialog);
        }
    }

    /**
     * 对象入队并显示弹窗
     *
     * @param dialog 弹窗
     */
    public void addDataAndShow(DialogTask dialog) {
        if (dialog != null) {
            if(canShow()){
                mDialogQueue.offer(dialog);
                showDialog();
            }else {
                mDialogQueue.offer(dialog);
            }

        }
    }

    /**
     * 处理弹窗是第一次过来的时候就需要调用 show()
     *
     * @return true 能否展示
     */
    public boolean canShow() {
        return mDialogQueue.canShow();
    }

    public void setInfoNull(){
        mDialogTask = null;
    }

    /**
     * 移除指定对象
     * @param callID 呼叫ID
     */
    public void remove(String callID){
        mDialogQueue.remove(callID);
    }

    /**
     * 移除指定对象
     * @param callID 呼叫ID
     */
    public void removeAndNotify(String callID){
        mDialogQueue.remove(callID);
        if(mDialogListener != null){
            mDialogListener.remove(callID);
        }
    }

    /**
     * 移除所有对象
     */
    public void removeAllNotify(){
        mDialogQueue.clear();
    }

    public void clear() {
        mDialogTask = null;
        setDialogListener(null);
        mDialogQueue.clear();
    }

    public void init(){
    }

    public void unInit(){
        clear();
    }

    public DialogTask getDialogTask() {
        return mDialogTask;
    }

    public DialogQueue getDialogQueue() {
        return mDialogQueue;
    }

    /**
     * 显示下一个弹窗
     */
    public void showNextDialog() {
        mDialogTask = null;
        showDialog();
    }


    /**
     * 弹窗显示
     */
    public void showDialog() {
        if (mDialogTask == null) {
            //从队列中拿出一个实例,并从列表中移除
            mDialogTask = mDialogQueue.poll();
            //当队列为空的时候拿出来的会是null
            if (mDialogTask != null) {
                if (mDialogListener != null) {
                    mDialogListener.onData(mDialogTask);
                }
            }
        }
    }

    public void setDialogListener(DialogListener dialogListener) {
        this.mDialogListener = dialogListener;
    }

    public interface DialogListener {
        void onData(DialogTask dialogTask);
        void remove(String callID);
    }

    public static class DialogQueue {

        private Queue<DialogTask> mDialogQueue = new LinkedList<>();

        /**
         * 进队
         *
         * @param dialogTask  弹窗任务
         */
        public void offer(DialogTask dialogTask) {
            mDialogQueue.offer(dialogTask);
        }

        /**
         * 出队
         *
         * @return 弹窗任务
         */
        public DialogTask poll() {
            return mDialogQueue.poll();
        }

        /**
         * 判断当前对列没有数据，那就入队并且show()
         *
         * @return 能否展示
         */
        public boolean canShow() {
            return mDialogQueue.size() == 0;
        }


        public void clear() {
            mDialogQueue.clear();
        }

        /**
         * 移除指定对象
         *
         * @param callID 呼叫ID
         */
        public void remove(String  callID) {
            for (DialogTask stateInfo : mDialogQueue) {
                if (callID != null && callID.equals(stateInfo.getCallID())) {
                    mDialogQueue.remove(stateInfo);
                }
            }
        }

        public Queue<DialogTask> getDialogQueue() {
            return mDialogQueue;
        }

        public void setDialogQueue(Queue<DialogTask> mDialogQueue) {
            this.mDialogQueue = mDialogQueue;
        }
    }


}

