package im.zego.chathouse.room.service;


import android.os.Handler;
import android.os.HandlerThread;
import android.os.Message;
import android.util.Log;

import androidx.annotation.NonNull;

import im.zego.chathouse.model.HeartbeatRespInfo;
import im.zego.chathouse.protocol.ICommonResponseCallback;
import im.zego.chathouse.protocol.IHeartBeatCallback;
import im.zego.chathouse.protocol.RoomAPI;


/**
 * 心跳服务
 * 固定时间会调用心跳接口
 */
public class HeartBeatService {

    private final String TAG = "HeartBeatUtil";

    private static final int HEART_BEAT_REQUEST = 666;

    private static final long DEFAULT_HEART_BEAT_TIME = 10L * 1000;
    private long mHeartBeatTime = DEFAULT_HEART_BEAT_TIME;

    private HandlerThread mThread;
    private Handler heartBeatHandler = null;
    private String mRoomID;

    public HeartBeatService(String roomID){
        mRoomID = roomID;
    }

    public boolean isInit = false;

    /**
     * 初始化，开启心跳
     */
    public void init(){
        if(isInit){
            return;
        }
        isInit = true;

        mThread = new HandlerThread("HeartBeatUtil");
        mThread.start();

        heartBeatHandler = new Handler(mThread.getLooper()){
            @Override
            public void handleMessage(@NonNull Message msg) {
                super.handleMessage(msg);

                if(msg.what == HEART_BEAT_REQUEST){
                    Log.e(TAG,"HEART_BEAT_REQUEST:");
                    heartBeatRequest(new ICommonResponseCallback() {
                        @Override
                        public void onResponse(int errorCode) {
                            request(mHeartBeatTime);
                        }
                    });

                }
            }
        };
        request(0);
    }

    /**
     * 反初始化，停止心跳
     */
    public void unInit(){

        if(!isInit){
            return;
        }
        isInit = false;

        if(mThread != null) {
            mThread.quit();
        }
        mHeartBeatTime = DEFAULT_HEART_BEAT_TIME;
        heartBeatHandler = null;
        mRoomID =null;
    }

    /**
     * 心跳请求
     * @param commonResponseCallback
     */
    private void heartBeatRequest(ICommonResponseCallback commonResponseCallback){

        RoomAPI.heartBeat(mRoomID, new IHeartBeatCallback() {
            @Override
            public void onHeartBeat(int errorCode, HeartbeatRespInfo heartbeatRespInfo) {
                Log.e(TAG,"errorCode:" + errorCode);
                if(errorCode == 0){
                    int interval = heartbeatRespInfo.getInterval() > 5 ? heartbeatRespInfo.getInterval() - 5 : heartbeatRespInfo.getInterval();
                    reset(interval * 1000L);
                }
                commonResponseCallback.onResponse(errorCode);
            }
        });

    }

    private void reset(long heartBeatTime){
        mHeartBeatTime = heartBeatTime;
        if(heartBeatHandler != null){
            heartBeatHandler.removeMessages(HEART_BEAT_REQUEST);

        }
    }

    /**
     * 延迟N毫秒请求心跳
     * @param time
     */
    private void request(long time){
        Log.e(TAG,"request:" + time);
        if(heartBeatHandler != null) {
            heartBeatHandler.sendEmptyMessageDelayed(HEART_BEAT_REQUEST, time);
        }
    }


}
