package im.zego.chathouse.room.service;

import static im.zego.chathouse.Constants.INVITE_CALL_TIME_OUT;
import static im.zego.chathouse.Constants.MESSAGE_CMD_CALL_INVITE_EXTENDED_DATA;
import static im.zego.chathouse.ErrorCode.ERROR_INVITE_MIC_ALREADY_EXIST;
import static im.zego.chathouse.ErrorCode.ERROR_MIC_NOT_EXIST_INVITE;
import static im.zego.chathouse.ErrorCode.ERROR_USER_NO_EXIST;
import static im.zego.chathouse.sdk.zim.ZIMEventHandlerWrap.DATA_PRIORITY;

import android.os.SystemClock;

import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import im.zego.chathouse.R;
import im.zego.chathouse.manager.UserManager;
import im.zego.chathouse.model.CallData;
import im.zego.chathouse.model.DialogTask;
import im.zego.chathouse.model.InviteExtendedData;
import im.zego.chathouse.model.Seat;
import im.zego.chathouse.model.User;
import im.zego.chathouse.protocol.APIBase;
import im.zego.chathouse.room.RoomManager;
import im.zego.chathouse.sdk.zim.ZIMEventHandlerWrap;
import im.zego.chathouse.sdk.zim.ZIMSDKManager;
import im.zego.chathouse.sdk.zim.callback.IZIMCommonCallback;
import im.zego.enjoycommon.log.L;
import im.zego.enjoycommon.util.MiscUtils;
import im.zego.enjoycommon.util.T;
import im.zego.zim.ZIM;
import im.zego.zim.callback.ZIMCallAcceptanceSentCallback;
import im.zego.zim.callback.ZIMCallCancelSentCallback;
import im.zego.zim.callback.ZIMCallInvitationSentCallback;
import im.zego.zim.callback.ZIMCallRejectionSentCallback;
import im.zego.zim.entity.ZIMCallInvitationAcceptedInfo;
import im.zego.zim.entity.ZIMCallInvitationCancelledInfo;
import im.zego.zim.entity.ZIMCallInvitationReceivedInfo;
import im.zego.zim.entity.ZIMCallInvitationRejectedInfo;
import im.zego.zim.entity.ZIMCallInvitationSentInfo;
import im.zego.zim.entity.ZIMError;
import im.zego.zim.entity.ZIMUserInfo;

/**
 * 邀请服务
 * 一个麦位最多只能对应一个邀请信息
 */
public class InviteService {

    private static final String TAG = "InviteService";

    public static final int MAX_TIME_OUT = INVITE_CALL_TIME_OUT;

    private final String mRoomID;
    private final ZIMEventHandlerWrap mZimEventHandlerWrap;

    //记录麦位对应的邀请，一个麦位最多只能对应一个邀请
    private final Map<Integer, InviteExtendedData> inviteSeatMap = new HashMap<>();

    public InviteService(String roomID) {
        mRoomID = roomID;
        mZimEventHandlerWrap = new ZIMEventHandlerWrap(mRoomID, DATA_PRIORITY) {
            @Override
            public void onCallInvitationReceived(ZIM zim, ZIMCallInvitationReceivedInfo info, String callID) {
                super.onCallInvitationReceived(zim, info, callID);

                try {
                    JSONObject jsonObject = new JSONObject(info.extendedData);
                    final int cmd = jsonObject.optInt("cmd");
                    final String roomID = jsonObject.optString("room_id");
                    String dataJson = "";
                    // 当data为null时会抛出异常
                    try {
                        dataJson = jsonObject.getJSONObject("data").toString();
                    } catch (Exception ignored) {

                    }

                    if(cmd == MESSAGE_CMD_CALL_INVITE_EXTENDED_DATA && RoomManager.getInstance().getCurrentRoomID().equals(roomID)) {

                        InviteExtendedData data = APIBase.getGson().fromJson(dataJson, InviteExtendedData.class);
                        if (data != null) {
                            data.callID = callID;
                            if (data.seatIndex != 0) {

                                //房主接受消息，表示是观众主动申请上麦
                                if (RoomManager.getInstance().isHost()) {
                                    InviteExtendedData inviteExtendedData = inviteSeatMap.get(data.seatIndex);
                                    //如果当前麦位已存在邀请，则直接拒绝
                                    if (inviteExtendedData != null) {

                                        rejectInvite(data.callID,data.inviteeID,data.inviterID, errorCode -> {

                                        });
                                    } else {

                                        //如果已经在其他麦位上被邀请或者申请则直接拒绝
                                        if (isAlreadyApplySeat(data.inviterID)) {
                                            rejectInvite(data.callID,data.inviteeID,data.inviterID,  errorCode -> {

                                            });
                                        } else {

                                            RoomManager.getInstance().applyRoomAttributesService(roomAttributesService -> {
                                                Seat seat = roomAttributesService.getRoomAttributes().getSeat(data.seatIndex);
                                                if (seat != null && seat.getStatus() != Seat.SeatStatus.CLOSED.status) {

                                                    putSeat(data.seatIndex, data);

                                                    //提交请求弹窗到队列
                                                    RoomManager.getInstance().applyDialogQueueService(services -> {
                                                        DialogTask dialogTask = new DialogTask();
                                                        dialogTask.setCallID(data.callID);
                                                        dialogTask.setData(data);
                                                        dialogTask.setTime(SystemClock.elapsedRealtime());
                                                        services.addDataAndShow(dialogTask);
                                                    });


                                                    //将该用户状态改为请求中
                                                    RoomManager.getInstance().applyUserService(services -> services.setUserStatus(data.inviterID, User.Status.INVITING.status));

                                                    //修改房间属性为请求中
                                                    RoomManager.getInstance().applyRoomAttributesService(services -> services.invitingSeat(data.seatIndex, data.inviterID));


                                                } else {
                                                    //如果麦位是关闭状态，则不能申请，直接拒绝
                                                    rejectInvite(data.callID,data.inviteeID,data.inviterID, errorCode -> {

                                                    });
                                                }
                                            });


                                        }


                                    }
                                } else {
                                    //主播邀请观众，观众接受到消息。执行保存麦位信息
                                    putSeat(data.seatIndex, data);
                                }


                            }
                        }
                    }

                } catch (Exception ignored) {

                }
            }

            @Override
            public void onCallInvitationCancelled(ZIM zim, ZIMCallInvitationCancelledInfo info, String callID) {
                super.onCallInvitationCancelled(zim, info, callID);

                try {
                    JSONObject jsonObject = new JSONObject(info.extendedData);
                    final int cmd = jsonObject.optInt("cmd");
                    final String roomID = jsonObject.optString("room_id");

                    String dataJson = "";
                    // 当data为null时会抛出异常
                    try {
                        dataJson = jsonObject.getJSONObject("data").toString();
                    } catch (Exception ignored) {

                    }

                    if(cmd == MESSAGE_CMD_CALL_INVITE_EXTENDED_DATA && RoomManager.getInstance().getCurrentRoomID().equals(roomID)) {
                        InviteExtendedData data = APIBase.getGson().fromJson(dataJson, InviteExtendedData.class);
                        if (data != null && data.seatIndex != 0) {
                            removeSeat(data.seatIndex);
                            //如果是房主，表示事件是观众主动申请上麦后点击取消申请
                            if (RoomManager.getInstance().isHost()) {
                                //设置麦位状态为空闲
                                RoomManager.getInstance().applyRoomAttributesService(services -> services.downSeat(data.seatIndex));
                                //移除房主选择是否同意的弹窗
                                RoomManager.getInstance().applyDialogQueueService(services -> services.removeAndNotify(callID));
                            }
                        }
                    }
                } catch (Exception ignored) {

                }
            }

            @Override
            public void onCallInvitationAccepted(ZIM zim, ZIMCallInvitationAcceptedInfo info, String callID) {
                super.onCallInvitationAccepted(zim, info, callID);

                try {
                    JSONObject jsonObject = new JSONObject(info.extendedData);
                    final int cmd = jsonObject.optInt("cmd");
                    final String roomID = jsonObject.optString("room_id");

                    String dataJson = "";
                    // 当data为null时会抛出异常
                    try {
                        dataJson = jsonObject.getJSONObject("data").toString();
                    } catch (Exception ignored) {

                    }

                    if(cmd == MESSAGE_CMD_CALL_INVITE_EXTENDED_DATA && RoomManager.getInstance().getCurrentRoomID().equals(roomID)) {

                        InviteExtendedData data = APIBase.getGson().fromJson(dataJson, InviteExtendedData.class);
                        if (data != null) {
                            //如果是房主，则说明是房主主动邀请，对方接受，直接进行上麦
                            if (RoomManager.getInstance().isHost()) {

                                RoomManager.getInstance().applyRoomAttributesService(roomAttributesService -> {
                                    Seat seat = roomAttributesService.getRoomAttributes().getSeat(data.seatIndex);
                                    if (seat != null && seat.getStatus() != Seat.SeatStatus.CLOSED.status) {
                                        //麦位未关闭，则上麦
                                        RoomManager.getInstance().applyRoomAttributesService(services -> services.upSeat(data.seatIndex, data.inviteeID));
                                    } else {
                                        //如果关闭状态，且非房主麦位，则清除邀请状态
                                        if (data.seatIndex != 0) {
                                            removeSeat(data.seatIndex);
                                        }

                                    }
                                });

                            }
                        }
                    }

                } catch (Exception ignored) {

                }
            }

            @Override
            public void onCallInvitationRejected(ZIM zim, ZIMCallInvitationRejectedInfo info, String callID) {
                super.onCallInvitationRejected(zim, info, callID);

                try {
                    JSONObject jsonObject = new JSONObject(info.extendedData);
                    final int cmd = jsonObject.optInt("cmd");
                    final String roomID = jsonObject.optString("room_id");

                    String dataJson = "";
                    // 当data为null时会抛出异常
                    try {
                        dataJson = jsonObject.getJSONObject("data").toString();
                    } catch (Exception ignored) {

                    }

                    if(cmd == MESSAGE_CMD_CALL_INVITE_EXTENDED_DATA && RoomManager.getInstance().getCurrentRoomID().equals(roomID)) {
                        InviteExtendedData data = APIBase.getGson().fromJson(dataJson, InviteExtendedData.class);
                        if (data != null && data.seatIndex != 0) {
                            removeSeat(data.seatIndex);
                            //如果是房主，则说明是房主主动邀请观众上麦，被观众拒绝
                            if (RoomManager.getInstance().isHost()) {
                                RoomManager.getInstance().applyUserService(services -> {
                                    User user = services.getUser(data.inviteeID);
                                    if (user != null && user.getName() != null) {
                                        T.show(String.format(MiscUtils.getApplication().getString(R.string.chathouse_rejected_invite_seat), user.getName()));
                                    } else {
                                        T.show(R.string.chathouse_user_rejected_invite_seat);
                                    }
                                });

                                RoomManager.getInstance().applyRoomAttributesService(services -> services.downSeat(data.seatIndex));
                            } else {
                                //非房主，则是自己申请上麦，被主播拒绝
                                T.show(R.string.chathouse_anchor_rejected_your_apply_seat);
                            }
                        }
                    }
                } catch (Exception ignored) {

                }
            }

            @Override
            public void onCallInvitationTimeout(ZIM zim, String callID) {
                super.onCallInvitationTimeout(zim, callID);
                Iterator<Map.Entry<Integer, InviteExtendedData>> iterator = inviteSeatMap.entrySet().iterator();
                while (iterator.hasNext()) {
                    Map.Entry<Integer, InviteExtendedData> next = iterator.next();
                    InviteExtendedData value = next.getValue();
                    if (value != null && value.callID != null && value.callID.equals(callID)) {
                        //呼叫超时，则清除邀请信息和用户状态信息
                        iterator.remove();
                        if(RoomManager.getInstance().isHost()) {
                            innerHostRemoveSeat(value.inviterID);
                        }else {
                            innerHostRemoveSeat(value.inviteeID);
                        }
                        //如果是房主，则说明是别人申请上麦，房主在时间内未确认，直接修改麦位状态并移除申请弹窗
                        if (RoomManager.getInstance().isHost()) {

                            RoomManager.getInstance().applyRoomAttributesService(services -> services.downSeat(value.seatIndex));

                            RoomManager.getInstance().applyDialogQueueService(services -> services.removeAndNotify(callID));
                        }
                    }
                }
            }

            @Override
            public void onCallInviteesAnsweredTimeout(ZIM zim, ArrayList<String> invitees, String callID) {
                super.onCallInviteesAnsweredTimeout(zim, invitees, callID);
                Iterator<Map.Entry<Integer, InviteExtendedData>> iterator = inviteSeatMap.entrySet().iterator();
                while (iterator.hasNext()) {
                    Map.Entry<Integer, InviteExtendedData> next = iterator.next();
                    InviteExtendedData value = next.getValue();
                    if (value != null && value.callID != null && value.callID.equals(callID)) {
                        //应答超时，则清除邀请信息和用户状态信息
                        iterator.remove();

                        //如果是房主，则说明是房主主动邀请观众上麦，观众在时间内未确认
                        if (RoomManager.getInstance().isHost()) {
                            innerHostRemoveSeat(value.inviteeID);
                            T.show(R.string.chathouse_invite_seat_fail_please_try_again);
                            RoomManager.getInstance().applyRoomAttributesService(services -> services.downSeat(value.seatIndex));
                        } else {
                            innerHostRemoveSeat(value.inviterID);
                            T.show(R.string.chathouse_timeout_is_not_confirm);
                        }
                        return;
                    }
                }

                if(RoomManager.getInstance().isHost()){
                    T.show(R.string.chathouse_invite_seat_fail_please_try_again);
                }
            }

            @Override
            public void onRoomMemberLeft(ZIM zim, ArrayList<ZIMUserInfo> memberList, String roomID) {
                super.onRoomMemberLeft(zim, memberList, roomID);
                //如果该用户退出房间则要清除麦位记录信息
                if (memberList != null) {
                    for (ZIMUserInfo zimUserInfo : memberList) {
                        Iterator<Map.Entry<Integer, InviteExtendedData>> iterator = inviteSeatMap.entrySet().iterator();
                        while (iterator.hasNext()) {
                            Map.Entry<Integer, InviteExtendedData> next = iterator.next();
                            InviteExtendedData value = next.getValue();
                            if (zimUserInfo.userID.equals(value.inviterID) || zimUserInfo.userID.equals(value.inviteeID)) {
                                //用户退出移除麦位信息
                                L.i(TAG,"onRoomMemberLeft:" +  zimUserInfo.userID);
                                iterator.remove();

                                //用户退房移除弹窗（目前产品逻辑不做）
//                                if(RoomManager.getInstance().isHost()){
//                                    RoomManager.getInstance().applyDialogQueueService(services -> {
//                                        services.removeAndNotify(value.callID);
//                                    });
//                                }
                            }

                        }

                    }
                }
            }
        };
    }

    /**
     * 房主发起邀请
     *
     * @param position 麦位
     * @param inviteUserID 邀请ID
     * @param callback 回调
     */
    public void hostInvite(int position, String inviteUserID, IZIMCommonCallback callback) {

        //该麦位已经存在邀请，则不可发起邀请
        InviteExtendedData data = inviteSeatMap.get(position);
        if (data != null) {
            callback.onZIMCallback(ERROR_INVITE_MIC_ALREADY_EXIST);
            return;
        }

        //用户如果不在房间服务内，则不可发起邀请
        UserService userService = RoomManager.getInstance().getUserService();
        if (userService != null) {
            if (userService.getUser(inviteUserID) == null) {
                callback.onZIMCallback(ERROR_USER_NO_EXIST);
                return;
            }
        }

        InviteExtendedData extendedData = new InviteExtendedData();
        extendedData.status = InviteExtendedData.InvitationStatus.INVITATION_STATUS_NOT_DETERMINED.status;
        extendedData.inviteeID = inviteUserID;
        extendedData.inviterID = UserManager.getInstance().getUserID();
        extendedData.seatIndex = position;

        //保存当前麦位的邀请
        putSeat(position, extendedData);

        CallData<InviteExtendedData> callData = new CallData<>();
        callData.cmd = MESSAGE_CMD_CALL_INVITE_EXTENDED_DATA;
        callData.roomID = RoomManager.getInstance().getCurrentRoomID();
        callData.data = extendedData;
        ZIMSDKManager.getInstance().callInvite(inviteUserID, APIBase.getGson().toJson(callData), MAX_TIME_OUT, new ZIMCallInvitationSentCallback() {
            @Override
            public void onCallInvitationSent(String callID, ZIMCallInvitationSentInfo info, ZIMError errorInfo) {

                if (errorInfo.code.value() == 0) {

                    InviteExtendedData inviteExtendedData = inviteSeatMap.get(position);
                    if (inviteExtendedData != null) {
                        inviteExtendedData.callID = callID;
                    }

                    //将该用户状态改为请求中
                    RoomManager.getInstance().applyUserService(services -> services.setUserStatus(inviteUserID, User.Status.INVITING.status));

                    //修改房间属性为请求中
                    RoomManager.getInstance().applyRoomAttributesService(services -> services.invitingSeat(position, inviteUserID));


                } else {
                    //失败移除邀请
                    removeSeat(position);
                }


                if (callback != null) {
                    callback.onZIMCallback(errorInfo.code.value());
                }
            }
        });
    }

    /**
     * 获取邀请信息
     * @param position 麦位
     * @return 邀请附加信息
     */
    public InviteExtendedData getInviteData(int position) {
        return inviteSeatMap.get(position);

    }

    /**
     * 提交邀请到对应的麦位
     *
     * @param position 麦位
     * @param extendedData 附加信息
     */
    private void putSeat(int position, InviteExtendedData extendedData) {
        inviteSeatMap.put(position, extendedData);
    }

    /**
     * 移除麦位
     *
     * @param position 麦位
     */
    private void removeSeat(int position) {
        L.i(TAG,"removeSeat position: " + position);
        InviteExtendedData data = inviteSeatMap.remove(position);
        innerHostRemoveSeat(data);

    }

    /**
     * 房主移除麦位邀请时，设置用户信息为空闲中
     *
     * @param data 附加信息
     */
    private void innerHostRemoveSeat(InviteExtendedData data) {
        //如果是房主则设置状态为空闲
        if (data != null && RoomManager.getInstance().isHost()) {
            if (UserManager.getInstance().getUserID().equals(data.inviterID)) {
                RoomManager.getInstance().applyUserService(services -> services.setUserStatus(data.inviteeID, User.Status.VACANT.status));
            }else {
                RoomManager.getInstance().applyUserService(services -> services.setUserStatus(data.inviterID, User.Status.VACANT.status));
            }
        }
    }

    /**
     * 房主移除麦位邀请时，设置用户信息为空闲中
     *
     * @param uid 用户ID
     */
    private void innerHostRemoveSeat(String uid) {
        //如果是房主则设置状态为空闲
        if (uid != null && RoomManager.getInstance().isHost()) {
            RoomManager.getInstance().applyUserService(services -> services.setUserStatus(uid, User.Status.VACANT.status));
        }
    }

    /**
     * 观众是否能够申请麦位
     *
     * @return true 能够申请
     */
    public boolean canApplySeat() {

        if (RoomManager.getInstance().isHost()) {
            return false;
        }

        for (Map.Entry<Integer, InviteExtendedData> next : inviteSeatMap.entrySet()) {
            InviteExtendedData value = next.getValue();
            //没有申请,也没有被房主邀请
            if (value != null && ((UserManager.getInstance().getUserID().equals(value.inviteeID)) ||
                    (UserManager.getInstance().getUserID().equals(value.inviterID)))) {
                return false;
            }
        }
        return true;
    }

    /**
     * 是否已经申请麦位或被邀请
     *
     * @return true 已经申请麦位或被邀请
     */
    public boolean isAlreadyApplySeat(String userID) {

        if (userID == null) {
            return false;
        }

        for (Map.Entry<Integer, InviteExtendedData> next : inviteSeatMap.entrySet()) {
            InviteExtendedData value = next.getValue();
            if (value != null && ((userID.equals(value.inviteeID)) ||
                    (userID.equals(value.inviterID)))) {
                return true;
            }
        }
        return false;
    }


    /**
     * 观众申请麦位
     */
    public void audienceApplySeat(int position, String inviteUserID, IZIMCommonCallback callback) {
        InviteExtendedData data = inviteSeatMap.get(position);
        if (data != null) {
            callback.onZIMCallback(ERROR_INVITE_MIC_ALREADY_EXIST);
            return;
        }

        InviteExtendedData extendedData = new InviteExtendedData();
        extendedData.status = InviteExtendedData.InvitationStatus.INVITATION_STATUS_NOT_DETERMINED.status;
        extendedData.inviteeID = inviteUserID;
        extendedData.inviterID = UserManager.getInstance().getUserID();
        extendedData.seatIndex = position;

        putSeat(position, extendedData);

        CallData<InviteExtendedData> callData = new CallData<>();
        callData.cmd = MESSAGE_CMD_CALL_INVITE_EXTENDED_DATA;
        callData.roomID = RoomManager.getInstance().getCurrentRoomID();
        callData.data = extendedData;
        ZIMSDKManager.getInstance().callInvite(inviteUserID, APIBase.getGson().toJson(callData), MAX_TIME_OUT, new ZIMCallInvitationSentCallback() {
            @Override
            public void onCallInvitationSent(String callID, ZIMCallInvitationSentInfo info, ZIMError errorInfo) {

                if (errorInfo.code.value() == 0) {

                    InviteExtendedData inviteExtendedData = inviteSeatMap.get(position);
                    if (inviteExtendedData != null) {
                        inviteExtendedData.callID = callID;
                    }

                } else {
                    removeSeat(position);
                    T.show(R.string.chathouse_net_error_please_try_again);
                }


                if (callback != null) {
                    callback.onZIMCallback(errorInfo.code.value());
                }
            }
        });
    }

    /**
     * 接受邀请
     *
     * @param position 麦位
     */
    public void acceptedInvite(int position,String callID, IZIMCommonCallback callback) {

        InviteExtendedData data = inviteSeatMap.get(position);
        if (data ==  null || callID == null || !callID.equals(data.callID)) {
            callback.onZIMCallback(ERROR_MIC_NOT_EXIST_INVITE);
            return;
        }

        data.status = InviteExtendedData.InvitationStatus.INVITATION_STATUS_ACCEPTED.status;
        CallData<InviteExtendedData> callData = new CallData<>();
        callData.cmd = MESSAGE_CMD_CALL_INVITE_EXTENDED_DATA;
        callData.roomID = RoomManager.getInstance().getCurrentRoomID();
        callData.data = data;
        ZIMSDKManager.getInstance().callAccept(data.callID, APIBase.getGson().toJson(callData), new ZIMCallAcceptanceSentCallback() {
            @Override
            public void onCallAcceptanceSent(String callID, ZIMError errorInfo) {
                if (callback != null) {
                    callback.onZIMCallback(errorInfo.code.value());
                }
                L.i(TAG,"acceptedInvite onCallAcceptanceSent callID =" + callID + " :  errorInfo = "  + errorInfo.code.value());
            }
        });

        //如果是别人请求房主上麦，则直接修改房主属性
        if (RoomManager.getInstance().isHost()) {
            RoomManager.getInstance().applyRoomAttributesService(services -> services.upSeat(data.seatIndex, data.inviterID));
        }
    }

    /**
     * 拒绝邀请
     *
     * @param position 麦位
     * @param callback 回调
     */
    public void rejectInvite(int position,String callID, IZIMCommonCallback callback) {
        InviteExtendedData data = inviteSeatMap.get(position);
        if (data == null || callID == null || !callID.equals(data.callID)) {
            callback.onZIMCallback(ERROR_MIC_NOT_EXIST_INVITE);
            return;
        }
        data.status = InviteExtendedData.InvitationStatus.INVITATION_STATUS_REJECTED.status;

        // 房主直接移除麦位邀请并更新用户信息
        if (RoomManager.getInstance().isHost()) {
            if (data.seatIndex != 0) {
                removeInvite(data.seatIndex,data.inviterID);
            }

            //修改房间属性为空闲
            RoomManager.getInstance().applyRoomAttributesService(services -> services.downSeat(data.seatIndex));
        }
        CallData<InviteExtendedData> callData = new CallData<>();
        callData.cmd = MESSAGE_CMD_CALL_INVITE_EXTENDED_DATA;
        callData.roomID = RoomManager.getInstance().getCurrentRoomID();
        callData.data = data;
        ZIMSDKManager.getInstance().callReject(data.callID, APIBase.getGson().toJson(callData), new ZIMCallRejectionSentCallback() {
            @Override
            public void onCallRejectionSent(String callID, ZIMError errorInfo) {

                //拒绝需要移除当前麦位邀请信息，表示邀请流程结束，改麦位为空闲状态
                if (errorInfo.code.value() == 0) {
                    if (data.seatIndex != 0) {
                        removeSeat(data.seatIndex);
                    }
                }

                if (callback != null) {
                    callback.onZIMCallback(errorInfo.code.value());
                }

                L.i(TAG,"rejectInvite onCallRejectionSent callID =" + callID + " :  errorInfo = "  + errorInfo.code.value());
            }
        });
    }

    /**
     * 拒绝邀请
     *
     * @param callback 回调
     */
    public void rejectInvite(String call,String inviteeID,String inviterID, IZIMCommonCallback callback) {
        InviteExtendedData data = new InviteExtendedData();
        data.status = InviteExtendedData.InvitationStatus.INVITATION_STATUS_REJECTED.status;
        data.callID = call;
        data.inviteeID = inviteeID;
        data.inviterID = inviterID;
        CallData<InviteExtendedData> callData = new CallData<>();
        callData.cmd = MESSAGE_CMD_CALL_INVITE_EXTENDED_DATA;
        callData.roomID = RoomManager.getInstance().getCurrentRoomID();
        callData.data = data;
        ZIMSDKManager.getInstance().callReject(data.callID, APIBase.getGson().toJson(callData), new ZIMCallRejectionSentCallback() {
            @Override
            public void onCallRejectionSent(String callID, ZIMError errorInfo) {

                //拒绝需要移除当前麦位邀请信息，表示邀请流程结束，改麦位为空闲状态
                if (errorInfo.code.value() == 0) {
                    if (data.seatIndex != 0) {
                        removeSeat(data.seatIndex);
                    }
                }

                if (callback != null) {
                    callback.onZIMCallback(errorInfo.code.value());
                }

                L.i(TAG,"rejectInvite onCallRejectionSent callID =" + callID + " :  errorInfo = "  + errorInfo.code.value());
            }
        });
    }

    /**
     * 取消邀请
     *
     * @param position 麦位
     * @param callback 回调
     */
    public void cancelInvite(int position,String callID, IZIMCommonCallback callback) {
        InviteExtendedData data = inviteSeatMap.get(position);
        if (data == null || callID == null || !callID.equals(data.callID)) {
            callback.onZIMCallback(ERROR_MIC_NOT_EXIST_INVITE);
            return;
        }
        data.status = InviteExtendedData.InvitationStatus.INVITATION_STATUS_CANCELLED.status;
        CallData<InviteExtendedData> callData = new CallData<>();
        callData.cmd = MESSAGE_CMD_CALL_INVITE_EXTENDED_DATA;
        callData.roomID = RoomManager.getInstance().getCurrentRoomID();
        callData.data = data;
        ZIMSDKManager.getInstance().callCancel(data.inviterID,data.callID,APIBase.getGson().toJson(callData), new ZIMCallCancelSentCallback() {
            @Override
            public void onCallCancelSent(String callID, ArrayList<String> errorInvitees, ZIMError errorInfo) {
                if (errorInfo.code.value() == 0) {
                    if (data.seatIndex != 0) {
                        removeSeat(data.seatIndex);
                        if (RoomManager.getInstance().isHost()) {
                            RoomManager.getInstance().applyRoomAttributesService(services -> services.downSeat(data.seatIndex));
                        }
                    }
                }


                if (callback != null) {
                    callback.onZIMCallback(errorInfo.code.value());
                }
                L.i(TAG,"cancelInvite onCallCancelSent callID =" + callID + " :  errorInfo = "  + errorInfo.code.value());
            }

        });
    }


    /**
     * 移除麦位信息并更新用户信息
     *
     * @param seatIndex 麦位
     * @param uid 用户ID
     */
    public void removeInvite(int seatIndex, String uid) {
        Iterator<Map.Entry<Integer, InviteExtendedData>> iterator = inviteSeatMap.entrySet().iterator();
        while (iterator.hasNext()) {
            Map.Entry<Integer, InviteExtendedData> next = iterator.next();
            InviteExtendedData value = next.getValue();
            if (value != null && value.seatIndex != 0 && value.seatIndex == seatIndex) {
                iterator.remove();
                innerHostRemoveSeat(uid);
            }
        }
    }

    /**
     * 移除麦位信息
     *
     * @param seatIndex 麦位
     */
    public void removeInvite(int seatIndex) {
        Iterator<Map.Entry<Integer, InviteExtendedData>> iterator = inviteSeatMap.entrySet().iterator();
        while (iterator.hasNext()) {
            Map.Entry<Integer, InviteExtendedData> next = iterator.next();
            InviteExtendedData value = next.getValue();
            if (value != null && value.seatIndex != 0 && value.seatIndex == seatIndex) {
                iterator.remove();
            }
        }
    }

    public void init() {
        ZIMSDKManager.getInstance().getDispatchService().registerZIMEventHandlerCallbacks(mZimEventHandlerWrap);
    }


    public void unInit() {
        ZIMSDKManager.getInstance().getDispatchService().unregisterZIMEventHandlerCallbacks(mZimEventHandlerWrap);
        inviteSeatMap.clear();
    }
}
