package im.zego.chathouse.room.service;

import static im.zego.chathouse.Constants.INVITE_CALL_TIME_OUT;
import static im.zego.chathouse.ErrorCode.ERROR_MIC_NOT_EXIST_INVITE;

import android.app.Activity;
import android.content.DialogInterface;
import android.os.SystemClock;
import android.view.View;

import com.google.android.material.bottomsheet.BottomSheetDialog;

import im.zego.chathouse.R;
import im.zego.chathouse.help.PermissionHelp;
import im.zego.chathouse.model.DialogTask;
import im.zego.chathouse.model.InviteExtendedData;
import im.zego.chathouse.model.User;
import im.zego.chathouse.room.RoomManager;
import im.zego.chathouse.room.custom.RoomAlreadyClosedDialog;
import im.zego.chathouse.room.custom.RoomApplyDialog;
import im.zego.chathouse.room.custom.RoomCommonDialog;
import im.zego.chathouse.room.custom.RoomInviteDialog;
import im.zego.chathouse.room.custom.RoomNetConnectDialog;
import im.zego.chathouse.room.help.OperationDialogHelp;
import im.zego.chathouse.room.help.PersonListDialogHelp;
import im.zego.chathouse.sdk.zim.callback.IZIMCommonCallback;
import im.zego.chathouse.utils.ViewUtil;
import im.zego.enjoycommon.util.T;

public class RoomDialogService {
    /**
     * 邀请连麦
     */
    private RoomInviteDialog mRoomInviteDialog;

    private BottomSheetDialog mPersonListDialog;


    private BottomSheetDialog mOperationDialog;

    private RoomApplyDialog mRoomApplyDialog;

    private RoomCommonDialog mRoomEndMicDialog;

    private RoomCommonDialog mCloseRoomDialog;

    private RoomAlreadyClosedDialog mRoomAlreadyClosedDialog;

    private RoomNetConnectDialog mRoomNetConnectingDialog;

    private RoomNetConnectDialog mRoomNetDisconnectedDialog;

    private final PersonListDialogHelp mPersonListDialogHelp = new PersonListDialogHelp();
    private final OperationDialogHelp mOperationDialogHelp = new OperationDialogHelp();

    /**
     * 显示邀请弹窗
     * @param context 上下文
     * @param data 附加信息
     * @param onClickRejectListener 拒绝点击监听
     * @param onClickAgreeListener 同意点击监听
     */
    public void showRoomInviteDialog(Activity context, InviteExtendedData data, DialogInterface.OnClickListener onClickRejectListener, DialogInterface.OnClickListener onClickAgreeListener){
        if(context == null || context.isFinishing() || context.isDestroyed() || data == null){
            return;
        }

        if(mRoomInviteDialog != null && mRoomInviteDialog.isShowing()){
            mRoomInviteDialog.dismiss();
        }

        mRoomInviteDialog = new RoomInviteDialog(context, false);

        mRoomInviteDialog.setAgreeListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {


                PermissionHelp.micPermission(ViewUtil.getActivity(v), new PermissionHelp.IPermissionSucceededListener() {
                    @Override
                    public void onPermissionSucceed() {
                        RoomManager.getInstance().applyInviteService(services -> {
                            services.acceptedInvite(data.seatIndex, data.callID,new IZIMCommonCallback() {
                                @Override
                                public void onZIMCallback(int errorCode) {
                                    if (errorCode == 0) {
                                        if (mRoomInviteDialog != null) {
                                            mRoomInviteDialog.dismiss();
                                        }
                                    }else if(errorCode == ERROR_MIC_NOT_EXIST_INVITE){
                                        T.show(context.getString(R.string.chathouse_up_seat_fail_please_try_again));
                                        if(mRoomApplyDialog != null) {
                                            mRoomApplyDialog.dismiss();
                                        }
                                    } else {
//                                        T.show(String.format(context.getString(R.string.chathouse_agree_apply_seat_failed_error_code), errorCode));
                                        T.show(context.getString(R.string.chathouse_up_seat_fail_please_try_again));
                                    }
                                }
                            });
                        });

                        if(onClickAgreeListener != null){
                            onClickAgreeListener.onClick(mRoomInviteDialog,v.getId());
                        }
                    }
                });

            }
        });
        mRoomInviteDialog.setRejectListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                RoomManager.getInstance().applyInviteService(services -> {
                    services.rejectInvite(data.seatIndex,  data.callID, new IZIMCommonCallback() {
                        @Override
                        public void onZIMCallback(int errorCode) {
                            if(errorCode == 0){
                                if(mRoomInviteDialog != null) {
                                    mRoomInviteDialog.dismiss();
                                }
                            }else if(errorCode == ERROR_MIC_NOT_EXIST_INVITE){
                                T.show(context.getString(R.string.chathouse_up_seat_fail_please_try_again));
                                if(mRoomApplyDialog != null) {
                                    mRoomApplyDialog.dismiss();
                                }
                            } else {
                                T.show(context.getString(R.string.chathouse_up_seat_fail_please_try_again));
//                                T.show(String.format(context.getString(R.string.chathouse_reject_apply_seat_failed_error_code), errorCode));
                            }
                        }
                    });
                });

                if(onClickRejectListener != null){
                    onClickRejectListener.onClick(mRoomInviteDialog,v.getId());
                }
            }
        });
    }


    public void dismissRoomInviteDialog(){
        if(mRoomInviteDialog != null && mRoomInviteDialog.isShowing()){
            mRoomInviteDialog.dismiss();
        }
    }

    /**
     * 显示用户列表弹窗
     * @param activity 上下文
     * @param seatMicIndex 麦位
     */
    public void showPersonListDialog(Activity activity, int seatMicIndex){
        mPersonListDialog = mPersonListDialogHelp.showPersonListDialog(activity,seatMicIndex);
    }

    /**
     * 显示操作弹窗
     * @param activity 上下文
     * @param seatMicIndex 麦位
     * @param canSpeak 能否说话
     */
    public void showOperationDialog(Activity activity, int seatMicIndex,boolean canSpeak){
        mOperationDialog = mOperationDialogHelp.showOperationDialog(activity,seatMicIndex,canSpeak);
        //记录麦位位置，用于取消弹窗
        if(mOperationDialog != null){
            View view = mOperationDialog.findViewById(R.id.operation_cancel);
            if(view != null){
                view.setTag(seatMicIndex);
            }
        }
    }

    /**
     * 关闭操作弹窗
     * @param seatMicIndex 麦位
     */
    public void dismissOperationDialog(int seatMicIndex){
        if(mOperationDialog != null && mOperationDialog.isShowing()){
            View view = mOperationDialog.findViewById(R.id.operation_cancel);
            if(view != null){
                Object object = view.getTag();
                if(object != null && seatMicIndex == (int)object){
                    mOperationDialog.dismiss();
                }
            }
        }
    }

    /**
     * 展示房间申请弹窗
     * @param activity 上下文
     * @param dialogTask 弹窗队列
     * @param service 弹窗队列服务
     */
    public void showRoomApplyDialog(Activity activity,DialogTask dialogTask,DialogQueueService service){
        InviteExtendedData data = dialogTask.getData();
        long countDown = (SystemClock.elapsedRealtime() - dialogTask.getTime());
        if(countDown >= INVITE_CALL_TIME_OUT * 1000){
            service.showNextDialog();
            return;
        }
        if(data != null) {

            //收到申请弹窗时，关闭房间人员列表
            if(mPersonListDialog != null && mPersonListDialog.isShowing()){
                mPersonListDialog.dismiss();
            }

            mRoomApplyDialog = new RoomApplyDialog(activity, false);
            RoomManager.getInstance().applyUserService(services1 -> {
                User user = services1.getUser(data.inviterID);
                if(user != null){
                    mRoomApplyDialog.setMsgContent(user.getName());
                }
            });

            mRoomApplyDialog.setCancelListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    RoomManager.getInstance().applyInviteService(services1 -> {
                        services1.rejectInvite(data.seatIndex,data.callID, new IZIMCommonCallback() {
                            @Override
                            public void onZIMCallback(int errorCode) {
                                if(mRoomApplyDialog != null) {
                                    mRoomApplyDialog.dismiss();
                                }


                            }
                        });
                    });
                }
            });
            mRoomApplyDialog.setAcceptListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    RoomManager.getInstance().applyInviteService(services1 -> {
                        services1.acceptedInvite(data.seatIndex,data.callID, new IZIMCommonCallback() {
                            @Override
                            public void onZIMCallback(int errorCode) {
                                if(errorCode == 0){
                                    if(mRoomApplyDialog != null) {
                                        mRoomApplyDialog.dismiss();
                                    }
                                }else if(errorCode == ERROR_MIC_NOT_EXIST_INVITE){
                                    T.show(activity.getString(R.string.chathouse_user_already_left_room));
                                    if(mRoomApplyDialog != null) {
                                        mRoomApplyDialog.dismiss();
                                    }
                                } else {
//                                    T.show(String.format(activity.getString(R.string.chathouse_request_fail_error_code), errorCode));
                                    T.show(activity.getString(R.string.chathouse_up_seat_fail_please_try_again));
                                }
                            }
                        });
                    });
                }
            });
            mRoomApplyDialog.countDown(INVITE_CALL_TIME_OUT * 1000 - countDown);
            mRoomApplyDialog.setCallID(data.callID);
            mRoomApplyDialog.setOnDismissListener(new DialogInterface.OnDismissListener() {
                @Override
                public void onDismiss(DialogInterface dialog) {
                    service.showNextDialog();
                }
            });

        }
    }

    public void removeRoomApplyDialog(String callID){
        if(callID != null) {
            if (mRoomApplyDialog != null && callID.equals(mRoomApplyDialog.getCallID()) && mRoomApplyDialog.isShowing()) {
                mRoomApplyDialog.dismiss();
            }
        }
    }

    public void removeAllRoomApplyDialog(){

        if (mRoomApplyDialog != null && mRoomApplyDialog.isShowing()) {
            mRoomApplyDialog.dismiss();
        }
        RoomManager.getInstance().applyDialogQueueService(DialogQueueService::removeAllNotify);


    }

    /**
     * 展示结束连麦弹窗
     * @param activity 上下文
     * @param onClickCancelListener 点击取消回调
     * @param onClickConfirmListener 点击确定回调
     */
    public void showRoomEndMicDialog(Activity activity,DialogInterface.OnClickListener onClickCancelListener, DialogInterface.OnClickListener onClickConfirmListener){
        if(activity == null || activity.isFinishing() || activity.isDestroyed()){
            return;
        }

        if(mRoomEndMicDialog != null && mRoomEndMicDialog.isShowing()){
            mRoomEndMicDialog.dismiss();
        }

        mRoomEndMicDialog = new RoomCommonDialog(activity,false);
        mRoomEndMicDialog.setMsgTitle(activity.getString(R.string.chathouse_end_seat));
        mRoomEndMicDialog.setMsgContent(activity.getString(R.string.chathouse_whether_or_not_anchor_end_seat));
        mRoomEndMicDialog.setConfirmListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                mRoomEndMicDialog.dismiss();

                if(onClickConfirmListener != null){
                    onClickConfirmListener.onClick(mRoomEndMicDialog,v.getId());
                }
            }
        });
        mRoomEndMicDialog.setCancelListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                mRoomEndMicDialog.dismiss();

                if(onClickCancelListener != null){
                    onClickCancelListener.onClick(mRoomEndMicDialog,v.getId());
                }


            }
        });

    }

    /**
     * 展示关闭房间弹窗
     * @param activity 上下文
     * @param onClickCancelListener 点击取消弹窗
     * @param onClickConfirmListener 点击确认弹窗
     */
    public void showCloseRoomDialog(Activity activity,DialogInterface.OnClickListener onClickCancelListener, DialogInterface.OnClickListener onClickConfirmListener){
        if(activity == null || activity.isFinishing() || activity.isDestroyed()){
            return;
        }

        if(mCloseRoomDialog != null && mCloseRoomDialog.isShowing()){
            mCloseRoomDialog.dismiss();
        }

        mCloseRoomDialog = new RoomCommonDialog(activity,false);

        if(RoomManager.getInstance().isHost()) {
            mCloseRoomDialog.setMsgTitle(activity.getString(R.string.chathouse_end_live));
            mCloseRoomDialog.setMsgContent(activity.getString(R.string.chathouse_end_live_room_will_be_disbanded));
        }else {
            mCloseRoomDialog.setMsgTitle(activity.getString(R.string.chathouse_exit_room));
            mCloseRoomDialog.setMsgContent(activity.getString(R.string.chathouse_confirm_your_exit_no_more_talking));
        }
        mCloseRoomDialog.setConfirmListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {


                if(onClickConfirmListener != null){
                    onClickConfirmListener.onClick(mCloseRoomDialog,v.getId());
                }
            }
        });
        mCloseRoomDialog.setCancelListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                mCloseRoomDialog.dismiss();
                if(onClickCancelListener != null){
                    onClickCancelListener.onClick(mCloseRoomDialog,v.getId());
                }


            }
        });

    }

    /**
     * 展示房间已经关闭弹窗
     * @param activity 上下文
     * @param onClickConfirmListener 点击确认回调
     */
    public void showRoomAlreadyClosedDialog(Activity activity, DialogInterface.OnClickListener onClickConfirmListener){
        if(activity == null || activity.isFinishing() || activity.isDestroyed()){
            return;
        }

        if(mRoomAlreadyClosedDialog != null && mRoomAlreadyClosedDialog.isShowing()){
            return;
        }
        closeAllRoomBusinessDialog();
        mRoomAlreadyClosedDialog = new RoomAlreadyClosedDialog(activity,false);

        mRoomAlreadyClosedDialog.setConfirmListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                if(onClickConfirmListener != null){
                    onClickConfirmListener.onClick(mRoomAlreadyClosedDialog,v.getId());
                }
            }
        });

    }

    /**
     * 展示网络连接中弹窗
     * @param activity 上下文
     * @param onClickQuitListener 点击离开回调
     */
    public void showRoomNetConnectingDialog(Activity activity, DialogInterface.OnClickListener onClickQuitListener){
        if(activity == null || activity.isFinishing() || activity.isDestroyed()){
            return;
        }

        if(mRoomNetDisconnectedDialog != null && mRoomNetDisconnectedDialog.isShowing()){
            return;
        }

        if(mRoomNetConnectingDialog != null && mRoomNetConnectingDialog.isShowing()){
            mRoomNetConnectingDialog.dismiss();
        }

        mRoomNetConnectingDialog = new RoomNetConnectDialog(activity,false);

        mRoomNetConnectingDialog.setQuitListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                if(onClickQuitListener != null){
                    onClickQuitListener.onClick(mRoomNetConnectingDialog,v.getId());
                }
            }
        });

    }

    public void dismissRoomNetConnectingDialog(){
        if(mRoomNetConnectingDialog != null && mRoomNetConnectingDialog.isShowing()){
            mRoomNetConnectingDialog.dismiss();
        }
    }

    /**
     * 展示彻底断开网络连接
     * @param activity 上下文
     * @param onClickQuitListener 点击退出回调
     */
    public void showRoomNetDisConnectedDialog(Activity activity, DialogInterface.OnClickListener onClickQuitListener){
        if(activity == null || activity.isFinishing() || activity.isDestroyed()){
            return;
        }

        if(mRoomNetDisconnectedDialog != null && mRoomNetDisconnectedDialog.isShowing()){
            return;
        }

        if(mRoomAlreadyClosedDialog != null && mRoomAlreadyClosedDialog.isShowing()){
            return;
        }

        closeAllRoomBusinessDialog();
        mRoomNetDisconnectedDialog = new RoomNetConnectDialog(activity,false);
        mRoomNetDisconnectedDialog.setTitle(activity.getString(R.string.chathouse_net_connect_is_error_please_leave_the_room_and_try_again));
        mRoomNetDisconnectedDialog.setQuitListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                if(onClickQuitListener != null){
                    onClickQuitListener.onClick(mRoomNetDisconnectedDialog,v.getId());
                }
            }
        });

    }

    /**
     * 关闭房间内所有业务弹窗
     */
    private void closeAllRoomBusinessDialog(){
        if(mRoomInviteDialog != null){
            mRoomInviteDialog.dismiss();
        }

        if(mPersonListDialog != null){
            mPersonListDialog.dismiss();
        }


        if(mOperationDialog != null){
            mOperationDialog.dismiss();
        }

        removeAllRoomApplyDialog();

        if(mRoomEndMicDialog != null){
            mRoomEndMicDialog.dismiss();
        }

        if(mCloseRoomDialog != null){
            mCloseRoomDialog.dismiss();
        }
    }



    /**
     * 关闭所有弹窗,释放引用
     */
    public void closeAllDialog(){
        if(mRoomInviteDialog != null){
            mRoomInviteDialog.dismiss();
            mRoomInviteDialog = null;
        }

        if(mPersonListDialog != null){
            mPersonListDialog.dismiss();
            mPersonListDialog = null;
        }


        if(mOperationDialog != null){
            mOperationDialog.dismiss();
            mOperationDialog = null;
        }


        if(mRoomApplyDialog != null){
            mRoomApplyDialog.dismiss();
            mRoomApplyDialog = null;
        }


        if(mRoomEndMicDialog != null){
            mRoomEndMicDialog.dismiss();
            mRoomEndMicDialog = null;
        }

        if(mCloseRoomDialog != null){
            mCloseRoomDialog.dismiss();
            mCloseRoomDialog = null;
        }

        if(mRoomAlreadyClosedDialog != null){
            mRoomAlreadyClosedDialog.dismiss();
            mRoomAlreadyClosedDialog = null;
        }


        if(mRoomNetConnectingDialog != null){
            mRoomNetConnectingDialog.dismiss();
            mRoomNetConnectingDialog = null;
        }


        if(mRoomNetDisconnectedDialog != null){
            mRoomNetDisconnectedDialog.dismiss();
            mRoomNetDisconnectedDialog = null;
        }

    }

    public void init(){

    }

    public void unInit(){
        closeAllDialog();
    }

}
