package im.zego.chathouse.room.service;

import im.zego.chathouse.room.callback.IServicesNotNullCallback;

/**
 * 当前正处于的房间Service，一个房间ID，对应一个服务
 */
public class RunningRoomService {

    private final String mRoomID;

    private RoomInfoService mRoomInfoService;
    private HeartBeatService mHeartBeatService;
    private TimerTaskService mTimerTaskService;
    private RoomConnectService mRoomConnectService;
    private RoomAttributesService mRoomAttributesService;
    private UserService mUserService;
    private InviteService mInviteService;
    private DialogQueueService mDialogQueueService;
    private RoomDialogService mRoomDialogService;
    private RoomStreamService mRoomStreamService;

    public RunningRoomService(String roomID){
        mRoomID = roomID;
    }


    /**
     * 创建服务，在登录前调用
     */
    public void createServices(){
        mRoomInfoService = new RoomInfoService(mRoomID);

        mHeartBeatService = new HeartBeatService(mRoomID);
        mTimerTaskService = new TimerTaskService();
        mRoomConnectService = new RoomConnectService(mRoomID);
        mRoomAttributesService = new RoomAttributesService(mRoomID);
        mUserService = new UserService(mRoomID);
        mInviteService = new InviteService(mRoomID);
        mDialogQueueService = new DialogQueueService();
        mRoomDialogService = new RoomDialogService();
        mRoomStreamService = new RoomStreamService(mRoomID);


        mUserService.init();
        mInviteService.init();
        mRoomAttributesService.init();
        mRoomConnectService.init();
        mDialogQueueService.init();
        mRoomDialogService.init();
        mRoomStreamService.init();
    }


    /**
     * 登录成功后调用，用于部分服务初始化
     */
    public void initRoomComplete(){
        mHeartBeatService.init();
        mTimerTaskService.init();

    }


    /**
     * 退出房间或者登录失败调用，用于销毁服务
     */
    public void destroyServices(){

        mHeartBeatService.unInit();
        mTimerTaskService.unInit();
        mRoomConnectService.unInit();
        mUserService.unInit();
        mInviteService.unInit();
        mRoomAttributesService.unInit();
        mDialogQueueService.unInit();
        mRoomDialogService.unInit();
        mRoomStreamService.unInit();

        mRoomInfoService = null;
        mHeartBeatService = null;
        mTimerTaskService = null;
        mRoomConnectService = null;
        mRoomAttributesService = null;
        mUserService = null;
        mInviteService = null;
        mDialogQueueService = null;
        mRoomDialogService = null;
        mRoomStreamService = null;
    }

    /**
     * 获取房间信息服务
     * @return 房间信息服务
     */
    public RoomInfoService getRoomInfoService(){
        return mRoomInfoService;
    }

    /**
     * 获取用户服务
     * @return 用户服务
     */
    public UserService getUserService(){
        return mUserService;
    }

    public void applyRoomInfoServices(IServicesNotNullCallback<RoomInfoService> callback) {

        if(mRoomInfoService != null && callback != null){
            callback.block(mRoomInfoService);
        }

    }

    public void applyRoomConnectService(IServicesNotNullCallback<RoomConnectService> callback) {

        if(mRoomConnectService != null && callback != null){
            callback.block(mRoomConnectService);
        }

    }

    public void applyRoomAttributesService(IServicesNotNullCallback<RoomAttributesService> callback) {

        if(mRoomAttributesService != null && callback != null){
            callback.block(mRoomAttributesService);
        }

    }

    public void applyUserService(IServicesNotNullCallback<UserService> callback) {

        if(mUserService != null && callback != null){
            callback.block(mUserService);
        }

    }

    public void applyInviteService(IServicesNotNullCallback<InviteService> callback) {

        if(mInviteService != null && callback != null){
            callback.block(mInviteService);
        }

    }

    public void applyDialogQueueService(IServicesNotNullCallback<DialogQueueService> callback) {

        if(mDialogQueueService != null && callback != null){
            callback.block(mDialogQueueService);
        }

    }

    public void applyRoomDialogService(IServicesNotNullCallback<RoomDialogService> callback) {

        if(mRoomDialogService != null && callback != null){
            callback.block(mRoomDialogService);
        }

    }


    public void applyTimerTaskService(IServicesNotNullCallback<TimerTaskService> callback) {

        if(mTimerTaskService != null && callback != null){
            callback.block(mTimerTaskService);
        }

    }

    public void applyRoomStreamService(IServicesNotNullCallback<RoomStreamService> callback) {

        if(mRoomStreamService != null && callback != null){
            callback.block(mRoomStreamService);
        }

    }

}
