package im.zego.chathouse.room.service;

import android.os.Handler;
import android.os.HandlerThread;
import android.os.Message;

import androidx.annotation.NonNull;

import im.zego.enjoycommon.log.L;

/**
 * 定时任务服务
 */
public class TimerTaskService {

    private static final String TAG = "TimerTaskService";

    private HandlerThread mThread;
    private Handler mTimerTaskHandler = null;

    public boolean isInit = false;
    /**
     * 初始化，开启消息队列
     */
    public void init(){
        if(isInit){
            return;
        }
        isInit = true;
        mThread = new HandlerThread(TAG);
        mThread.start();

        mTimerTaskHandler = new Handler(mThread.getLooper()){
            @Override
            public void handleMessage(@NonNull Message msg) {
                super.handleMessage(msg);
                if(msg.obj instanceof Runnable){
                    ((Runnable)msg.obj).run();
                }
            }
        };
    }

    /**
     * 反初始化，停止所有任务
     */
    public void unInit(){
        if(!isInit){
            return;
        }
        isInit = false;

        if(mThread != null) {
            mThread.quit();
        }
        mTimerTaskHandler = null;
    }

    /**
     * 添加任务
     * @param what 任务类型ID
     * @param runnable 任务
     * @param delayTime 延迟时间
     */
    public void addTask(int what,Runnable runnable,long delayTime){

        Message message = Message.obtain();
        message.what = what;
        message.obj = runnable;
        if(mTimerTaskHandler != null) {
            L.i(TAG,"addTask: what = " + what +" :: delayTime = " + delayTime);
            mTimerTaskHandler.sendMessageDelayed(message, delayTime);
        }
    }

    /**
     * 移除任务
     * @param what 类型ID
     */
    public void removeTask(int what){
        if(mTimerTaskHandler != null) {
            L.i(TAG,"removeTask: what = " + what);
            mTimerTaskHandler.removeMessages(what);
        }
    }

}
