package im.zego.chathouse.room.service;

import static im.zego.chathouse.sdk.zim.ZIMEventHandlerWrap.DATA_PRIORITY;

import android.os.SystemClock;
import android.text.TextUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import im.zego.chathouse.model.User;
import im.zego.chathouse.room.service.callback.UserServiceHandler;
import im.zego.chathouse.sdk.zim.ZIMEventHandlerWrap;
import im.zego.chathouse.sdk.zim.ZIMSDKManager;
import im.zego.zim.ZIM;
import im.zego.zim.entity.ZIMUserInfo;

/**
 * 用户服务
 */
public class UserService {

    private final String mRoomID;

    private final Map<String, User> mUserList = new HashMap<>();

    private final List<UserServiceHandler> mUserServiceHandler =
            new ArrayList<>();

    private final ZIMEventHandlerWrap mZimEventHandlerWrap;

    public UserService(String roomID){
        mRoomID = roomID;

        mZimEventHandlerWrap = new ZIMEventHandlerWrap(mRoomID,DATA_PRIORITY) {
            @Override
            public void onRoomMemberJoined(ZIM zim, ArrayList<ZIMUserInfo> memberList, String roomID) {
                super.onRoomMemberJoined(zim, memberList, roomID);
                if(memberList != null){
                    for(ZIMUserInfo zimUserInfo : memberList){
                        User user = getUser(zimUserInfo.userID);
                        if(user == null){
                            user = new User();
                            mUserList.put(zimUserInfo.userID,user);
                        }
                        user.setId(zimUserInfo.userID);
                        user.setName(zimUserInfo.userName);
                        user.setLoginTime(SystemClock.elapsedRealtime());

                        User test = new User();
                        test.setRole(User.UserRole.GUEST.ordinal());

                    }
                }
            }

            @Override
            public void onRoomMemberLeft(ZIM zim, ArrayList<ZIMUserInfo> memberList, String roomID) {
                super.onRoomMemberLeft(zim, memberList, roomID);
                if(memberList != null) {
                    for (ZIMUserInfo zimUserInfo : memberList) {
                        mUserList.remove(zimUserInfo.userID);
                    }
                }
            }


        };
    }

    public User getUser(String userID){
        return mUserList.get(userID);
    }

    public void putUser(String userID,User user){
        if(user != null) {

            mUserList.put(userID,user);
        }
    }


    public void setUserBaseInfo(String userID,String userName){

        if(TextUtils.isEmpty(userID)){
            return;
        }

        User user = getUser(userID);
        if(user == null){
            user = new User();
            mUserList.put(userID,user);
        }
        user.setId(userID);
        user.setName(userName);
    }

    /**
     * 设置用户状态并通知更新
     * @param userID 用户ID
     * @param status 状态
     */
    public void setUserStatus(String userID,int status){
        User user = getUser(userID);
        if(user != null){
            user.setStatus(status);
            onUserStatusChange(user);
        }
    }

    /**
     * 设置用户麦克风状态并通知更新
     * @param userID 用户ID
     * @param open true 打开
     */
    public void setUserMic(String userID,boolean open){
        User user = getUser(userID);
        if(user != null){
            user.setMicOn(open);
            onUserMicChange(user);
        }
    }

    public void registerUserServiceHandlerCallbacks(UserServiceHandler callback) {
        synchronized (mUserServiceHandler) {
            mUserServiceHandler.add(callback);
        }
    }

    public void unregisterUserServiceHandlerCallbacks(UserServiceHandler callback) {
        synchronized (mUserServiceHandler) {
            mUserServiceHandler.remove(callback);
        }
    }

    public void clearUserServiceHandlerCallbacks() {
        synchronized (mUserServiceHandler) {
            mUserServiceHandler.clear();
        }
    }

    private Object[] collectUserServiceHandlerCallbacks() {
        Object[] callbacks = null;
        synchronized (mUserServiceHandler) {
            if (mUserServiceHandler.size() > 0) {
                callbacks = mUserServiceHandler.toArray();
            }
        }
        return callbacks;
    }

    public void onUserStatusChange(User user){
        Object[] callbacks = collectUserServiceHandlerCallbacks();
        if (callbacks != null) {
            for (Object callback : callbacks) {
                ((UserServiceHandler) callback).onUserStatusChange(user);
            }
        }
    }

    public void onUserMicChange(User user){
        Object[] callbacks = collectUserServiceHandlerCallbacks();
        if (callbacks != null) {
            for (Object callback : callbacks) {
                ((UserServiceHandler) callback).onUserMicChange(user);
            }
        }
    }

    public void init(){
        ZIMSDKManager.getInstance().getDispatchService().registerZIMEventHandlerCallbacks(mZimEventHandlerWrap);
    }


    public void unInit(){
        ZIMSDKManager.getInstance().getDispatchService().unregisterZIMEventHandlerCallbacks(mZimEventHandlerWrap);
        mUserList.clear();
        clearUserServiceHandlerCallbacks();
    }

}
