package im.zego.chathouse.roomlist;

import static im.zego.chathouse.ErrorCode.ERROR_APP_ID_EXPIRED;
import static im.zego.chathouse.ErrorCode.ERROR_FAIL_NETWORK;
import static im.zego.chathouse.ErrorCode.ERROR_ROOM_NOT_EXIST;
import static im.zego.chathouse.ErrorCode.ERROR_UID_HAS_CREATED_ROOM;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.WindowManager;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.FrameLayout;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;

import com.google.android.material.bottomsheet.BottomSheetBehavior;
import com.google.android.material.bottomsheet.BottomSheetDialog;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import im.zego.chathouse.R;
import im.zego.chathouse.UserProfile;
import im.zego.chathouse.room.RoomActivity;
import im.zego.chathouse.room.RoomManager;
import im.zego.chathouse.help.PermissionHelp;
import im.zego.chathouse.manager.UserManager;
import im.zego.chathouse.model.RoomListInfo;
import im.zego.chathouse.protocol.ICommonResponseCallback;
import im.zego.chathouse.protocol.IGetRoomListCallback;
import im.zego.chathouse.protocol.RoomAPI;
import im.zego.chathouse.roomlist.adapter.EmptyAndLoadMoreAdapterWrapper;
import im.zego.chathouse.roomlist.adapter.RoomListAdapter;
import im.zego.chathouse.roomlist.view.LoadMoreView;
import im.zego.chathouse.roomlist.view.RoomListDecoration;
import im.zego.chathouse.utils.ToastUtil;
import im.zego.chathouse.view.ProcessView;
import im.zego.enjoycommon.util.MiscUtils;
import im.zego.enjoycommon.util.ScreenAdaptationUtil;
import im.zego.enjoycommon.util.T;


public class ChatHouseListActivity extends AppCompatActivity {


    private static final int STATE_NORMAL = 0;
    private static final int STATE_REFRESH = 1;
    private static final int STATE_LOADING = 2;
    private static final int STATE_FINISH = 3;
    private static final int STATE_ERROR = 4;

    private static final int BEGIN_TIME = 0;

    private static final int COUNT_SIZE = 20;
    private long mTimestamp = BEGIN_TIME;


    private FrameLayout mFlRoomList;
    private ConstraintLayout mCreateRoom;

    private BottomSheetDialog mCreateRoomDialog;
    private BottomSheetDialog mJoinRoomDialog;
    private ProcessView mProcessView;

    private SwipeRefreshLayout mSrlRoomList;
    private RecyclerView mRvRoomList;

    private RoomListAdapter mAdapter;
    private EmptyAndLoadMoreAdapterWrapper emptyAndLoadMoreAdapterWrapper;

    private LoadMoreView loadMoreView;

    private int mState;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        ScreenAdaptationUtil.init(this.getApplicationContext(), 375);
        ScreenAdaptationUtil.setCustomDensity(this);

        initUserInfo();
        setContentView(R.layout.chathouse_activity_chat_house_list);
        initView();
        initRecycleView();
        initListener();
        initLoadData();
        initWindowBar();

    }

    private void initUserInfo(){
        UserProfile.reset();
        String uid = UserProfile.USER_ID;
        String name = UserProfile.USER_NAME;
        String userIcon = UserProfile.USER_ICON;
        //保存用户信息
        UserManager.getInstance().setUserInfo(uid, name, userIcon);
    }

    private void initWindowBar() {
        setViewPaddingTop(findViewById(R.id.fl_room_list), MiscUtils.getStatusBarHeight(this));
        WindowManager.LayoutParams lp = getWindow().getAttributes();
        lp.flags = lp.flags | WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS;
        getWindow().setAttributes(lp);
    }

    /**
     * 重设 view 的宽高
     */
    private void setViewPaddingTop(View view, int paddingTop) {
        view.setPadding(view.getPaddingLeft(),paddingTop,view.getPaddingRight(),view.getPaddingBottom());
    }


    private void initView() {
        mFlRoomList = findViewById(R.id.fl_room_list);
        mCreateRoom = findViewById(R.id.cl_create_room);
        mSrlRoomList = findViewById(R.id.refresh_room_list);
        mRvRoomList = findViewById(R.id.room_list);
        mProcessView = new ProcessView(ChatHouseListActivity.this);
        mCreateRoomDialog = new BottomSheetDialog(ChatHouseListActivity.this, R.style.ChatHouseBottomSheetDialog);
        mJoinRoomDialog = new BottomSheetDialog(ChatHouseListActivity.this, R.style.ChatHouseBottomSheetDialog);
    }

    private void initRecycleView() {

        GridLayoutManager layoutManager = new GridLayoutManager(this, 2);
        layoutManager.setSpanSizeLookup(new GridLayoutManager.SpanSizeLookup() {

            @Override
            public int getSpanSize(int position) {
                if ((position == 0 && layoutManager.getItemCount() == 1) || layoutManager.getItemCount() - 1 == position) {
                    return layoutManager.getSpanCount();
                } else {
                    return 1;
                }
            }
        });
        mRvRoomList.setLayoutManager(layoutManager);

        mAdapter = new RoomListAdapter();

        View emptyView = LayoutInflater.from(this).inflate(R.layout.chathouse_empty_room_list, mSrlRoomList, false);

        loadMoreView = new LoadMoreView(this);

        emptyAndLoadMoreAdapterWrapper = new EmptyAndLoadMoreAdapterWrapper(mAdapter);
        emptyAndLoadMoreAdapterWrapper.setEmptyView(emptyView);
        emptyAndLoadMoreAdapterWrapper.setLoadMoreView(loadMoreView);
        mRvRoomList.addItemDecoration(new RoomListDecoration(this,18,13,16));
        mRvRoomList.setAdapter(emptyAndLoadMoreAdapterWrapper);
    }

    private void initListener() {
        mCreateRoom.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                PermissionHelp.micPermission(ChatHouseListActivity.this, () -> showCreateRoomDialog());
            }
        });


        mSrlRoomList.setOnRefreshListener(ChatHouseListActivity.this::onRefresh);

        mAdapter.setOnClickListener(new RoomListAdapter.OnClickListener() {
            @Override
            public void onClick(View v, int position, RoomListInfo.RoomListDTO data) {

                PermissionHelp.micPermission(ChatHouseListActivity.this, () -> {
//                                showJoinRoomDialog(data.getRoomId());
                        joinRoom(data.getRoomId(),UserManager.getInstance().getUserName());
                });


            }
        });

        emptyAndLoadMoreAdapterWrapper.setOnLoadMoreListener(ChatHouseListActivity.this::onLoadMore);

    }

    private void initLoadData() {
        mSrlRoomList.setRefreshing(true);
    }

    private void onRefresh() {
        mState = STATE_REFRESH;

        mTimestamp = BEGIN_TIME;
        RoomAPI.getRoomList( COUNT_SIZE, mTimestamp, new IGetRoomListCallback() {
            @Override
            public void onGetRoomList(int retCode, RoomListInfo roomInfoList) {
                mSrlRoomList.setRefreshing(false);
                if (retCode == 0) {
                    if (roomInfoList != null && roomInfoList.getRoomList() != null) {

                        List<RoomListInfo.RoomListDTO> list = roomInfoList.getRoomList();
                        if (list.size() < COUNT_SIZE) {
                            mState = STATE_FINISH;
                            loadMoreView.showFinish();
                        } else {
                            mState = STATE_NORMAL;
                        }

                        if (list.size() > 0) {
                            mTimestamp = list.get(list.size() - 1).getCreateTime()-1;
                        }

                        mAdapter.refreshData(roomInfoList.getRoomList());
                        emptyAndLoadMoreAdapterWrapper.notifyDataSetChanged();
                    } else {
                        mState = STATE_NORMAL;
                        mAdapter.refreshData(new ArrayList<>());
                        emptyAndLoadMoreAdapterWrapper.notifyDataSetChanged();
                    }
                } else {
                    if (retCode == ERROR_FAIL_NETWORK) {
                        ToastUtil.showWarnToast(R.string.fail_network);

                    }else if(retCode == ERROR_APP_ID_EXPIRED){
                        ToastUtil.showWarnToast(getString(R.string.chathouse_error_message_100010));
                    }
                    mState = STATE_ERROR;
                }
            }
        });
    }

    /**
     * 上拉加载更多
     */
    private void onLoadMore() {
        if (mState == STATE_REFRESH) {
            return;
        }

        if (mState == STATE_NORMAL) {
            mState = STATE_LOADING;
            loadMoreView.showLoading();

            RoomAPI.getRoomList( COUNT_SIZE, mTimestamp, new IGetRoomListCallback() {
                @Override
                public void onGetRoomList(int retCode, RoomListInfo roomInfoList) {
                    if (retCode == 0) {
                        if (roomInfoList != null && roomInfoList.getRoomList() != null) {

                            List<RoomListInfo.RoomListDTO> list = roomInfoList.getRoomList();
                            if (list.size() < COUNT_SIZE) {
                                mState = STATE_FINISH;
                                loadMoreView.showFinish();
                            } else {
                                mState = STATE_NORMAL;
                            }

                            if (list.size() > 0) {
                                mTimestamp = list.get(list.size() - 1).getCreateTime()-1;
                            }

                            mAdapter.addData(roomInfoList.getRoomList());
                            emptyAndLoadMoreAdapterWrapper.notifyItemRangeInserted(mAdapter.getItemCount() - roomInfoList.getRoomList().size(), roomInfoList.getRoomList().size());
                        } else {
                            mState = STATE_ERROR;
                            loadMoreView.showFinish();
                        }
                    } else {
                        if (retCode == ERROR_FAIL_NETWORK) {
                            ToastUtil.showWarnToast(R.string.fail_network);
                        }else if(retCode == ERROR_APP_ID_EXPIRED){
                            ToastUtil.showWarnToast(getString(R.string.chathouse_error_message_100010));
                        }

                        mState = STATE_ERROR;
                        loadMoreView.showFinish();
                    }
                }
            });

        }
    }


    /**
     * 创建房间的弹窗
     */
    private void showCreateRoomDialog() {
        View view = View.inflate(ChatHouseListActivity.this, R.layout.chathouse_dialog_create_room, null);
        view.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                InputMethodManager inputMethodManager = (InputMethodManager) ChatHouseListActivity.this.getSystemService(Activity.INPUT_METHOD_SERVICE);
                inputMethodManager.hideSoftInputFromWindow(view.getWindowToken(), InputMethodManager.HIDE_NOT_ALWAYS);
            }
        });
        final EditText mEtNickName = view.findViewById(R.id.et_nick_name);
        final EditText mEtRoomName = view.findViewById(R.id.et_room_name);
        int randomNum = getKTVRandomNum();
        mEtNickName.setText(UserManager.getInstance().getUserName());
        mEtRoomName.setText(getRandomRoomName(randomNum));
        final Button mBtCreateRoom = view.findViewById(R.id.bt_create_room);

        view.findViewById(R.id.v_top_line).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                mCreateRoomDialog.dismiss();
            }
        });

        mBtCreateRoom.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                if (mEtRoomName.getText() == null || "".equals(mEtRoomName.getText().toString().trim())) {
                    T.show(getString(R.string.chathouse_please_enter_a_room_name));
                    return;
                }
                if (mEtNickName.getText() == null || "".equals(mEtNickName.getText().toString().trim())) {
                    T.show( getString(R.string.chathouse_please_enter_a_user_name));
                    return;
                }
                mCreateRoomDialog.dismiss();

                String cover = UserManager.getInstance().getUserIcon();
                String userName = mEtNickName.getText().toString().trim();
                UserManager.getInstance().setUserName(userName);
                mProcessView.setIsShowShade(false).setCancelable(false).show((ViewGroup) mFlRoomList.getRootView());
                RoomManager.getInstance().createAndJoinRoom(userName, mEtRoomName.getText().toString().trim(),
                        cover, ChatHouseListActivity.this.getApplication(), new ICommonResponseCallback() {
                            @Override
                            public void onResponse(int errorCode) {

                                mProcessView.dismiss();
                                if (errorCode == 0) {

                                    if(mCreateRoomDialog != null && mCreateRoomDialog.isShowing()){
                                        mCreateRoomDialog.dismiss();
                                    }
                                    RoomManager.getInstance().applyRoomInfoServices(
                                            services -> {
                                                Intent intent = new Intent(ChatHouseListActivity.this, RoomActivity.class);
                                                intent.putExtra(RoomActivity.ROOM_ID_INTENT_TAG,services.getRoomID());
                                                startActivity(intent);
                                            }
                                    );

                                }else if(errorCode == ERROR_FAIL_NETWORK){
                                    ToastUtil.showWarnToast(getResources().getString(R.string.login_tips_fail_network));
                                }else if(errorCode == ERROR_UID_HAS_CREATED_ROOM){
                                    ToastUtil.showWarnToast(getString(R.string.chathouse_users_can_only_create_one_room_at_a_time));
                                }else {
                                    ToastUtil.showWarnToast(getString(R.string.chathouse_failed_to_join_please_try_again) + errorCode);
                                }
                            }
                        });

            }
        });
        mCreateRoomDialog.setContentView(view);
        final BottomSheetBehavior<View> mDialogBehavior = BottomSheetBehavior.from((View) view.getParent());
        mDialogBehavior.addBottomSheetCallback(new BottomSheetBehavior.BottomSheetCallback() {
            @Override
            public void onStateChanged(@NonNull View bottomSheet, int newState) {
                if (newState == BottomSheetBehavior.STATE_HIDDEN) {
                    mCreateRoomDialog.dismiss();
                    mDialogBehavior.setState(BottomSheetBehavior.STATE_COLLAPSED);
                }
            }

            @Override
            public void onSlide(@NonNull View bottomSheet, float slideOffset) {
            }
        });
        Window localWindow = mCreateRoomDialog.getWindow();
        localWindow.setGravity(Gravity.BOTTOM);
        localWindow.setWindowAnimations(R.style.BottomSheetDialogAnim);

        mCreateRoomDialog.show();

    }

    /**
     * 加入房间的弹窗
     */
    private void showJoinRoomDialog(String roomID) {
        View view = View.inflate(ChatHouseListActivity.this, R.layout.chathouse_dialog_join_room, null);
        view.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                InputMethodManager inputMethodManager = (InputMethodManager) ChatHouseListActivity.this.getSystemService(Activity.INPUT_METHOD_SERVICE);
                inputMethodManager.hideSoftInputFromWindow(view.getWindowToken(), InputMethodManager.HIDE_NOT_ALWAYS);
            }
        });
        final EditText mEtNickName = view.findViewById(R.id.et_nick_name);
        mEtNickName.setText(UserManager.getInstance().getUserName());
        final Button mBtJoinRoom = view.findViewById(R.id.bt_join_room);

        view.findViewById(R.id.v_top_line).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                mJoinRoomDialog.dismiss();
            }
        });

        mBtJoinRoom.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                if (mEtNickName.getText() == null || "".equals(mEtNickName.getText().toString().trim())) {
                    T.show( getString(R.string.chathouse_please_enter_a_user_name));
                    return;
                }
                mJoinRoomDialog.dismiss();

                String userName = mEtNickName.getText().toString().trim();

                joinRoom(roomID,userName);


            }
        });
        mJoinRoomDialog.setContentView(view);
        final BottomSheetBehavior<View> mDialogBehavior = BottomSheetBehavior.from((View) view.getParent());
        mDialogBehavior.addBottomSheetCallback(new BottomSheetBehavior.BottomSheetCallback() {
            @Override
            public void onStateChanged(@NonNull View bottomSheet, int newState) {
                if (newState == BottomSheetBehavior.STATE_HIDDEN) {
                    mJoinRoomDialog.dismiss();
                    mDialogBehavior.setState(BottomSheetBehavior.STATE_COLLAPSED);
                }
            }

            @Override
            public void onSlide(@NonNull View bottomSheet, float slideOffset) {
            }
        });
        Window localWindow = mJoinRoomDialog.getWindow();
        localWindow.setGravity(Gravity.BOTTOM);
        localWindow.setWindowAnimations(R.style.BottomSheetDialogAnim);

        mJoinRoomDialog.show();

    }


    public void joinRoom(String roomID,String userName){
        mProcessView.setIsShowShade(false).setCancelable(false).show((ViewGroup) mFlRoomList.getRootView());
        RoomManager.getInstance().joinRoom(roomID,userName,
                UserManager.getInstance().getUserIcon(),
                ChatHouseListActivity.this.getApplication(), new ICommonResponseCallback() {
                    @Override
                    public void onResponse(int errorCode) {
                        mProcessView.dismiss();
                        if (errorCode == 0) {

                            if(mJoinRoomDialog != null && mJoinRoomDialog.isShowing()){
                                mJoinRoomDialog.dismiss();
                            }

                            RoomManager.getInstance().applyRoomInfoServices(
                                    services -> {
                                        Intent intent = new Intent(ChatHouseListActivity.this, RoomActivity.class);
                                        intent.putExtra(RoomActivity.ROOM_ID_INTENT_TAG,services.getRoomID());
                                        startActivity(intent);
                                    }
                            );
                        }else if(errorCode == ERROR_FAIL_NETWORK ){

                            ToastUtil.showWarnToast(getResources().getString(R.string.login_tips_fail_network));
                        }else if(errorCode == 6000322 || errorCode == ERROR_ROOM_NOT_EXIST) {
                            ToastUtil.showWarnToast(getResources().getString(R.string.chathouse_room_no_exist_please_refresh_the_list));
                        }else{
                            ToastUtil.showWarnToast(getString(R.string.chathouse_failed_to_join_please_try_again) + errorCode);
                        }
                    }
                });
    }

    public String getRandomRoomName(int randomNum) {
        return String.format(Locale.ENGLISH, getString(R.string.chathouse_chathouse_room), randomNum);
    }

    public static int getKTVRandomNum() {
        return (int) (Math.random() * 1000);
    }

    @Override
    protected void onStop() {
        super.onStop();
    }

    @Override
    public void onBackPressed() {
        super.onBackPressed();
    }

    @Override
    public void onResume() {
        super.onResume();
        onRefresh();
    }

}
