package im.zego.chathouse.roomlist.adapter;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.recyclerview.widget.RecyclerView;

import com.bumptech.glide.Glide;
import com.bumptech.glide.load.engine.DiskCacheStrategy;

import java.util.ArrayList;
import java.util.List;

import im.zego.chathouse.R;
import im.zego.chathouse.model.RoomListInfo;
import im.zego.chathouse.utils.GlideImageLoader;


public class RoomListAdapter extends RecyclerView.Adapter<RoomListAdapter.RoomListViewHolder> {

    private List<RoomListInfo.RoomListDTO> mData = new ArrayList<>();

    public RoomListAdapter() {
    }

    @NonNull
    @Override
    public RoomListViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.chathouse_item_room_list, parent, false);
        return new RoomListViewHolder(view);
    }

    @Override
    public void onBindViewHolder(final @NonNull RoomListViewHolder holder, final int position) {
        RoomListInfo.RoomListDTO info = mData.get(holder.getAdapterPosition());
        holder.mTvRoomName.setText(info.getSubject());
        holder.mTvPeopleNum.setText(String.valueOf(info.getOnlineCount()));

        GlideImageLoader.getInstance().displayImageByUrl(info.getCoverImg(),holder.mIvBg);
        Glide.with(holder.mIvBg.getContext())
                .load(info.getCoverImg())
                .placeholder(R.mipmap.chathouse_room_cover_place)
                .dontAnimate()
                .diskCacheStrategy(DiskCacheStrategy.ALL) // 设置缓存的策略
                .into(holder.mIvBg);
        holder.mClRoomListItem.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (mOnClickListener != null) {
                    mOnClickListener.onClick(v, holder.getAdapterPosition(), mData.get(holder.getAdapterPosition()));
                }
            }
        });
    }

    @Override
    public int getItemCount() {
        if (mData == null) {
            return 0;
        }
        return mData.size();
    }

    static class RoomListViewHolder extends RecyclerView.ViewHolder {

        public ConstraintLayout mClRoomListItem;
        public TextView mTvRoomName;
        public TextView mTvPeopleNum;
        private final ImageView mIvBg;

        private RoomListViewHolder(View itemView) {
            super(itemView);
            mClRoomListItem = itemView.findViewById(R.id.cl_room_list_item);
            mTvRoomName = itemView.findViewById(R.id.tv_room_name);
            mTvPeopleNum = itemView.findViewById(R.id.tv_people_num);
            mIvBg = itemView.findViewById(R.id.iv_bg);
        }
    }

    public void refreshData(List<RoomListInfo.RoomListDTO> newData) {
        mData = newData;
    }


    public void addData(List<RoomListInfo.RoomListDTO> newData) {
        mData.addAll(newData);

    }

    private OnClickListener mOnClickListener;

    public interface OnClickListener {
        void onClick(View v, int position, RoomListInfo.RoomListDTO data);
    }

    public void setOnClickListener(OnClickListener onClickListener) {
        mOnClickListener = onClickListener;
    }

}
