package im.zego.chathouse.sdk.rtc;

import android.text.TextUtils;

import org.json.JSONObject;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import im.zego.chathouse.sdk.constants.ConnectionState;
import im.zego.chathouse.sdk.rtc.callbacks.IRTCEventCallback;
import im.zego.zegoexpress.constants.ZegoAudioRoute;
import im.zego.zegoexpress.constants.ZegoStreamQualityLevel;
import im.zego.zegoexpress.constants.ZegoUpdateType;
import im.zego.zegoexpress.entity.ZegoPlayStreamQuality;
import im.zego.zegoexpress.entity.ZegoPublishStreamQuality;
import im.zego.zegoexpress.entity.ZegoStream;


/**
 * RTC分发服务
 */
public class RTCDispatchService implements IRTCEventCallback {

    private final List<RTCEventHandlerWrap> mRTCEventHandlerCallbacks =
            new ArrayList<>();

    public void registerRTCEventHandlerCallbacks(RTCEventHandlerWrap callback) {
        synchronized (mRTCEventHandlerCallbacks) {
            mRTCEventHandlerCallbacks.add(callback);
        }
    }

    public void unregisterRTCEventHandlerCallbacks(RTCEventHandlerWrap callback) {
        synchronized (mRTCEventHandlerCallbacks) {
            mRTCEventHandlerCallbacks.remove(callback);
        }
    }

    public void clearRTCEventHandlerCallbacksByRoomID(String roomID) {
        synchronized (mRTCEventHandlerCallbacks) {

            if (!TextUtils.isEmpty(roomID)) {

                for (Iterator<RTCEventHandlerWrap> iterator = mRTCEventHandlerCallbacks.iterator(); iterator.hasNext(); ) {

                    RTCEventHandlerWrap wrap = iterator.next();

                    if (roomID.equals(wrap.mRoomID)) {
                        iterator.remove();
                    }

                }
            }
        }
    }


    private Object[] collectRTCEventHandlerCallbacks() {
        Object[] callbacks = null;
        synchronized (mRTCEventHandlerCallbacks) {
            if (mRTCEventHandlerCallbacks.size() > 0) {
                Collections.sort(mRTCEventHandlerCallbacks);
                callbacks = mRTCEventHandlerCallbacks.toArray();
            }
        }
        return callbacks;
    }


    @Override
    public void onSoundLevelUpdate(HashMap<String, Float> userVolumeMap) {
        Object[] callbacks = collectRTCEventHandlerCallbacks();
        if (callbacks != null) {
            for (int i = 0; i < callbacks.length; i++) {
                ((RTCEventHandlerWrap) callbacks[i]).onSoundLevelUpdate(userVolumeMap);
            }
        }
    }

    @Override
    public void onMixSoundLevelUpdate(HashMap<Integer, Float> userVolumeMap) {
        Object[] callbacks = collectRTCEventHandlerCallbacks();
        if (callbacks != null) {
            for (int i = 0; i < callbacks.length; i++) {
                ((RTCEventHandlerWrap) callbacks[i]).onMixSoundLevelUpdate(userVolumeMap);
            }
        }
    }

    @Override
    public void onAudioRouteChange(ZegoAudioRoute zegoAudioRoute) {
        Object[] callbacks = collectRTCEventHandlerCallbacks();
        if (callbacks != null) {
            for (int i = 0; i < callbacks.length; i++) {
                ((RTCEventHandlerWrap) callbacks[i]).onAudioRouteChange(zegoAudioRoute);
            }
        }
    }

    @Override
    public void onPlayerQualityUpdate(String streamID, ZegoPlayStreamQuality quality) {
        Object[] callbacks = collectRTCEventHandlerCallbacks();
        if (callbacks != null) {
            for (int i = 0; i < callbacks.length; i++) {
                ((RTCEventHandlerWrap) callbacks[i]).onPlayerQualityUpdate(streamID,quality);
            }
        }
    }

    @Override
    public void onRoomStreamUpdate(String roomID, ZegoUpdateType updateType, ArrayList<ZegoStream> streamList, JSONObject extendedData) {
        Object[] callbacks = collectRTCEventHandlerCallbacks();
        if (callbacks != null) {
            for (int i = 0; i < callbacks.length; i++) {
                ((RTCEventHandlerWrap) callbacks[i]).onRoomStreamUpdate(roomID,updateType,streamList,extendedData);
            }
        }
    }

    @Override
    public void onPublisherQualityUpdate(String streamID, ZegoPublishStreamQuality quality) {
        Object[] callbacks = collectRTCEventHandlerCallbacks();
        if (callbacks != null) {
            for (int i = 0; i < callbacks.length; i++) {
                ((RTCEventHandlerWrap) callbacks[i]).onPublisherQualityUpdate(streamID,quality);
            }
        }
    }

    @Override
    public void onNetworkSpeedTestQualityUpdate(ZegoStreamQualityLevel zegoStreamQualityLevel) {
        Object[] callbacks = collectRTCEventHandlerCallbacks();
        if (callbacks != null) {
            for (int i = 0; i < callbacks.length; i++) {
                ((RTCEventHandlerWrap) callbacks[i]).onNetworkSpeedTestQualityUpdate(zegoStreamQualityLevel);
            }
        }
    }

    @Override
    public void onUserNetQualityLevelUpdate(String userID, ZegoStreamQualityLevel zegoStreamQualityLevel) {
        Object[] callbacks = collectRTCEventHandlerCallbacks();
        if (callbacks != null) {
            for (int i = 0; i < callbacks.length; i++) {
                ((RTCEventHandlerWrap) callbacks[i]).onUserNetQualityLevelUpdate(userID,zegoStreamQualityLevel);
            }
        }
    }

    @Override
    public void onConnectionState(ConnectionState state) {
        Object[] callbacks = collectRTCEventHandlerCallbacks();
        if (callbacks != null) {
            for (int i = 0; i < callbacks.length; i++) {
                ((RTCEventHandlerWrap) callbacks[i]).onConnectionState(state);
            }
        }
    }

    @Override
    public void onRemoteMicStateUpdate(String streamID, boolean open) {
        Object[] callbacks = collectRTCEventHandlerCallbacks();
        if (callbacks != null) {
            for (int i = 0; i < callbacks.length; i++) {
                ((RTCEventHandlerWrap) callbacks[i]).onRemoteMicStateUpdate(streamID,open);
            }
        }
    }

}
