package im.zego.chathouse.sdk.rtc;

import static im.zego.chathouse.Constants.MIX_STREAM_SUFFIX;
import static im.zego.enjoycommon.util.ZegoGoUtil.ZEGO_LOGS_SUBPATH;
import static im.zego.zegoexpress.constants.ZegoRoomState.CONNECTED;
import static im.zego.zegoexpress.constants.ZegoRoomState.CONNECTING;
import static im.zego.zegoexpress.constants.ZegoRoomState.DISCONNECTED;

import android.app.Application;
import android.content.Context;
import android.graphics.Rect;
import android.os.Environment;

import org.json.JSONObject;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import im.zego.chathouse.ErrorCode;
import im.zego.chathouse.KeyCenter;
import im.zego.chathouse.R;
import im.zego.chathouse.protocol.ICommonResponseCallback;
import im.zego.chathouse.protocol.TokenManager;
import im.zego.chathouse.room.help.TimerTaskHelp;
import im.zego.chathouse.sdk.constants.ConnectionState;
import im.zego.chathouse.sdk.rtc.callbacks.IConnectionStateChangeListener;
import im.zego.chathouse.sdk.rtc.callbacks.IRTCCommonCallback;
import im.zego.enjoycommon.log.L;
import im.zego.enjoycommon.util.MiscUtils;
import im.zego.enjoycommon.util.T;

import im.zego.zegoexpress.ZegoExpressEngine;
import im.zego.zegoexpress.callback.IZegoDestroyCompletionCallback;
import im.zego.zegoexpress.callback.IZegoEventHandler;
import im.zego.zegoexpress.callback.IZegoMixerStartCallback;
import im.zego.zegoexpress.callback.IZegoMixerStopCallback;
import im.zego.zegoexpress.constants.ZegoAECMode;
import im.zego.zegoexpress.constants.ZegoANSMode;
import im.zego.zegoexpress.constants.ZegoAudioChannel;
import im.zego.zegoexpress.constants.ZegoAudioCodecID;
import im.zego.zegoexpress.constants.ZegoAudioConfigPreset;
import im.zego.zegoexpress.constants.ZegoAudioRoute;
import im.zego.zegoexpress.constants.ZegoLanguage;
import im.zego.zegoexpress.constants.ZegoMixerInputContentType;
import im.zego.zegoexpress.constants.ZegoNetworkMode;
import im.zego.zegoexpress.constants.ZegoNetworkSpeedTestType;
import im.zego.zegoexpress.constants.ZegoPlayerState;
import im.zego.zegoexpress.constants.ZegoPublishChannel;
import im.zego.zegoexpress.constants.ZegoPublisherState;
import im.zego.zegoexpress.constants.ZegoRemoteDeviceState;
import im.zego.zegoexpress.constants.ZegoReverbPreset;
import im.zego.zegoexpress.constants.ZegoRoomState;
import im.zego.zegoexpress.constants.ZegoScenario;
import im.zego.zegoexpress.constants.ZegoStreamQualityLevel;
import im.zego.zegoexpress.constants.ZegoUpdateType;
import im.zego.zegoexpress.entity.ZegoAudioConfig;
import im.zego.zegoexpress.entity.ZegoAutoMixerTask;
import im.zego.zegoexpress.entity.ZegoBroadcastMessageInfo;
import im.zego.zegoexpress.entity.ZegoEngineConfig;
import im.zego.zegoexpress.entity.ZegoEngineProfile;
import im.zego.zegoexpress.entity.ZegoMixerAudioConfig;
import im.zego.zegoexpress.entity.ZegoMixerInput;
import im.zego.zegoexpress.entity.ZegoMixerOutput;
import im.zego.zegoexpress.entity.ZegoMixerTask;
import im.zego.zegoexpress.entity.ZegoNetworkSpeedTestConfig;
import im.zego.zegoexpress.entity.ZegoNetworkSpeedTestQuality;
import im.zego.zegoexpress.entity.ZegoPlayStreamQuality;
import im.zego.zegoexpress.entity.ZegoPublishStreamQuality;
import im.zego.zegoexpress.entity.ZegoRoomConfig;
import im.zego.zegoexpress.entity.ZegoStream;
import im.zego.zegoexpress.entity.ZegoUser;

public class RTCSDKManager {

    private static final String TAG = "RTCSDKManager";

    private ZegoExpressEngine mSDKEngine = null;

    /**
     * 房间最大人数
     */
    public static final int MAX_USER_COUNT = 10000000;
    /**
     * 用户信息
     */
    private ZegoUser mUserInfo = null;
    /**
     * 房间ID
     */
    private String mRoomID = null;

    /**
     * 声浪集合
     */
    private final HashMap<String, Float> mUserVolumeMap = new HashMap<>();
    /**
     * 是否首次登录,true 表示首次登录房间
     */
    private boolean mHasFirstLoginRoom = true;

    private IRTCCommonCallback mLoginCallback = null;

    private IConnectionStateChangeListener mConnectionStateChangeListener;

    private final RTCDispatchService mDispatchService = new RTCDispatchService();

    private RTCSDKManager() {
    }

    private static final class Holder {
        private static final RTCSDKManager INSTANCE = new RTCSDKManager();
    }

    public static RTCSDKManager getInstance() {
        return Holder.INSTANCE;
    }

    /**
     * 初始化引擎
     */
    public void init(Context application) {

        if (mSDKEngine != null) {
            mSDKEngine.startPerformanceMonitor(3000);
            return;
        }
        preEngineConfig(application);

        ZegoEngineProfile profile = new ZegoEngineProfile();
        profile.appID = KeyCenter.APP_ID;
        profile.appSign = KeyCenter.APP_SIGN;
        profile.scenario = ZegoScenario.GENERAL;
        profile.application = (Application) application.getApplicationContext();

        mSDKEngine = ZegoExpressEngine.createEngine(profile, mEventHandler);

        if(mSDKEngine != null) {
            engineConfig();
//            ZGKTVCopyrightedMusicManager.getInstance().init(application, null);
        }else {
            T.show(application.getString(R.string.chathouse_failed_to_create_the_engine));
        }
    }

    //反初始化
    public void unInit() {
        if (mSDKEngine == null) {
            return;
        }

//        mConnectionStateChangeListener = null;
        mSDKEngine.setEventHandler(null);
        ZegoExpressEngine.destroyEngine(new IZegoDestroyCompletionCallback() {
            @Override
            public void onDestroyCompletion() {

            }
        });
        mSDKEngine = null;
    }

    /**
     * 初始化引擎前配置
     */
    private void preEngineConfig(Context application) {
        //配置日志路径
        ZegoEngineConfig engineConfig = new ZegoEngineConfig();
        if (Environment.getExternalStorageState().equals(Environment.MEDIA_MOUNTED)) {
            if (application.getExternalFilesDir(null) != null) {
                engineConfig.logConfig.logPath = application.getExternalFilesDir(null).getAbsolutePath() + File.separator + ZEGO_LOGS_SUBPATH;
                engineConfig.logConfig.logSize = 10485760L;
            } else {
                engineConfig.logConfig.logPath = application.getFilesDir().getAbsolutePath() + File.separator + ZEGO_LOGS_SUBPATH;
                engineConfig.logConfig.logSize = 10485760L;
            }
        }
        //房间超时时间,单位秒
        engineConfig.advancedConfig.put("room_retry_time","300");

        //推流质量回调周期
        engineConfig.advancedConfig.put("publish_quality_interval", "500");
        //拉流质量回调周期
        engineConfig.advancedConfig.put("play_quality_interval", "500");

        ZegoExpressEngine.setEngineConfig(engineConfig);
    }

    /**
     *   引擎配置
     */
    private void engineConfig() {
        if (mSDKEngine == null) {
            return;
        }
        //启动系统性能监控。
        mSDKEngine.startPerformanceMonitor(3000);
        //不需要视频功能 关闭视频功能
        mSDKEngine.enableCamera(false);

        mSDKEngine.setDebugVerbose(true, ZegoLanguage.CHINESE);

        ZegoAudioConfig audioConfig = new ZegoAudioConfig(ZegoAudioConfigPreset.HIGH_QUALITY);
        audioConfig.codecID = ZegoAudioCodecID.LOW3;
        mSDKEngine.setAudioConfig(audioConfig, ZegoPublishChannel.MAIN);
    }

    /**
     * 高质量音频配置(用于房主/主持人),有背景音时使用此配置
     */
    public void highQualityAudioConfig(){
        if (mSDKEngine == null) {
            return;
        }
        if( (mSDKEngine.getAudioConfig(ZegoPublishChannel.MAIN).channel !=  ZegoAudioChannel.STEREO) || (mSDKEngine.getAudioConfig(ZegoPublishChannel.MAIN).bitrate !=  128) ) {
            ZegoAudioConfig audioConfig = new ZegoAudioConfig(ZegoAudioConfigPreset.HIGH_QUALITY);
            audioConfig.codecID = ZegoAudioCodecID.LOW3;
            audioConfig.bitrate = 128;
            audioConfig.channel = ZegoAudioChannel.STEREO;

            mSDKEngine.setAudioConfig(audioConfig, ZegoPublishChannel.MAIN);

        }
    }

    /**
     * 标准音频配置(用于连麦观众)
     */
    public void standardQualityAudioConfig(){
        if (mSDKEngine == null) {
            return;
        }
        ZegoAudioConfig audioConfig = new ZegoAudioConfig(ZegoAudioConfigPreset.STANDARD_QUALITY);
        audioConfig.codecID = ZegoAudioCodecID.LOW3;
        audioConfig.bitrate = 48;
        audioConfig.channel = ZegoAudioChannel.MONO;
        mSDKEngine.setAudioConfig(audioConfig, ZegoPublishChannel.MAIN);
    }

    //开启网络测速
    public void startNetworkMonitor() {
//        L.i(TAG,"startNetworkMonitor");

        if(mSDKEngine == null){
            return;
        }
        ZegoNetworkSpeedTestConfig zegoNetworkSpeedTestConfig = new ZegoNetworkSpeedTestConfig();
        zegoNetworkSpeedTestConfig.testUplink = true;
        zegoNetworkSpeedTestConfig.expectedUplinkBitrate = 128;
        mSDKEngine.startNetworkSpeedTest(zegoNetworkSpeedTestConfig);
    }

    /**
     * 开启3A效果
     */
    public void initAudioProcessParams() {
        if (mSDKEngine == null) {
            return;
        }
        enableAEC(true);  //是否开启回声消除；true 表示开启；false 表示关闭
        enableAGC(false);  //是否开启自动增益控制；true 表示开启；false 表示关闭
        enableANS(true);  //是否开启噪声抑制；true 表示开启噪声抑制；false 表示关闭噪声抑制
        mSDKEngine.setAECMode(ZegoAECMode.AGGRESSIVE); //设置回声消除模式
        mSDKEngine.setANSMode(ZegoANSMode.MEDIUM); //设置音频噪声抑制模式
    }

    /**
     * 设置耳返音量
     */
    public void setLoopBackVolume(int volume) {
        if (mSDKEngine == null) {
            return;
        }
        mSDKEngine.setHeadphoneMonitorVolume(volume * 2);
    }

    /**
     * 设置推流端采集音量
     */
    public void setCaptureVolume(int volume) {
        if (mSDKEngine == null) {
            return;
        }
        mSDKEngine.setCaptureVolume(volume);
    }

    /**
     * 是否开启回声消除
     * @param enable true 表示开启；false 表示关闭
     */
    public void enableAEC(boolean enable) {
        if (mSDKEngine == null) {
            return;
        }
        L.i(TAG, "enableAEC :" + enable);
        mSDKEngine.enableAEC(enable);
    }

    /**
     * 是否开启噪声抑制
     * @param enable true 表示开启噪声抑制；false 表示关闭噪声抑制
     */
    public void enableANS(boolean enable) {
        if (mSDKEngine == null) {
            return;
        }
        L.i(TAG, "enableANS :" + enable);
        mSDKEngine.enableANS(enable);
    }

    /**
     * 是否开启自动增益控制
     * @param enable true 表示开启；false 表示关闭
     */
    public void enableAGC(boolean enable) {
        if (mSDKEngine == null) {
            return;
        }
        L.i(TAG, "enableAGC :" + enable);
        mSDKEngine.enableAGC(enable);
    }

    /**
     * 开启耳返
     * @param enable false: 关闭耳返
     */
    public void enableHeadphoneMonitor(boolean enable) {
        if (mSDKEngine == null) {
            return;
        }
        L.i(TAG, "enableHeadphoneMonitor :" + enable);
        mSDKEngine.enableHeadphoneMonitor(enable);
    }

    /**
     * 设置混响
     */
    public void setReverbPreset(ZegoReverbPreset mode) {
        if (mSDKEngine == null) {
            return;
        }
        L.i(TAG, "setVerbPresetMode :" + mode);
        mSDKEngine.setReverbPreset(mode);
    }

    public void enableMic(boolean enable){
        if (mSDKEngine == null) {
            return;
        }
        L.i(TAG, "enableMic :" + enable);
        mSDKEngine.muteMicrophone(!enable);
    }

    //设置用户信息，需在登录前设置
    public void setUserID(String userID,  String userName) {
        mUserInfo = new ZegoUser(String.valueOf(userID), userName);
    }


    //登录房间
    public void loginRoom( String roomID, IRTCCommonCallback callback) {
        if (mSDKEngine == null) {
            return;
        }
        mSDKEngine.setEventHandler(null);
        mSDKEngine.setEventHandler(mEventHandler);
        enableMic(true);
        initAudioProcessParams();

        if (mSDKEngine == null) {
            callback.onRTCCallback(ErrorCode.ERROR_RTC_SDK_NOT_INITIALIZED);
            return;
        } else if (mUserInfo == null) {
            callback.onRTCCallback(ErrorCode.ERROR_RTC_SDK_INVALID_USERID);
            return;
        }

        mRoomID = roomID;

        mHasFirstLoginRoom = true;
        mLoginCallback = callback;

        ZegoRoomConfig roomConfig = new ZegoRoomConfig();
        roomConfig.maxMemberCount = MAX_USER_COUNT;
        roomConfig.isUserStatusNotify = true;
        roomConfig.token = TokenManager.getInstance().getToken();
        mSDKEngine.logoutRoom();
        mSDKEngine.loginRoom(roomID, mUserInfo, roomConfig);
        mSDKEngine.startSoundLevelMonitor(500);
    }

    //退出房间
    public void logoutRoom() {

        if (mSDKEngine == null) {
            return;
        }
        mSDKEngine.stopNetworkSpeedTest();
        if (mRoomID != null) {
            mSDKEngine.logoutRoom(mRoomID);
//            mSDKEngine.setEventHandler(null);
            mRoomID = null;

            mHasFirstLoginRoom = true;
        }
        if (mSDKEngine != null) {
            mSDKEngine.stopSoundLevelMonitor();
        }
    }

    //更新混流
    public void updateMixStream(Map<Integer, String> mixStreamMap) {
        L.i(TAG, "updateMixStream");

        if(mSDKEngine == null){
            return;
        }


        ZegoMixerTask mixerTask = new ZegoMixerTask(getMixStreamID());
        mixerTask.enableSoundLevel(true);


        ZegoMixerAudioConfig audioConfig = new ZegoMixerAudioConfig();
        audioConfig.bitrate = 48;
        audioConfig.codecID = ZegoAudioCodecID.DEFAULT;
        audioConfig.channel = ZegoAudioChannel.MONO;
        mixerTask.setAudioConfig(audioConfig);

        ArrayList<ZegoMixerInput> inputs = new ArrayList<>();
        Rect audioRect = new Rect(0, 0, 0, 0);



        for(Map.Entry<Integer, String> entry: mixStreamMap.entrySet()){
            L.i(TAG,"updateMixStream  : id = " + entry.getKey()  + " : streamID = " + entry.getValue());
            ZegoMixerInput streamInput = new ZegoMixerInput(entry.getValue(), ZegoMixerInputContentType.AUDIO, audioRect);
            streamInput.soundLevelID = entry.getKey();
            inputs.add(streamInput);
        }

        mixerTask.setInputList(inputs);

        ZegoMixerOutput output = new ZegoMixerOutput(getMixStreamID());
        ArrayList<ZegoMixerOutput> outputs = new ArrayList<>();
        outputs.add(output);
        mixerTask.setOutputList(outputs);

        mSDKEngine.startMixerTask(mixerTask, new IZegoMixerStartCallback() {
            @Override
            public void onMixerStartResult(int resultCode, JSONObject jsonObject) {
                if (resultCode == 1005000 || resultCode == 1005099) {
                    T.show(R.string.chathouse_mixer_stream_service_has_been_opened);
                }
                L.e(TAG,"startMixerTask : mixStreamID = " +  getMixStreamID()   + " errorCode = " + resultCode + " : jsonObject = " + jsonObject.toString() );
            }
        });
    }


    public void autoMixerStream(){
        L.e(TAG,"autoMixerStream :" + mRoomID);

        if(mSDKEngine == null){
            return;
        }

        ZegoAutoMixerTask task = new ZegoAutoMixerTask();
        task.taskID = "autoMixerStream";
        task.roomID = mRoomID;

        ZegoMixerAudioConfig audioConfig = new ZegoMixerAudioConfig();
        audioConfig.bitrate = 48;
        audioConfig.codecID = ZegoAudioCodecID.DEFAULT;
        audioConfig.channel = ZegoAudioChannel.MONO;
        task.audioConfig = audioConfig;
        task.enableSoundLevel = true;
        ArrayList<ZegoMixerOutput> outputList = new ArrayList<>();
        ZegoMixerOutput output = new ZegoMixerOutput(getMixStreamID());
        outputList.add(output);
        task.outputList = outputList;

        mSDKEngine.startAutoMixerTask(task, new IZegoMixerStartCallback() {
            @Override
            public void onMixerStartResult(int resultCode, JSONObject jsonObject) {
                if (resultCode == 1005000 || resultCode == 1005099) {
                    T.show(R.string.chathouse_mixer_stream_service_has_been_opened);
                }
                L.e(TAG,"autoMixerStream : mixStreamID = " +  getMixStreamID()   + " errorCode = " + resultCode + " : jsonObject = " + jsonObject.toString() );
            }
        });

    }

    public void stopMixStream(IRTCCommonCallback callback){
        L.e(TAG,"stopMixerTask start: mixStreamID = " +  getMixStreamID() );

        if(mSDKEngine == null){
            return;
        }

        ZegoMixerTask task = new ZegoMixerTask(getMixStreamID());

        mSDKEngine.stopMixerTask(task, new IZegoMixerStopCallback() {
            @Override
            public void onMixerStopResult(int resultCode) {
                L.e(TAG,"stopMixerTask : mixStreamID = " +  getMixStreamID()   + " errorCode = " + resultCode );
                if(callback != null){
                    callback.onRTCCallback(resultCode);
                }
            }
        });
    }


    public String getMixStreamID(){
        return mRoomID + MIX_STREAM_SUFFIX;
    }


    public String getMainStreamId(){
        return mUserInfo.userID;
    }

    /**
     * 开始推流
     * @param streamID 流ID
     */
    public void startPublishingStream(String streamID){
        if(mSDKEngine != null){
            L.i(TAG,"startPublishingStream streamID :" + streamID);
            mSDKEngine.startPublishingStream(streamID);
        }
    }

    /**
     * 停止推流
     */
    public void stopPublishingStream(){
        if(mSDKEngine != null){
            L.i(TAG,"stopPublishingStream:");
            mSDKEngine.stopPublishingStream();
        }
    }

    /**
     * 开始拉流
     * @param streamID 流ID
     */
    public void startPlayingStream(String streamID){
        if(mSDKEngine != null){
            L.i(TAG,"startPlayingStream streamID :" + streamID);
            mSDKEngine.startPlayingStream(streamID);
        }
    }

    /**
     * 停止拉流
     * @param streamID 流ID
     */
    public void stopPlayingStream(String streamID){
        if(mSDKEngine != null){
            L.i(TAG,"stopPlayingStream streamID :" + streamID);
            mSDKEngine.stopPlayingStream(streamID);
        }
    }

    /**
     * 停止拉混流
     */
    public void stopPlayingMix() {
        L.i(TAG, "stopPlayingAux");
        if(mSDKEngine != null) {
            mSDKEngine.stopPlayingStream(getMixStreamID());
        }
    }

    /**
     * 开始拉混流
     */
    public void startPlayingMix() {

        if(mSDKEngine == null){
            return;
        }
        //拉混流
        //设置拉流jitter buffer大小
        L.i(TAG, "startPlayingAux");
        mSDKEngine.startPlayingStream(getMixStreamID());
    }

    public RTCDispatchService getDispatchService() {
        return mDispatchService;
    }

    /**
     * 设置连接状态变更通知
     */
    public void setConnectionStateChangeListener(IConnectionStateChangeListener connectionStateChangeListener) {
        this.mConnectionStateChangeListener = connectionStateChangeListener;
    }

    private final IZegoEventHandler mEventHandler = new IZegoEventHandler() {

        @Override
        public void onDebugError(int errorCode, String funcName, String info) {
            super.onDebugError(errorCode, funcName, info);


            L.e(TAG, "onDebugError-:errorCode:" + errorCode + ":funcName:" + funcName);
            L.e(TAG, "onDebugError-:errorCode:" + errorCode + ":info:" + info);
        }

        /**
         * 房间token即将过期回调，调用token刷新接口刷新
         * @param roomID 房间ID
         */
        @Override
        public void onRoomTokenWillExpire(String roomID, int remainTimeInSecond) {
            super.onRoomTokenWillExpire(roomID, remainTimeInSecond);
            TimerTaskHelp.refreshToken(new ICommonResponseCallback() {
                @Override
                public void onResponse(int errorCode) {
                    if(errorCode == 0){
                        if (mSDKEngine == null) {
                            return;
                        }
                        mSDKEngine.renewToken(roomID, TokenManager.getInstance().getToken());
                    }else {
                        T.show(String.format(MiscUtils.getApplication().getString(R.string.chathouse_token_update_failed), errorCode));
                    }
                }
            });
        }

        /**
         * 推流状态更新
         * @param streamID 流ID
         * @param state 状态
         * @param errorCode 错误码
         * @param extendedData 附加信息
         */
        @Override
        public void onPublisherStateUpdate(String streamID, ZegoPublisherState state, int errorCode, JSONObject extendedData) {
            L.i(TAG,"onPublisherStateUpdate: streamID :" + streamID +": state :" + state + ": errorCode :" + errorCode);
//            startNetworkMonitor();
        }

        /**
         * 拉流状态更新
         * @param streamID 流ID
         * @param state 状态
         * @param errorCode 错误码
         * @param extendedData 附加信息
         */
        @Override
        public void onPlayerStateUpdate(String streamID, ZegoPlayerState state, int errorCode, JSONObject extendedData) {
            L.i(TAG,"onPlayerStateUpdate: streamID :" + streamID +": state :" + state + ": errorCode :" + errorCode);
//            startNetworkMonitor();
        }

        /**
         * 房间流变更通知
         * @param roomID 房间ID
         * @param updateType 更新类型 新增或删除
         * @param streamList 流列表
         * @param extendedData 附加信息
         */
        @Override
        public void onRoomStreamUpdate(String roomID, ZegoUpdateType updateType, ArrayList<ZegoStream> streamList, JSONObject extendedData) {
            if (mSDKEngine == null) {
                return;
            } else if (mRoomID == null) {
                return;
            }
            for(ZegoStream zegoStream : streamList) {
                L.i(TAG, "onRoomStreamUpdate: streamID :" + zegoStream.streamID + ": state :" + updateType + ": roomID :" + roomID);
            }


//            if (mEventCallback != null) {
//                mEventCallback.onRoomStreamUpdate(roomID, updateType, streamList, extendedData);
//            }
            mDispatchService.onRoomStreamUpdate(roomID,updateType,streamList,extendedData);

        }

        /**
         * 推流质量更新
         * @param streamID 流ID
         * @param quality 质量
         */
        @Override
        public void onPublisherQualityUpdate(String streamID, ZegoPublishStreamQuality quality) {
//            L.i(TAG,"onPublisherQualityUpdate:" + streamID + " : quality : " + quality.level);
//            if(mEventCallback != null){
//                mEventCallback.onPublisherQualityUpdate(streamID,quality);
//            }
            mDispatchService.onPublisherQualityUpdate(streamID,quality);
        }

        @Override
        public void onIMRecvCustomCommand(String roomID, ZegoUser fromUser, String command) {

        }

        @Override
        public void onIMRecvBroadcastMessage(String roomID, ArrayList<ZegoBroadcastMessageInfo> messageList) {

        }

        /**
         * 本地采集音频声浪回调
         *
         * 回调通知周期为 100 ms。
         * @param soundLevel 本地采集的声浪值，取值范围为 0.0 ~ 100.0
         */
        @Override
        public void onCapturedSoundLevelUpdate(float soundLevel) {
//            if (mEventCallback != null) {
                mUserVolumeMap.clear();
                mUserVolumeMap.put(getMainStreamId(),soundLevel);
//                mEventCallback.onUserVolumeUpdate(mUserVolumeMap);
                mDispatchService.onSoundLevelUpdate(mUserVolumeMap);
//            }

        }

        @Override
        public void onMixerSoundLevelUpdate(HashMap<Integer, Float> soundLevels) {
            mDispatchService.onMixSoundLevelUpdate(soundLevels);
        }

        /**
         * 拉流质量更新
         * @param streamID 流ID
         * @param quality 质量
         */
        @Override
        public void onPlayerQualityUpdate(String streamID, ZegoPlayStreamQuality quality) {
//            L.i(TAG,"onPlayerQualityUpdate:" + streamID + " : quality : " + quality.level);
//            L.i(TAG,"onPlayerQualityUpdate:" + streamID + " : audioKBPS : " + quality.audioKBPS);
//            if(mEventCallback != null){
//                mEventCallback.onPlayerQualityUpdate(streamID,quality);
//            }
            mDispatchService.onPlayerQualityUpdate(streamID,quality);
        }

        /**
         * 远端拉流音频声浪回调
         *
         * 回调通知周期为 100 ms。
         * @param soundLevels 远端的声浪键值对，key 为流 ID，value 为对应的流的声浪值，value 取值范围为 0.0 ~ 100.0
         */
        @Override
        public void onRemoteSoundLevelUpdate(HashMap<String, Float> soundLevels) {
//            if (mEventCallback != null) {
//                mEventCallback.onUserVolumeUpdate(soundLevels);
//            }
            mDispatchService.onSoundLevelUpdate(soundLevels);
        }

        /**
         * 音频设备路由变更通知
         * @param audioRoute 音频设备
         */
        @Override
        public void onAudioRouteChange(ZegoAudioRoute audioRoute) {
            switch (audioRoute) {
                case HEADPHONE:
                case BLUETOOTH:
                    enableAEC(false);
                    break;
                default:
                    enableAEC(true);
                    break;
            }

        }

        @Override
        public void onNetworkModeChanged(ZegoNetworkMode mode) {
            super.onNetworkModeChanged(mode);
        }

        @Override
        public void onNetworkQuality(String userID, ZegoStreamQualityLevel upstreamQuality, ZegoStreamQualityLevel downstreamQuality) {
            super.onNetworkQuality(userID, upstreamQuality, downstreamQuality);
            //检测用户网络质量
//            L.i(TAG,"onNetworkQuality:" + userID + " : quality : " + upstreamQuality);
//            L.i(TAG,"onNetworkQuality:" + userID + " : downstreamQuality : " + downstreamQuality);
//            if (mEventCallback != null) {
//                mEventCallback.onUserNetQualityLevelUpdate(userID,upstreamQuality);
//            }
            mDispatchService.onUserNetQualityLevelUpdate(userID,upstreamQuality);
        }

        @Override
        public void onNetworkSpeedTestQualityUpdate(ZegoNetworkSpeedTestQuality onNetworkSpeedTestQualityUpdate, ZegoNetworkSpeedTestType type) {
            //网络测速质量回调通知
//            L.i(TAG,"onNetworkSpeedTestQualityUpdate:" + onNetworkSpeedTestQualityUpdate.quality + " : type : " + type);
//            if (mEventCallback != null) {
//                mEventCallback.onNetworkSpeedTestQualityUpdate(onNetworkSpeedTestQualityUpdate.quality);
//            }
            mDispatchService.onNetworkSpeedTestQualityUpdate(onNetworkSpeedTestQualityUpdate.quality);
        }

        @Override
        public void onNetworkSpeedTestError(int errorCode, ZegoNetworkSpeedTestType type) {
            //网络测速质量错误回调通知
//            L.i(TAG,"onNetworkSpeedTestError:" + errorCode + " : type : " + type);
            /*
              1015004 服务端结束网络测速。网络测速时间过长。
              1015005 开始推流前未主动停止网络测速。
              1015006 开始拉流前未主动停止网络测速。
             */
            if(errorCode == 1015004 || errorCode == 1015005 || errorCode == 1015006){

                return;
            }

//            if (mEventCallback != null) {
//                mEventCallback.onNetworkSpeedTestQualityUpdate(ZegoStreamQualityLevel.BAD);
//            }
            mDispatchService.onNetworkSpeedTestQualityUpdate(ZegoStreamQualityLevel.BAD);
        }

        /**
         * 房间状态回调
         * @param roomID 房间ID
         * @param state 状态
         * @param errorCode 错误码
         * @param extendedData 附带信息
         */
        @Override
        public void onRoomStateUpdate(String roomID, ZegoRoomState state, int errorCode, JSONObject extendedData) {
            L.e(TAG,"roomID:" + roomID + " : errorCode : " + errorCode);
            L.e(TAG,"roomID:" + roomID + " : state : " + state);

            if (state == CONNECTED) {

                if (mHasFirstLoginRoom) {

                    if(mLoginCallback != null)
                    {
                        mLoginCallback.onRTCCallback(errorCode);
                        mLoginCallback = null;
                    }

                    mHasFirstLoginRoom = false;
                } else {
                    L.i(TAG, "detect RTC Room reconnected");
//                    if(mConnectionStateChangeListener != null){
//                        mConnectionStateChangeListener.onConnectionState(ConnectionState.CONNECTED);
//                    }
                    mDispatchService.onConnectionState(ConnectionState.CONNECTED);

                }
            } else if (state == DISCONNECTED) {

                if (mHasFirstLoginRoom) {
                    if(mLoginCallback != null)
                    {
                        mLoginCallback.onRTCCallback(errorCode);
                        mLoginCallback = null;
                    }

                    mHasFirstLoginRoom = true;
                }else {
//                    if(mConnectionStateChangeListener != null){
//                        mConnectionStateChangeListener.onConnectionState(ConnectionState.DISCONNECTED);
//                    }
                    mDispatchService.onConnectionState(ConnectionState.DISCONNECTED);
                }

            }else if(state == CONNECTING){
                if (!mHasFirstLoginRoom) {
//                    if(mConnectionStateChangeListener != null){
//                        mConnectionStateChangeListener.onConnectionState(ConnectionState.CONNECTING);
//                    }
                    mDispatchService.onConnectionState(ConnectionState.CONNECTING);

                    //断网则设置网络状态为差
//                    if (mEventCallback != null) {
//                        mEventCallback.onNetworkSpeedTestQualityUpdate(ZegoStreamQualityLevel.BAD);
//                    }
                    mDispatchService.onNetworkSpeedTestQualityUpdate(ZegoStreamQualityLevel.BAD);
                }
            }
        }

        @Override
        public void onRecvExperimentalAPI(String content) {
        }

        @Override
        public void onPlayerRecvVideoFirstFrame(String streamID) {
        }

        @Override
        public void onPlayerRecvAudioFirstFrame(String streamID) {
        }

        @Override
        public void onRemoteMicStateUpdate(String streamID, ZegoRemoteDeviceState zegoRemoteDeviceState) {
            super.onRemoteMicStateUpdate(streamID, zegoRemoteDeviceState);
            L.i(TAG,"streamID = " + streamID +" :   RemoteDeviceState = " + zegoRemoteDeviceState.name());
            if(zegoRemoteDeviceState == ZegoRemoteDeviceState.OPEN){
                mDispatchService.onRemoteMicStateUpdate(streamID,true);
            }else if(zegoRemoteDeviceState == ZegoRemoteDeviceState.MUTE){
                mDispatchService.onRemoteMicStateUpdate(streamID,false);
            }
        }

        @Override
        public void onAutoMixerSoundLevelUpdate(HashMap<String, Float> hashMap) {
            super.onAutoMixerSoundLevelUpdate(hashMap);
            mDispatchService.onSoundLevelUpdate(hashMap);
        }
    };

}
