package im.zego.chathouse.sdk.rtc.callbacks;

import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;

import im.zego.chathouse.sdk.constants.ConnectionState;
import im.zego.zegoexpress.constants.ZegoAudioRoute;
import im.zego.zegoexpress.constants.ZegoStreamQualityLevel;
import im.zego.zegoexpress.constants.ZegoUpdateType;
import im.zego.zegoexpress.entity.ZegoPlayStreamQuality;
import im.zego.zegoexpress.entity.ZegoPublishStreamQuality;
import im.zego.zegoexpress.entity.ZegoStream;

/**
 * RTC 事件回调
 */
public interface IRTCEventCallback {

    /**
     * 声浪信息更新
     * @param userVolumeMap 音浪集合
     */
    void onSoundLevelUpdate(HashMap<String, Float> userVolumeMap);

    /**
     * 手动混流声浪信息更新
     * @param userVolumeMap 音浪集合
     */
    void onMixSoundLevelUpdate(HashMap<Integer, Float> userVolumeMap);

    /**
     * 音频设备路由变更通知  当有耳机插拔、扬声器和听筒切换等音频路由发生变化时会抛出此回调。
     * @param audioRoute 音频设备
     */
    void onAudioRouteChange(ZegoAudioRoute audioRoute);

    /**
     * 拉流质量更新
     * @param streamID 流ID
     * @param quality 流质量
     */
    void onPlayerQualityUpdate(String streamID, ZegoPlayStreamQuality quality);

    /**
     * 相同房间内其他用户推的流增加或减少的通知
     * @param roomID 房间ID
     * @param updateType 更新类型 增加或者删除
     * @param streamList 流列表
     * @param extendedData 扩展数据
     */
    void onRoomStreamUpdate(String roomID, ZegoUpdateType updateType, ArrayList<ZegoStream> streamList, JSONObject extendedData);

    /**
     * 推流质量更新
     * @param streamID 流ID
     * @param quality 流质量
     */
    void onPublisherQualityUpdate(String streamID, ZegoPublishStreamQuality quality);

    /**
     * 网络测速更新
     *
     * @param streamQualityLevel 网络质量等级
     */
    void onNetworkSpeedTestQualityUpdate(ZegoStreamQualityLevel streamQualityLevel);

    /**
     * 用户网络质量更新
     * @param streamQualityLevel 流质量更新
     */
    void onUserNetQualityLevelUpdate(String userID,ZegoStreamQualityLevel streamQualityLevel);


    /**
     * 连接状态服务
     * @param state 连接状态
     */
    void onConnectionState(ConnectionState state);

    /**
     * 远程麦克风状态切换
     * @param streamID 流ID
     * @param open true 打开
     */
    void onRemoteMicStateUpdate(String streamID, boolean open);
}
