package im.zego.chathouse.sdk.zim;

import android.text.TextUtils;

import org.json.JSONObject;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import im.zego.zim.ZIM;
import im.zego.zim.callback.ZIMEventHandler;
import im.zego.zim.entity.ZIMCallInvitationAcceptedInfo;
import im.zego.zim.entity.ZIMCallInvitationCancelledInfo;
import im.zego.zim.entity.ZIMCallInvitationReceivedInfo;
import im.zego.zim.entity.ZIMCallInvitationRejectedInfo;
import im.zego.zim.entity.ZIMMessage;
import im.zego.zim.entity.ZIMRoomAttributesUpdateInfo;
import im.zego.zim.entity.ZIMUserInfo;
import im.zego.zim.enums.ZIMConnectionEvent;
import im.zego.zim.enums.ZIMConnectionState;
import im.zego.zim.enums.ZIMRoomEvent;
import im.zego.zim.enums.ZIMRoomState;

/**
 * ZIM 分发服务
 */
public class ZIMDispatchService {

    private final List<ZIMEventHandlerWrap> mZIMEventHandlerCallbacks =
            new ArrayList<>();

    public void registerZIMEventHandlerCallbacks(ZIMEventHandlerWrap callback) {
        synchronized (mZIMEventHandlerCallbacks) {
            mZIMEventHandlerCallbacks.add(callback);
        }
    }

    public void unregisterZIMEventHandlerCallbacks(ZIMEventHandlerWrap callback) {
        synchronized (mZIMEventHandlerCallbacks) {
            mZIMEventHandlerCallbacks.remove(callback);
        }
    }

    public void clearZIMEventHandlerCallbacksByRoomID(String roomID) {
        synchronized (mZIMEventHandlerCallbacks) {
            if (!TextUtils.isEmpty(roomID)) {

                for (Iterator<ZIMEventHandlerWrap> iterator = mZIMEventHandlerCallbacks.iterator(); iterator.hasNext(); ) {

                    ZIMEventHandlerWrap wrap = iterator.next();

                    if (roomID.equals(wrap.mRoomID)) {
                        iterator.remove();
                    }

                }
            }
        }
    }


    private Object[] collectZIMEventHandlerCallbacks() {
        Object[] callbacks = null;
        synchronized (mZIMEventHandlerCallbacks) {
            if (mZIMEventHandlerCallbacks.size() > 0) {
                Collections.sort(mZIMEventHandlerCallbacks);
                callbacks = mZIMEventHandlerCallbacks.toArray();
            }
        }
        return callbacks;
    }

    public void onConnectionStateChanged(ZIM zim, ZIMConnectionState
            state, ZIMConnectionEvent event, JSONObject extendedData) {
        Object[] callbacks = collectZIMEventHandlerCallbacks();
        if (callbacks != null) {
            for (int i = 0; i < callbacks.length; i++) {
                ((ZIMEventHandler) callbacks[i]).onConnectionStateChanged(zim,state,event,extendedData);
            }
        }
    }

    public void onReceiveRoomMessage(ZIM zim, ArrayList<ZIMMessage> messageList, String fromRoomID){
        Object[] callbacks = collectZIMEventHandlerCallbacks();
        if (callbacks != null) {
            for (int i = 0; i < callbacks.length; i++) {
                ((ZIMEventHandler) callbacks[i]).onReceiveRoomMessage(zim,messageList,fromRoomID);
            }
        }
    }

    public void onRoomMemberLeft(ZIM zim, ArrayList<ZIMUserInfo> memberList, String roomID){
        Object[] callbacks = collectZIMEventHandlerCallbacks();
        if (callbacks != null) {
            for (int i = 0; i < callbacks.length; i++) {
                ((ZIMEventHandler) callbacks[i]).onRoomMemberLeft(zim,memberList,roomID);
            }
        }
    }


    public void onRoomMemberJoined(ZIM zim, ArrayList< ZIMUserInfo > memberList, String roomID){
        Object[] callbacks = collectZIMEventHandlerCallbacks();
        if (callbacks != null) {
            for (int i = 0; i < callbacks.length; i++) {
                ((ZIMEventHandler) callbacks[i]).onRoomMemberJoined(zim,memberList,roomID);
            }
        }
    }

    public void onRoomStateChanged(ZIM zim, ZIMRoomState state, ZIMRoomEvent event, JSONObject extendedData, String roomID) {
        Object[] callbacks = collectZIMEventHandlerCallbacks();
        if (callbacks != null) {
            for (int i = 0; i < callbacks.length; i++) {
                ((ZIMEventHandler) callbacks[i]).onRoomStateChanged(zim,state,event,extendedData,roomID);
            }
        }
    }

    public void onRoomAttributesUpdated(ZIM zim, ZIMRoomAttributesUpdateInfo info, String roomID) {
        Object[] callbacks = collectZIMEventHandlerCallbacks();
        if (callbacks != null) {
            for (int i = 0; i < callbacks.length; i++) {
                ((ZIMEventHandler) callbacks[i]).onRoomAttributesUpdated(zim,info,roomID);
            }
        }
    }

    public void onCallInvitationRejected(ZIM zim, ZIMCallInvitationRejectedInfo info, String callID) {
        Object[] callbacks = collectZIMEventHandlerCallbacks();
        if (callbacks != null) {
            for (int i = 0; i < callbacks.length; i++) {
                ((ZIMEventHandler) callbacks[i]).onCallInvitationRejected(zim,info,callID);
            }
        }
    }

    public void onCallInvitationAccepted(ZIM zim, ZIMCallInvitationAcceptedInfo info, String callID) {
        Object[] callbacks = collectZIMEventHandlerCallbacks();
        if (callbacks != null) {
            for (int i = 0; i < callbacks.length; i++) {
                ((ZIMEventHandler) callbacks[i]).onCallInvitationAccepted(zim,info,callID);
            }
        }
    }

    public void onCallInvitationCancelled(ZIM zim, ZIMCallInvitationCancelledInfo info, String callID) {
        Object[] callbacks = collectZIMEventHandlerCallbacks();
        if (callbacks != null) {
            for (int i = 0; i < callbacks.length; i++) {
                ((ZIMEventHandler) callbacks[i]).onCallInvitationCancelled(zim,info,callID);
            }
        }
    }

    public void onCallInvitationReceived(ZIM zim, ZIMCallInvitationReceivedInfo info, String callID) {
        Object[] callbacks = collectZIMEventHandlerCallbacks();
        if (callbacks != null) {
            for (int i = 0; i < callbacks.length; i++) {
                ((ZIMEventHandler) callbacks[i]).onCallInvitationReceived(zim,info,callID);
            }
        }
    }

    public void onCallInvitationTimeout(ZIM zim, String callID) {
        Object[] callbacks = collectZIMEventHandlerCallbacks();
        if (callbacks != null) {
            for (int i = 0; i < callbacks.length; i++) {
                ((ZIMEventHandler) callbacks[i]).onCallInvitationTimeout(zim,callID);
            }
        }
    }

    public void onCallInviteesAnsweredTimeout(ZIM zim, ArrayList<String> invitees, String callID) {
        Object[] callbacks = collectZIMEventHandlerCallbacks();
        if (callbacks != null) {
            for (int i = 0; i < callbacks.length; i++) {
                ((ZIMEventHandler) callbacks[i]).onCallInviteesAnsweredTimeout(zim,invitees,callID);
            }
        }
    }

    public void onRoomOnlineMemberCountQueried(int count,int errorCode) {
        Object[] callbacks = collectZIMEventHandlerCallbacks();
        if (callbacks != null) {
            for (int i = 0; i < callbacks.length; i++) {
                ((ZIMEventHandlerWrap) callbacks[i]).onRoomOnlineMemberCountQueried(count,errorCode);
            }
        }
    }

    public void onReceivePeerCommandMessage(String commandMessage) {
        Object[] callbacks = collectZIMEventHandlerCallbacks();
        if (callbacks != null) {
            for (int i = 0; i < callbacks.length; i++) {
                ((ZIMEventHandlerWrap) callbacks[i]).onReceivePeerCommandMessage(commandMessage);
            }
        }
    }

    public void onReceivePeerTextMessage(String commandMessage) {
        Object[] callbacks = collectZIMEventHandlerCallbacks();
        if (callbacks != null) {
            for (int i = 0; i < callbacks.length; i++) {
                ((ZIMEventHandlerWrap) callbacks[i]).onReceivePeerTextMessage(commandMessage);
            }
        }
    }
}
