package im.zego.chathouse.sdk.zim;



import static im.zego.chathouse.ErrorCode.ERROR_FAIL_NETWORK;
import static im.zego.zim.enums.ZIMConversationType.PEER;

import android.app.Application;
import android.text.TextUtils;

import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;


import im.zego.chathouse.KeyCenter;
import im.zego.chathouse.protocol.ICommonResponseCallback;
import im.zego.chathouse.protocol.TokenManager;
import im.zego.chathouse.room.help.TimerTaskHelp;
import im.zego.chathouse.room.help.ZIMMessageHelp;
import im.zego.chathouse.sdk.zim.callback.IZIMCommonCallback;
import im.zego.chathouse.sdk.zim.callback.IZIMQueryRoomMemberBaseInfoListCallback;
import im.zego.chathouse.sdk.zim.callback.IZIMQueryRoomMemberIDListCallback;
import im.zego.chathouse.sdk.zim.callback.IZIMQueryRoomMemberListCallback;
import im.zego.enjoycommon.log.L;
import im.zego.enjoycommon.util.T;
import im.zego.zim.ZIM;
import im.zego.zim.callback.ZIMCallAcceptanceSentCallback;
import im.zego.zim.callback.ZIMCallCancelSentCallback;
import im.zego.zim.callback.ZIMCallInvitationSentCallback;
import im.zego.zim.callback.ZIMCallRejectionSentCallback;
import im.zego.zim.callback.ZIMEventHandler;
import im.zego.zim.callback.ZIMLoggedInCallback;
import im.zego.zim.callback.ZIMMessageSentCallback;
import im.zego.zim.callback.ZIMRoomAttributesOperatedCallback;
import im.zego.zim.callback.ZIMRoomAttributesQueriedCallback;
import im.zego.zim.callback.ZIMRoomCreatedCallback;
import im.zego.zim.callback.ZIMRoomJoinedCallback;
import im.zego.zim.callback.ZIMRoomLeftCallback;
import im.zego.zim.callback.ZIMRoomMemberQueriedCallback;
import im.zego.zim.callback.ZIMRoomOnlineMemberCountQueriedCallback;
import im.zego.zim.callback.ZIMTokenRenewedCallback;
import im.zego.zim.callback.ZIMUserAvatarUrlUpdatedCallback;
import im.zego.zim.callback.ZIMUserExtendedDataUpdatedCallback;
import im.zego.zim.callback.ZIMUsersInfoQueriedCallback;
import im.zego.zim.entity.ZIMAppConfig;
import im.zego.zim.entity.ZIMCallAcceptConfig;
import im.zego.zim.entity.ZIMCallCancelConfig;
import im.zego.zim.entity.ZIMCallInvitationAcceptedInfo;
import im.zego.zim.entity.ZIMCallInvitationCancelledInfo;
import im.zego.zim.entity.ZIMCallInvitationReceivedInfo;
import im.zego.zim.entity.ZIMCallInvitationRejectedInfo;
import im.zego.zim.entity.ZIMCallInvitationSentInfo;
import im.zego.zim.entity.ZIMCallInviteConfig;
import im.zego.zim.entity.ZIMCallRejectConfig;
import im.zego.zim.entity.ZIMCommandMessage;
import im.zego.zim.entity.ZIMError;
import im.zego.zim.entity.ZIMErrorUserInfo;
import im.zego.zim.entity.ZIMMessage;
import im.zego.zim.entity.ZIMMessageSendConfig;
import im.zego.zim.entity.ZIMRoomAdvancedConfig;
import im.zego.zim.entity.ZIMRoomAttributesSetConfig;
import im.zego.zim.entity.ZIMRoomAttributesUpdateInfo;
import im.zego.zim.entity.ZIMRoomFullInfo;
import im.zego.zim.entity.ZIMRoomInfo;
import im.zego.zim.entity.ZIMRoomMemberQueryConfig;
import im.zego.zim.entity.ZIMTextMessage;
import im.zego.zim.entity.ZIMUserFullInfo;
import im.zego.zim.entity.ZIMUserInfo;
import im.zego.zim.entity.ZIMUsersInfoQueryConfig;
import im.zego.zim.enums.ZIMConnectionEvent;
import im.zego.zim.enums.ZIMConnectionState;
import im.zego.zim.enums.ZIMConversationType;
import im.zego.zim.enums.ZIMMessagePriority;
import im.zego.zim.enums.ZIMRoomEvent;
import im.zego.zim.enums.ZIMRoomState;

/**
 * ZIM封装类
 */
public class ZIMSDKManager {

    private static final String TAG = "ZIMSDKManager";

    private static ZIMSDKManager sdkManager;
    public ZIM zim;

    private String mUserID = null;

    private final ZIMDispatchService mDispatchService = new ZIMDispatchService();

    public static ZIMSDKManager getInstance() {
        if (sdkManager == null) {
            synchronized (ZIMSDKManager.class) {
                if (sdkManager == null) {
                    sdkManager = new ZIMSDKManager();
                }
            }
        }
        return sdkManager;
    }

    /**
     * 初始化:
     * @param application 上下文
     */
    public void init(Application application) {
        L.i(TAG,"init");
        if(zim == null) {
            ZIMAppConfig config = new ZIMAppConfig();
            config.appID = KeyCenter.APP_ID;
            config.appSign = KeyCenter.APP_SIGN;
            zim = ZIM.create(config, application);
            L.i(TAG,"init zim:" + zim);
            if(zim == null){
//                T.show("zim 创建失败");
                L.i(TAG,"zim create fail");
            }
        }

        if(zim != null) {
            zim.setEventHandler(eventHandler);
        }
    }

    /**
     * 反初始化SDK
     */
    public void unInit(){
        L.i(TAG,"unInit");
        updateConnectionState(ZIMConnectionState.DISCONNECTED);

        if(zim != null){
            zim.destroy();
            zim = null;
        }

        mUserID = null;
    }

    /**
     * 登录
     * @param userID 用户ID
     * @param userName 用户昵称
     * @param zimCommonCallback 回调
     */
    public void login(String userID, String userName,String token, IZIMCommonCallback zimCommonCallback){
        L.i(TAG,"login");
        if(zim == null){
            L.i(TAG,"login zim is NULL");
            if(zimCommonCallback != null) {
                zimCommonCallback.onZIMCallback(-1);
            }
            return;
        }

        // 登录时，需要开发者 按照 基础功能中的 "使用 Token 鉴权" 文档生成 token 即可
        ZIMUserInfo zimUserInfo = new ZIMUserInfo();
        zimUserInfo.userID = userID;
        zimUserInfo.userName = userName;

        zim.login(zimUserInfo,  null, new ZIMLoggedInCallback() {
            @Override
            public void onLoggedIn(ZIMError error) {
                // 开发者可根据 ZIMError 来判断是否登录成功。
                if(error.code.value() == 0){
                    mUserID = userID;
                }else {
                    mUserID = null;
                }
                if(zimCommonCallback != null) {
                    zimCommonCallback.onZIMCallback(error.code.value());
                }
                L.i(TAG,"login:" + error.code.value());
            }
        });
    }

    /**
     * 登出
     */
    public void logout(){
        L.i(TAG,"logout");
        updateConnectionState(ZIMConnectionState.DISCONNECTED);

        if(zim != null) {
            zim.logout();
        }
    }

    /**
     * 创建并加入一个房间
     * @param roomID 房间ID
     * @param roomName 房间昵称
     * @param zimCommonCallback 回调
     */
    public void createRoom(String roomID,String roomName,IZIMCommonCallback zimCommonCallback){
        L.i(TAG,"createRoom");
        if(zim == null){
            L.i(TAG,"createRoom zim is NULL");
            if(zimCommonCallback != null) {
                zimCommonCallback.onZIMCallback(-1);
            }
            return;
        }

        ZIMRoomInfo zimRoomInfo = new ZIMRoomInfo();
        zimRoomInfo.roomID = roomID;
        zimRoomInfo.roomName = roomName;


        ZIMRoomAdvancedConfig config = new ZIMRoomAdvancedConfig();
        // ZIM 的 createRoom 意为：创建并加入一个房间
        // https://doc-zh.zego.im/article/api?doc=zim_API~java_android~class~ZIM#create-room
        zim.createRoom(zimRoomInfo,config, new ZIMRoomCreatedCallback() {
            @Override
            public void onRoomCreated(ZIMRoomFullInfo zimRoomFullInfo, ZIMError error) {
                if(zimCommonCallback != null) {
                    zimCommonCallback.onZIMCallback(error.code.value());
                }
                L.i(TAG,"createRoom:" + "roomID:" + roomID +":error:" + error.code.value());
                L.i(TAG,"createRoom:" + "roomID:" + roomID +":error:" + error.message);
            }
        });
    }

    /**
     * 加入房间
     * @param roomID 房间ID
     * @param zimCommonCallback 回调
     */
    public void joinRoom(String roomID,IZIMCommonCallback zimCommonCallback){
        L.i(TAG,"joinRoom");
        if(zim == null){
            L.i(TAG,"joinRoom zim is NULL");
            if(zimCommonCallback != null) {
                zimCommonCallback.onZIMCallback(-1);
            }
            return;
        }

        zim.joinRoom(roomID,new ZIMRoomJoinedCallback() {
            @Override
            public void onRoomJoined(ZIMRoomFullInfo zimRoomFullInfo, ZIMError error) {
                if(zimCommonCallback != null) {
                    zimCommonCallback.onZIMCallback(error.code.value());
                }
                L.i(TAG,"loginRoom:" + "roomID:" + roomID +":error:" +  error.code.value());
                L.i(TAG,"loginRoom:" + "roomID:" + roomID +":error:" +  error.message);
            }
        });
    }


    /**
     * 离开房间
     * @param roomID 房间ID
     * @param zimCommonCallback 回调
     */
    public void logoutRoom(String roomID,IZIMCommonCallback zimCommonCallback){
        L.i(TAG,"logoutRoom");
        if(zim != null) {
            zim.leaveRoom(roomID, new ZIMRoomLeftCallback() {
                @Override
                public void onRoomLeft(String roomID, ZIMError errorInfo) {
                    if (zimCommonCallback != null) {
                        zimCommonCallback.onZIMCallback(errorInfo.code.value());
                    }
                    L.i(TAG, "logoutRoom:" + "roomID:" + roomID + ":error:" + errorInfo.code.value());
                }
            });
        }else {
            L.i(TAG,"logoutRoom zim is NULL");
            if (zimCommonCallback != null) {
                zimCommonCallback.onZIMCallback(-1);
            }
        }
    }

    /**
     * 更新连接状态并回调
     * @param state 状态
     */
    private void updateConnectionState(ZIMConnectionState state){
        mDispatchService.onConnectionStateChanged(zim,state,null,null);
    }

    /**
     * 发送文本消息
     * @param message 消息
     * @param toUserID 目标用户
     * @param callback 回调
     */
    public void sendTextMessage(String message,String toUserID,IZIMCommonCallback callback){
        L.i(TAG,"sendTextMessage");
        if(zim != null) {
            ZIMTextMessage zimTextMessage = new ZIMTextMessage();
            zimTextMessage.message = message;

            ZIMMessageSendConfig config = new ZIMMessageSendConfig();
            config.priority = ZIMMessagePriority.HIGH;
            zim.sendMessage(zimTextMessage, toUserID, PEER,config, new ZIMMessageSentCallback() {
                @Override
                public void onMessageAttached(ZIMMessage message) {

                }

                @Override
                public void onMessageSent(ZIMMessage message, ZIMError errorInfo) {
                    L.i(TAG, "sendTextMessage : " + zimTextMessage.message + " :  toUserID = " + toUserID + " : errorCode = " + errorInfo.code.value());
                    if (callback != null) {
                        callback.onZIMCallback(errorInfo.code.value());
                    }
                }
            });
        }

    }


    /**
     * 发送命令消息
     * @param message 消息
     * @param toUserID 目标用户ID
     * @param callback 回调
     */
    public void sendCommandMessage(String message,String toUserID,IZIMCommonCallback callback){
        L.i(TAG,"sendCommandMessage");
        if(zim != null) {
            ZIMCommandMessage zimCommandMessage = new ZIMCommandMessage();
            zimCommandMessage.message = message.getBytes();

            ZIMMessageSendConfig config = new ZIMMessageSendConfig();
            config.priority = ZIMMessagePriority.HIGH;
            zim.sendMessage(zimCommandMessage, toUserID, PEER,config, new ZIMMessageSentCallback() {
                @Override
                public void onMessageAttached(ZIMMessage message) {

                }

                @Override
                public void onMessageSent(ZIMMessage message, ZIMError errorInfo) {
                    L.i(TAG, "sendTextMessage : " + message + " :  toUserID = " + toUserID + " : errorCode = " + errorInfo.code.value());
                    if (callback != null) {
                        callback.onZIMCallback(errorInfo.code.value());
                    }
                }
            });
        }

    }

    /**
     * 设置房间属性
     * @param hashMap 集合
     * @param roomID 房间ID
     * @param callback 回调
     */
    public void setRoomAttributes(HashMap<String,String> hashMap,String roomID,IZIMCommonCallback callback){
        L.i(TAG,"setRoomAttributes");
        if(zim != null) {
            ZIMRoomAttributesSetConfig config = new ZIMRoomAttributesSetConfig();
            config.isUpdateOwner = true;
            config.isForce = true;
            config.isDeleteAfterOwnerLeft = false;
            if (hashMap != null) {
                L.i("setRoomAttributes", "setRoomAttributes : ------ ");
                Set<String> keySet = hashMap.keySet();
                for (String key : keySet) {
                    String value = hashMap.get(key);
                    L.i("setRoomAttributes", "setRoomAttributes     key = " + key + " , value = " + value);
                }
                L.i("setRoomAttributes", "setRoomAttributes : ------ ");
            }
            zim.setRoomAttributes(hashMap, roomID, config, new ZIMRoomAttributesOperatedCallback() {
                @Override
                public void onRoomAttributesOperated(String roomID, ArrayList<String> errorKeys, ZIMError errorInfo) {
                    L.i(TAG, "setRoomAttributes : " + errorInfo.code.value());
                    if (callback != null) {
                        callback.onZIMCallback(errorInfo.code.value());
                    }
                }
            });
        }
    }

    /**
     * 查询房间属性
     * @param roomID 房间ID
     * @param callback 回调
     */
    public void queryRoomAllAttributes(String roomID,ZIMRoomAttributesQueriedCallback callback) {
        L.i(TAG,"queryRoomAllAttributes");
        if(zim != null) {
            zim.queryRoomAllAttributes(roomID, new ZIMRoomAttributesQueriedCallback() {
                @Override
                public void onRoomAttributesQueried(String roomID, HashMap<String, String> roomAttributes, ZIMError errorInfo) {
                    L.i(TAG,"queryRoomAllAttributes  : code = " + errorInfo.code);
                    if (callback != null) {
                        callback.onRoomAttributesQueried(roomID, roomAttributes, errorInfo);
                    }
                }
            });
        }
    }

    /**
     * 设置用户头像
     */
    public void updateUserAvatarUrl(String userIcon,IZIMCommonCallback callback){
        L.i(TAG,"updateUserExtendedData");
        if(zim != null){
            zim.updateUserAvatarUrl(userIcon, new ZIMUserAvatarUrlUpdatedCallback() {
                        @Override
                        public void onUserAvatarUrlUpdated(String userAvatarUrl, ZIMError errorInfo) {
                             L.i(TAG,"updateUserAvatarUrl  userAvatarUrl = " + userAvatarUrl + " : code = " + errorInfo.code);
                            if(callback != null){
                                callback.onZIMCallback(errorInfo.code.value());
                            }
                        }
                    }

            );
        }
    }

    /**
     * 设置用户扩展数据
     */
    public void updateUserExtendedData(String data,IZIMCommonCallback callback){
        L.i(TAG,"updateUserExtendedData");
        if(zim != null){
            zim.updateUserExtendedData(data, new ZIMUserExtendedDataUpdatedCallback() {
                @Override
                public void onUserExtendedDataUpdated(String extendedData, ZIMError errorInfo) {
                    L.i(TAG,"updateUserExtendedData  extendedData = " + extendedData + " : code = " + errorInfo.code);
                    if(callback != null){
                        callback.onZIMCallback(errorInfo.code.value());
                    }
                }
            });
        }
    }

    /**
     * 查询房间用户列表
     * @param roomID 房间ID
     */
    public void queryRoomMemberListInRoom(String roomID, IZIMQueryRoomMemberIDListCallback callback){
        L.i(TAG,"queryRoomMemberListInRoom");
        if(zim != null) {
            ZIMRoomMemberQueryConfig config = new ZIMRoomMemberQueryConfig();
            config.count = 2000;
            zim.queryRoomMemberList(roomID, config, new ZIMRoomMemberQueriedCallback() {
                @Override
                public void onRoomMemberQueried(String roomID, ArrayList<ZIMUserInfo> memberList, String nextFlag, ZIMError errorInfo) {

                    L.i(TAG, "queryRoomMemberListInRoom code = " + errorInfo.code);
                    if (errorInfo.code.value() == 0) {
                        ArrayList<String> list = new ArrayList<>();
                        for (ZIMUserInfo zimUserInfo : memberList) {

                            list.add(zimUserInfo.userID);

                            L.i(TAG, "queryRoomMemberListInRoom code = " + errorInfo.code + " ： userID = " + zimUserInfo.userID + " : userName = " + zimUserInfo.userName
                            );
                        }
                        if(callback != null){
                            callback.onRoomMemberQueried(list,nextFlag,errorInfo.code.value());
                        }
                    }else{

                        if(callback != null){
                            callback.onRoomMemberQueried(null,nextFlag,errorInfo.code.value());
                        }

                    }
                }
            });


        }
    }


    /**
     * 查询房间用户列表包含基本信息
     * @param roomID 房间ID
     */
    public void queryRoomMemberListForBaseInfo(String roomID, IZIMQueryRoomMemberBaseInfoListCallback callback){
        L.i(TAG,"queryRoomMemberListInRoom");
        if(zim != null) {
            ZIMRoomMemberQueryConfig config = new ZIMRoomMemberQueryConfig();
            config.count = 2000;
            zim.queryRoomMemberList(roomID, config, new ZIMRoomMemberQueriedCallback() {
                @Override
                public void onRoomMemberQueried(String roomID, ArrayList<ZIMUserInfo> memberList, String nextFlag, ZIMError errorInfo) {

                    L.i(TAG, "queryRoomMemberListInRoom code = " + errorInfo.code);
                    if (errorInfo.code.value() == 0) {
                        if(callback != null){
                            callback.onRoomMemberQueried(memberList,nextFlag,errorInfo.code.value());
                        }
                    }else{

                        if(callback != null){
                            callback.onRoomMemberQueried(null,nextFlag,errorInfo.code.value());
                        }

                    }
                }
            });


        }
    }
    /**
     * 查询房间用户列表
     * @param roomID 房间ID
     */
    public void queryRoomMemberList(String roomID, IZIMQueryRoomMemberListCallback callback){
        L.i(TAG,"queryRoomMemberList");
        if(zim != null) {
            ZIMRoomMemberQueryConfig config = new ZIMRoomMemberQueryConfig();
            config.count = 2000;
            zim.queryRoomMemberList(roomID, config, new ZIMRoomMemberQueriedCallback() {
                @Override
                public void onRoomMemberQueried(String roomID, ArrayList<ZIMUserInfo> memberList, String nextFlag, ZIMError errorInfo) {

                    if (errorInfo.code.value() == 0) {
                        ArrayList<String> list = new ArrayList<>();
                        for (ZIMUserInfo zimUserInfo : memberList) {

                            list.add(zimUserInfo.userID);

                            L.i(TAG, "queryRoomMemberList code = " + errorInfo.code + " ： userID = " + zimUserInfo.userID + " : userName = " + zimUserInfo.userName
                            );
                        }
                        queryUsersInfo(list,callback);

//                        ZIMUsersInfoQueryConfig config = new ZIMUsersInfoQueryConfig();
//                        config.isQueryFromServer = false;
//                        zim.queryUsersInfo(list,config, new ZIMUsersInfoQueriedCallback() {
//                            @Override
//                            public void onUsersInfoQueried(ArrayList<ZIMUserFullInfo> userList, ArrayList<ZIMErrorUserInfo> errorUserList, ZIMError errorInfo) {
//                                for (ZIMUserFullInfo zimUserFullInfo : userList) {
//
//                                    L.i(TAG, "queryUsersInfo code = " + errorInfo.code + " ： userID = " + zimUserFullInfo.baseInfo.userID + " : userName = " + zimUserFullInfo.baseInfo.userName
//                                            + " : extendedData = " + zimUserFullInfo.extendedData);
//                                }
//
//                                if(errorInfo.code.value() == 0){
//                                    if(zim != null) {
//                                        ZIMUsersInfoQueryConfig config = new ZIMUsersInfoQueryConfig();
//                                        config.isQueryFromServer = true;
//                                        ArrayList<String> list = new ArrayList<>();
//                                        for(ZIMErrorUserInfo userInfo:errorUserList){
//                                            if(userInfo != null){
//                                                list.add(userInfo.userID);
//                                            }
//                                        }
//                                        queryServerUsersInfo(list, new IZIMQueryRoomMemberListCallback() {
//                                            @Override
//                                            public void onRoomMemberQueried(ArrayList<ZIMUserFullInfo> userList2, String nextFlag, int errorCode) {
//                                                if(callback != null){
//                                                    userList.addAll(userList2);
//                                                    callback.onRoomMemberQueried(userList,null,errorInfo.code.value());
//                                                }
//                                            }
//                                        });
//
//                                    }
//
//
//                                }else {
//
//
//                                    if (callback != null) {
//                                        callback.onRoomMemberQueried(userList, nextFlag, errorInfo.code.value());
//                                    }
//                                }
//
//                            }
//                        });
                    }else{

                        if(callback != null){
                            callback.onRoomMemberQueried(null,nextFlag,errorInfo.code.value());
                        }

                    }
                }
            });


        }
    }

    /**
     * 查询用户信息
     * @param list 列表
     * @param callback 回调
     */
    public void queryUsersInfo(ArrayList<String> list,IZIMQueryRoomMemberListCallback callback){
        L.i(TAG,"queryUsersInfo");
        if(zim != null){
            ZIMUsersInfoQueryConfig config = new ZIMUsersInfoQueryConfig();
            config.isQueryFromServer = false;
            zim.queryUsersInfo(list,config, new ZIMUsersInfoQueriedCallback() {
                @Override
                public void onUsersInfoQueried(ArrayList<ZIMUserFullInfo> userList, ArrayList<ZIMErrorUserInfo> errorUserList, ZIMError errorInfo) {
                    for (ZIMUserFullInfo zimUserFullInfo : userList) {

                        L.i(TAG, "queryUsersInfo code = " + errorInfo.code + " ： userID = " + zimUserFullInfo.baseInfo.userID + " : userName = " + zimUserFullInfo.baseInfo.userName
                                + " : extendedData = " + zimUserFullInfo.userAvatarUrl);
                    }
                    for (ZIMErrorUserInfo zimErrorUserInfo : errorUserList) {

                        L.i(TAG, "queryUsersInfo code = " + errorInfo.code + " ： userID = " + zimErrorUserInfo.userID + " : userName = " + zimErrorUserInfo.reason);
                    }

                    if(errorInfo.code.value() == 0){
                        ArrayList<String> list = new ArrayList<>();
                        for(ZIMErrorUserInfo userInfo:errorUserList){
                            if(userInfo != null){
                                list.add(userInfo.userID);
                            }
                        }

                        for(ZIMUserFullInfo zimUserFullInfo :userList){
                            if(zimUserFullInfo != null && TextUtils.isEmpty(zimUserFullInfo.userAvatarUrl)){
                                list.add(zimUserFullInfo.baseInfo.userID);
                            }
                        }

                        if(list.size() > 0) {
                            queryServerUsersInfo(list, new IZIMQueryRoomMemberListCallback() {
                                @Override
                                public void onRoomMemberQueried(ArrayList<ZIMUserFullInfo> userList2, String nextFlag, int errorCode) {
                                    if (callback != null) {

                                        for(ZIMUserFullInfo user2 : userList2){

                                            for (Iterator<ZIMUserFullInfo> iterator = userList.iterator(); iterator.hasNext();) {
                                                ZIMUserFullInfo user =  iterator.next();
                                                if(user.baseInfo.userID != null && user.baseInfo.userID.equals(user2.baseInfo.userID)){
                                                    iterator.remove();
                                                }
                                            }

                                        }

                                        userList.addAll(userList2);
                                        callback.onRoomMemberQueried(userList, null, errorInfo.code.value());
                                    }
                                }
                            });
                        }else {
                            if (callback != null) {
                                callback.onRoomMemberQueried(userList, null, errorInfo.code.value());
                            }
                        }
                    }else {

                        if (callback != null) {
                            callback.onRoomMemberQueried(userList, null, errorInfo.code.value());
                        }
                    }

                }
            });
        }
    }

    /**
     * 查询用户信息
     * @param list 列表
     * @param callback 回调
     */
    public void queryLocalUsersInfo(ArrayList<String> list,IZIMQueryRoomMemberListCallback callback){
        L.i(TAG,"queryUsersInfo");
        if(zim != null){
            ZIMUsersInfoQueryConfig config = new ZIMUsersInfoQueryConfig();
            config.isQueryFromServer = false;
            zim.queryUsersInfo(list,config, new ZIMUsersInfoQueriedCallback() {
                @Override
                public void onUsersInfoQueried(ArrayList<ZIMUserFullInfo> userList, ArrayList<ZIMErrorUserInfo> errorUserList, ZIMError errorInfo) {
                    for (ZIMUserFullInfo zimUserFullInfo : userList) {

                        L.i(TAG, "queryLocalUsersInfo code = " + errorInfo.code + " ： userID = " + zimUserFullInfo.baseInfo.userID + " : userName = " + zimUserFullInfo.baseInfo.userName
                                + " : extendedData = " + zimUserFullInfo.extendedData);
                    }

                    if(callback != null){
                        callback.onRoomMemberQueried(userList,null,errorInfo.code.value());

                    }

                }
            });
        }
    }

    /**
     * 查询用户信息
     * @param list 列表
     * @param callback 回调
     */
    public void queryServerUsersInfo(ArrayList<String> list,IZIMQueryRoomMemberListCallback callback){
        L.i(TAG,"queryServerUsersInfo");
        if(zim != null){
            ZIMUsersInfoQueryConfig config = new ZIMUsersInfoQueryConfig();
            config.isQueryFromServer = true;
            zim.queryUsersInfo(list,config, new ZIMUsersInfoQueriedCallback() {
                @Override
                public void onUsersInfoQueried(ArrayList<ZIMUserFullInfo> userList, ArrayList<ZIMErrorUserInfo> errorUserList, ZIMError errorInfo) {
                    for (ZIMUserFullInfo zimUserFullInfo : userList) {

                        L.i(TAG, "queryServerUsersInfo code = " + errorInfo.code + " ： userID = " + zimUserFullInfo.baseInfo.userID + " : userName = " + zimUserFullInfo.baseInfo.userName
                                + " : extendedData = " + zimUserFullInfo.extendedData);
                    }

                    int errorCode = errorInfo.code.value();
                    if(errorCode == 6000004 || errorCode == 6000005 || errorCode == 6000006 || errorCode == 6000012){
                        T.show("ZIM 查询用户信息接口 超过调用频率");
                    }

                    if(callback != null){
                        callback.onRoomMemberQueried(userList,null,errorInfo.code.value());

                    }

                }
            });
        }
    }


    /**
     * 查询房间总人数
     * @param roomID 房间ID
     */
    public void queryRoomOnlineMemberCount(String roomID){
        L.i(TAG,"queryRoomOnlineMemberCount");
        if(zim != null){
            zim.queryRoomOnlineMemberCount(roomID, new ZIMRoomOnlineMemberCountQueriedCallback() {
                @Override
                public void onRoomOnlineMemberCountQueried(String roomID, int count, ZIMError errorInfo) {
                    L.i(TAG,"queryRoomOnlineMemberCount count = " + count + " : errorCode  = " + errorInfo.code.value());
                    mDispatchService.onRoomOnlineMemberCountQueried(count,errorInfo.code.value());
                }
            });
        }
    }

    /**
     * 发送呼叫邀请
     * @param inviteUserID 邀请ID
     * @param timeout 超时时间
     */
    public void callInvite(String inviteUserID,String extendedData,int timeout,ZIMCallInvitationSentCallback callback){
        L.i(TAG,"callInvite");
        if(zim != null) {
            // 发送呼叫邀请
            List<String> invitees = new ArrayList<>();  // 被邀请人列表
            invitees.add(inviteUserID);       // 被邀请人id
            ZIMCallInviteConfig config = new ZIMCallInviteConfig();
            config.extendedData = extendedData;
            config.timeout = timeout; //邀请超时时间，单位为秒，范围1-600
            zim.callInvite(invitees, config, new ZIMCallInvitationSentCallback() {
                @Override
                public void onCallInvitationSent(String callID, ZIMCallInvitationSentInfo info, ZIMError errorInfo) {
                    // 此处的 callID 是用户发起呼叫后，SDK 内部生成的 ID，用于唯一标识一次呼叫邀请；之后发起人取消呼叫、被邀请人接受/拒绝呼叫，都会使用此 callID
                    L.i(TAG, "callInvite callID = " + callID + " : errorCode = " + errorInfo.code.value());
                    if (callback != null) {
                        callback.onCallInvitationSent(callID, info, errorInfo);
                    }
                }
            });
        }
    }

    /**
     * 取消呼叫邀请
     * @param inviteUserID 邀请用户ID
     * @param callID 呼叫ID
     */
    public void callCancel(String inviteUserID,String callID,String extendedData,ZIMCallCancelSentCallback callback){
        L.i(TAG,"callCancel");
        if(zim != null) {
            // 取消呼叫邀请
            List<String> invitees = new ArrayList<>();  // 被邀请人列表
            invitees.add(inviteUserID);       // 被邀请人id
            ZIMCallCancelConfig config = new ZIMCallCancelConfig();
            config.extendedData = extendedData;
            zim.callCancel(invitees, callID, config, new ZIMCallCancelSentCallback() {
                @Override
                public void onCallCancelSent(String callID, ArrayList<String> errorInvitees, ZIMError errorInfo) {
                    L.i(TAG, "callCancel callID = " + callID + " : errorCode = " + errorInfo.code.value());
                    if (callback != null) {
                        callback.onCallCancelSent(callID, errorInvitees, errorInfo);
                    }
                }
            });
        }
    }

    /**
     * 接受呼叫邀请
     * @param callID 呼叫ID
     */
    public void callAccept(String callID,String extendedData,ZIMCallAcceptanceSentCallback callback){
        L.i(TAG,"callAccept");
        if(zim != null) {
            ZIMCallAcceptConfig config = new ZIMCallAcceptConfig();
            config.extendedData = extendedData;
            zim.callAccept(callID, config, new ZIMCallAcceptanceSentCallback() {
                @Override
                public void onCallAcceptanceSent(String callID, ZIMError errorInfo) {
                    L.i(TAG, "callAccept callID = " + callID + " : errorCode = " + errorInfo.code.value());
                    if (callback != null) {
                        callback.onCallAcceptanceSent(callID, errorInfo);
                    }
                }
            });
        }
    }

    /**
     * 拒绝呼叫邀请
     * @param callID 呼叫ID
     * @param extendedData 附加信息
     */
    public void callReject(String callID,String extendedData,ZIMCallRejectionSentCallback callback){
        L.i(TAG,"callReject");
        if(zim != null) {
            ZIMCallRejectConfig config = new ZIMCallRejectConfig();
            config.extendedData = extendedData;
            zim.callReject(callID, config, new ZIMCallRejectionSentCallback() {
                @Override
                public void onCallRejectionSent(String callID, ZIMError errorInfo) {
                    L.i(TAG, "callReject callID = " + callID + " : errorCode = " + errorInfo.code.value());
                    if (callback != null) {
                        callback.onCallRejectionSent(callID, errorInfo);
                    }
                }
            });
        }
    }

    public void renewToken(){
        L.i(TAG,"renewToken");
        if (zim != null) {
            zim.renewToken(TokenManager.getInstance().getToken(), new ZIMTokenRenewedCallback() {
                @Override
                public void onTokenRenewed(String token, ZIMError errorInfo) {
                    // 开发者可根据 ZIMError 来判断更新 Token 是否成功
                    L.i(TAG, "zim renewToken:" + errorInfo.code);
                }
            });
        }
    }

    /**
     * 获取 ZIM 事件分发服务
     * @return
     */
    public ZIMDispatchService getDispatchService() {
        return mDispatchService;
    }

    private final ZIMEventHandler eventHandler = new ZIMEventHandler() {
        /**
         * 网路连接状态回调
         * @param state 状态
         * @param event 时间
         * @param extendedData 附加信息
         */
        @Override
        public void onConnectionStateChanged(ZIM zim, ZIMConnectionState
        state, ZIMConnectionEvent event, JSONObject extendedData) {
            super.onConnectionStateChanged(zim, state, event, extendedData);
            mDispatchService.onConnectionStateChanged(zim,state, event, extendedData);
            L.e(TAG, "onConnectionStateChanged:" + state.name());
        }

        @Override
        public void onError(ZIM zim, ZIMError errorInfo) {
            super.onError(zim, errorInfo);
            if(errorInfo.code.value() == 6000003){
                T.show("无效 appID,请确认此 appID 已开通 ZIM 服务");
            }
            L.e(TAG,"onError: code :" + errorInfo.code + " : message : " + errorInfo.message);
        }

        /**
         * token即将过期的回调
         */
        @Override
        public void onTokenWillExpire(ZIM zim, int second) {
            super.onTokenWillExpire(zim, second);
            TokenManager.getInstance().getToken(new ICommonResponseCallback() {
                @Override
                public void onResponse(int errorCode) {
                    if (errorCode == 0) {
                        renewToken();
                    }else if(errorCode == ERROR_FAIL_NETWORK) {

                        TimerTaskHelp.refreshToken(new ICommonResponseCallback() {
                            @Override
                            public void onResponse(int errorCode) {
                                if(errorCode == 0){
                                    renewToken();
                                }
                            }
                        });

                    }
                }
            });

        }

        /**
         * 接收1对1消息
         * @param messageList 消息列表
         * @param fromUserID 用户ID
         */
        @Override
        public void onReceivePeerMessage(ZIM zim, ArrayList< ZIMMessage > messageList, String fromUserID) {
            super.onReceivePeerMessage(zim, messageList, fromUserID);
            for(ZIMMessage zimMessage: messageList){
                if (zimMessage instanceof ZIMTextMessage) {
                    ZIMTextMessage zimTextMessage = (ZIMTextMessage) zimMessage;
                    L.i(TAG, "onReceivePeerMessage:" + zimTextMessage.message);
                    ZIMMessageHelp.disposeMessage(zimTextMessage.message);
                    mDispatchService.onReceivePeerTextMessage(zimTextMessage.message);
                }else if(zimMessage instanceof ZIMCommandMessage){
                    ZIMCommandMessage zimCommandMessage = (ZIMCommandMessage) zimMessage;
                    L.i(TAG, "onReceivePeerMessage zimCommandMessage:" + new String(zimCommandMessage.message));
                    String text = new String(zimCommandMessage.message);
                    mDispatchService.onReceivePeerCommandMessage(text);
                }
            }
        }

        /**
         * 房间消息回调
         * @param messageList 消息列表
         * @param fromRoomID 房间ID
         */
        @Override
        public void onReceiveRoomMessage(ZIM zim, ArrayList<ZIMMessage> messageList, String fromRoomID) {
            super.onReceiveRoomMessage(zim, messageList, fromRoomID);

            for(ZIMMessage message :messageList){
                L.i(TAG,"onReceiveRoomMessage " + message.getSenderUserID());
            }

            mDispatchService.onReceiveRoomMessage(zim,messageList,fromRoomID);
        }

        @Override
        public void onRoomMemberJoined(ZIM zim, ArrayList< ZIMUserInfo > memberList, String roomID) {
            super.onRoomMemberJoined(zim, memberList, roomID);

            for(ZIMUserInfo userInfo:memberList){
                L.i(TAG,"onRoomMemberJoined :userID = " + userInfo.userID + " : userName = "+ userInfo.userName);
            }

            mDispatchService.onRoomMemberJoined(zim,memberList,roomID);
        }

        @Override
        public void onRoomMemberLeft(ZIM zim, ArrayList<ZIMUserInfo> memberList, String roomID) {
            super.onRoomMemberLeft(zim, memberList, roomID);

            for(ZIMUserInfo userInfo:memberList){
                L.i(TAG,"onRoomMemberLeft :userID = " + userInfo.userID + " : userName = "+ userInfo.userName);
            }

            mDispatchService.onRoomMemberLeft(zim,memberList,roomID);
        }

        @Override
        public void onRoomStateChanged(ZIM zim, ZIMRoomState state, ZIMRoomEvent event, JSONObject extendedData, String roomID) {
            super.onRoomStateChanged(zim, state, event, extendedData, roomID);
            L.i(TAG,"onRoomStateChanged  state : " + state.name()  + ": event :" + event.name());
            mDispatchService.onRoomStateChanged(zim,state,event,extendedData,roomID);
        }

        @Override
        public void onRoomAttributesUpdated(ZIM zim, ZIMRoomAttributesUpdateInfo info, String roomID) {
            super.onRoomAttributesUpdated(zim, info, roomID);
            Map<String,String> map = info.roomAttributes;
            if(map != null) {
                for (Map.Entry<String, String> entry : map.entrySet()) {
                    L.i(TAG, "onRoomAttributesUpdated  key : " + entry.getKey() + ": value :" + entry.getValue());
                }
            }

            mDispatchService.onRoomAttributesUpdated(zim,info,roomID);
        }

        @Override
        public void onRoomAttributesBatchUpdated(ZIM zim, ArrayList<ZIMRoomAttributesUpdateInfo> info, String roomID) {
            super.onRoomAttributesBatchUpdated(zim, info, roomID);
        }

        @Override
        public void onCallInvitationRejected(ZIM zim, ZIMCallInvitationRejectedInfo info, String callID) {
            super.onCallInvitationRejected(zim, info, callID);
            L.i(TAG,"onCallInvitationRejected info = " + info.extendedData + "  : callID = " + callID);
            mDispatchService.onCallInvitationRejected(zim,info,callID);
        }

        @Override
        public void onCallInvitationAccepted(ZIM zim, ZIMCallInvitationAcceptedInfo info, String callID) {
            super.onCallInvitationAccepted(zim, info, callID);
            L.i(TAG,"onCallInvitationAccepted info = " + info.extendedData + "  : callID = " + callID);
            mDispatchService.onCallInvitationAccepted(zim,info,callID);
        }

        @Override
        public void onCallInvitationCancelled(ZIM zim, ZIMCallInvitationCancelledInfo info, String callID) {
            super.onCallInvitationCancelled(zim, info, callID);
            L.i(TAG,"onCallInvitationCancelled info = " + info.extendedData + "  : callID = " + callID);
            mDispatchService.onCallInvitationCancelled(zim,info,callID);
        }

        @Override
        public void onCallInvitationReceived(ZIM zim, ZIMCallInvitationReceivedInfo info, String callID) {
            super.onCallInvitationReceived(zim, info, callID);

            L.i(TAG,"onCallInvitationReceived info = " + info.extendedData + "  : callID = " + callID);

            mDispatchService.onCallInvitationReceived(zim,info,callID);
        }

        @Override
        public void onCallInvitationTimeout(ZIM zim, String callID) {
            super.onCallInvitationTimeout(zim, callID);
            L.i(TAG,"onCallInvitationTimeout invitee = " + "  : callID = " + callID);

            mDispatchService.onCallInvitationTimeout(zim,callID);
        }

        @Override
        public void onCallInviteesAnsweredTimeout(ZIM zim, ArrayList<String> invitees, String callID) {
            super.onCallInviteesAnsweredTimeout(zim, invitees, callID);
            for(String str:invitees){
                L.i(TAG,"onCallInviteesAnsweredTimeout invitee = " + str + "  : callID = " + callID);
            }
            mDispatchService.onCallInviteesAnsweredTimeout(zim,invitees,callID);
        }


    };



}
