package im.zego.chathouse.utils;

import android.app.Activity;
import android.widget.ImageView;

import com.bumptech.glide.Glide;
import com.bumptech.glide.load.engine.DiskCacheStrategy;

/**
 * 图片加载类
 */
public class GlideImageLoader {

    /**
     * 单例
     */
    private static volatile GlideImageLoader mInstance;

    private GlideImageLoader() {
    }

    public static GlideImageLoader getInstance() {
        if (mInstance == null) {
            synchronized (GlideImageLoader.class) {
                if (mInstance == null) {
                    mInstance = new GlideImageLoader();
                    return mInstance;
                }
            }
        }
        return mInstance;
    }

    /**
     * 在一个imageView里面异步展示一个图片
     *
     * @param resourceId 资源id
     * @param imageView 图片视图
     */
    private void displayImage(int resourceId, ImageView imageView) {
        Glide.with(imageView.getContext())
                .load(resourceId)
                .dontAnimate()
                .circleCrop()
                .diskCacheStrategy(DiskCacheStrategy.NONE) // 设置缓存的策略
                .into(imageView);
    }


    /**
     * 通过url 加载图片
     * @param url 图片url
     * @param imageView 图片视图
     * @param isCircle 是否是圆形
     */
    private void displayImage(String url, ImageView imageView,boolean isCircle) {

        if(imageView.getContext() instanceof Activity){
           if( ( (Activity)imageView.getContext()).isDestroyed()){
               return;
           }
        }

        if(isCircle) {
            Glide.with(imageView.getContext())
                    .load(url)
                    .dontAnimate()
                    .circleCrop()
                    .into(imageView);
        }else {
            Glide.with(imageView.getContext())
                    .load(url)
                    .dontAnimate()
                    .into(imageView);
        }
    }

    public void displayCircleImageByUrl(String url, ImageView imageView) {
            displayImage(url, imageView,true);

    }

    public void displayImageByUrl(String url, ImageView imageView) {
        displayImage(url, imageView,false);

    }

}
