package im.zego.chathouse.view;

import android.animation.Animator;
import android.animation.AnimatorListenerAdapter;
import android.animation.ValueAnimator;
import android.content.Context;
import android.content.res.TypedArray;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.util.AttributeSet;
import android.view.View;

import androidx.annotation.Nullable;

import im.zego.chathouse.R;


/**
 * 水波效果动画视图
 */
public class ChatHouseWaterWaveView extends View {

    private Paint mProgressPaint;

    private float mWaterWaveStartRadius = 100;//开始距离中心点的半径
    private float mWaterWaveEndRadius = 400;//开始距离中心点的半径
    private int mWaterWaveIntervalTime = 1000; //间隔时间,毫秒
    private float mWaterWaveSpeed = 0.05f;  //水波速度，每毫秒多少像素

    private long mStartTime = 0;
    //圆环宽度
    private float mProgressBarWidth = 10;

    ValueAnimator mValueAnimator;

    public ChatHouseWaterWaveView(Context context) {
        this(context,null);
    }

    public ChatHouseWaterWaveView(Context context, @Nullable AttributeSet attrs) {
        this(context, attrs,0);
    }

    public ChatHouseWaterWaveView(Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        init(context,attrs);
    }

    private void init(Context context, AttributeSet attrs){
        TypedArray a = context.obtainStyledAttributes(attrs, R.styleable.ChatHouseWaterWaveView);
        mProgressBarWidth = a.getDimension(R.styleable.ChatHouseWaterWaveView_waterWaveViewBarWidth,10);
        int mProgressColor = a.getColor(R.styleable.ChatHouseWaterWaveView_waterWaveViewColor, Color.parseColor("#FFA754FF"));
        mWaterWaveStartRadius = a.getDimension(R.styleable.ChatHouseWaterWaveView_waterWaveViewStartRadius,100);
        mWaterWaveEndRadius = a.getDimension(R.styleable.ChatHouseWaterWaveView_waterWaveViewEndRadius,400);
        mWaterWaveIntervalTime = a.getInt(R.styleable.ChatHouseWaterWaveView_waterWaveViewIntervalTime,400);
        mWaterWaveSpeed = a.getDimension(R.styleable.ChatHouseWaterWaveView_waterWaveViewSpeed,0.05f);
        a.recycle();


        mProgressPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
        mProgressPaint.setStyle(Paint.Style.STROKE);
        mProgressPaint.setStrokeWidth(mProgressBarWidth);
        mProgressPaint.setColor(mProgressColor);
        mProgressPaint.setStrokeCap(Paint.Cap.ROUND);
    }

    @Override
    protected void onDraw(Canvas canvas) {
        super.onDraw(canvas);

        if(mStartTime == 0){
            return;
        }

        float perTime = (mWaterWaveEndRadius - mWaterWaveStartRadius) / mWaterWaveSpeed;
        float total = perTime /  mWaterWaveIntervalTime;
        for(int i = 0; i < total;i++){
            if(mStartTime - (long) i * mWaterWaveIntervalTime >= 0){
                float position = ((mStartTime - i * mWaterWaveIntervalTime) % perTime) / perTime;
                float positionRadius = (position) * (mWaterWaveEndRadius - mWaterWaveStartRadius - mProgressBarWidth * 2) ;
                int alpha = (int) (255 - (position) * 255);
                mProgressPaint.setAlpha(alpha);
                canvas.drawCircle(getWidth()/2.0f,getHeight()/2.0f,mWaterWaveStartRadius + positionRadius,mProgressPaint);
            }
        }

    }

    /**
     *  开始倒计时
     */
    public void startCountDown(){
        if(mValueAnimator == null){
            mValueAnimator = ValueAnimator.ofFloat(0, 1);
            mValueAnimator.addListener(new AnimatorListenerAdapter() {
                @Override
                public void onAnimationEnd(Animator animation) {
                    super.onAnimationEnd(animation);
                    mStartTime = 0;
                }
            });
        }
        if(mValueAnimator.isRunning()){
            return;
        }
        mStartTime = 0;
        long temp = mStartTime;
        mValueAnimator.setDuration(Long.MAX_VALUE);
//        mValueAnimator.reverse();
        mValueAnimator.removeAllUpdateListeners();
        mValueAnimator.addUpdateListener(new ValueAnimator.AnimatorUpdateListener(){

            @Override
            public void onAnimationUpdate(ValueAnimator animation) {
                mStartTime = temp +  (long) (animation.getCurrentPlayTime());
                postInvalidate();
            }
        });


        mValueAnimator.start();
        // 触发绘制
        postInvalidate();
    }

    public void cancelCountDown(){
        if(mValueAnimator != null){
            mValueAnimator.cancel();
        }
    }

    @Override
    protected void onAttachedToWindow() {
        super.onAttachedToWindow();
    }

    @Override
    protected void onDetachedFromWindow() {
        super.onDetachedFromWindow();
        cancelCountDown();
    }
}
