package im.zego.enjoycommon.log;

import static im.zego.enjoycommon.util.ZegoGoUtil.ZEGO_LOGS_SUBPATH;

import android.app.Application;
import android.util.Log;

import com.elvishew.xlog.LogConfiguration;
import com.elvishew.xlog.XLog;
import com.elvishew.xlog.flattener.ClassicFlattener;
import com.elvishew.xlog.printer.Printer;
import com.elvishew.xlog.printer.file.FilePrinter;
import com.elvishew.xlog.printer.file.backup.NeverBackupStrategy;
import com.elvishew.xlog.printer.file.clean.FileLastModifiedCleanStrategy;

import java.io.File;

import im.zego.enjoycommon.util.ZegoGoUtil;

public class ZegoAppLog {
    public static void init(Application application) {
        // 初始化 XLog
        LogConfiguration config = new LogConfiguration.Builder().build();
        Printer filePrinter = new FilePrinter                      // 打印日志到文件的打印器
                .Builder(ZegoGoUtil.getStorageFilesPath(application) + File.separator + ZEGO_LOGS_SUBPATH)    // 指定保存日志文件的路径
                .fileNameGenerator(new CustomFileNameGenerator())        // 指定日志文件名生成器
                .backupStrategy(new NeverBackupStrategy())             // 指定日志文件备份策略，默认为 FileSizeBackupStrategy(1024 * 1024)
                .cleanStrategy(new FileLastModifiedCleanStrategy(3L*24*60*60*1000))     // 指定日志文件清除策略，默认为 NeverCleanStrategy()
                .flattener(new ClassicFlattener())      // 指定日志格式
                .build();
        XLog.init(config, filePrinter);
    }

    public static void i(String tag, String msg, Object... args) {
        String message = String.format(msg, args);
        XLog.i(String.format("%s, %s", tag, message));

        try {
            Log.i(tag, String.format(msg, args));
        }catch (Exception e){
            Log.i(tag, msg);
        }
    }

    public static void v(String tag, String msg, Object... args){
        String message = String.format(msg, args);
        XLog.v(String.format("%s, %s", tag, message));

        try {
            Log.v(tag, String.format(msg, args));
        }catch (Exception e){
            Log.v(tag, msg);
        }
    }

    public static void d(String tag, String msg, Object... args){
        String message = String.format(msg, args);
        XLog.d(String.format("%s, %s", tag, message));

        try {
            Log.d(tag, String.format(msg, args));
        }catch (Exception e){
            Log.d(tag, msg);
        }
    }

    public static void w(String tag, String msg, Object... args){
        String message = String.format(msg, args);
        XLog.w(String.format("%s, %s", tag, message));

        try {
            Log.w(tag, String.format(msg, args));
        }catch (Exception e){
            Log.w(tag, msg);
        }
    }
    
    public static void e(String tag, String msg, Object... args){
        String message = String.format(msg, args);
        XLog.e(String.format("%s, %s", tag, message));

        try {
            Log.e(tag, String.format(msg, args));
        }catch (Exception e){
            Log.e(tag, msg);
        }
    }
}
