package im.zego.enjoycommon.util;

import android.app.Activity;
import android.app.Application;
import android.content.ComponentCallbacks;
import android.content.Context;
import android.content.res.Configuration;
import android.os.Bundle;
import android.util.DisplayMetrics;

import androidx.annotation.NonNull;

/**
 * 屏幕适配工具类(适配宽度)
 */
public class ScreenAdaptationUtil {

    private static DisplayMetrics mAppDisplayMetrics;
    private static float mDensity;
    private static float mScaledDensity;
    private static float TARGET_WIDTH;

    private static ComponentCallbacks callback = null;
    private static Application.ActivityLifecycleCallbacks callback2 = null;

    public static void init(final Context application, float targetWidth) {
        TARGET_WIDTH = targetWidth;
        mAppDisplayMetrics = application.getResources().getDisplayMetrics();

        if(mDensity == 0) {
            mDensity = mAppDisplayMetrics.density;
            mScaledDensity = mAppDisplayMetrics.scaledDensity;
            application.unregisterComponentCallbacks(callback);
            callback = new ComponentCallbacks() {
                @Override
                public void onConfigurationChanged(Configuration newConfig) {
                    if (newConfig != null && newConfig.fontScale > 0) {
                        mScaledDensity = application.getResources().getDisplayMetrics().scaledDensity;
                    }
                }

                @Override
                public void onLowMemory() {
                }
            };
            application.registerComponentCallbacks(callback);

        }
        ((Application)application).unregisterActivityLifecycleCallbacks(callback2);
        callback2 = new Application.ActivityLifecycleCallbacks() {
            @Override
            public void onActivityCreated(Activity activity, Bundle savedInstanceState) {
                setCustomDensity(activity);
            }
            @Override
            public void onActivityStarted(Activity activity) {
            }

            @Override
            public void onActivityPreResumed(@NonNull Activity activity) {
                setCustomDensity(activity);
            }

            @Override
            public void onActivityResumed(Activity activity) {

            }
            @Override
            public void onActivityPaused(Activity activity) {
            }
            @Override
            public void onActivityStopped(Activity activity) {
            }
            @Override
            public void onActivitySaveInstanceState(Activity activity, Bundle outState) {
            }
            @Override
            public void onActivityDestroyed(Activity activity) {

            }
        };

        ((Application)application).registerActivityLifecycleCallbacks(callback2);
    }

    public static void unInit(final Context application)
    {
        application.unregisterComponentCallbacks(callback);
        ((Application)application).unregisterActivityLifecycleCallbacks(callback2);
        callback= null;
        callback2 = null;
    }

    public static void setCustomDensity(final Activity activity ) {
        float targetDensity = mAppDisplayMetrics.widthPixels / TARGET_WIDTH;
        float targetScaleDensity = targetDensity * (mScaledDensity / mDensity);
        int targetDensityDpi = (int) (160 * targetDensity);

        DisplayMetrics activityDisplayMetrics = activity.getResources().getDisplayMetrics();
        activityDisplayMetrics.density = targetDensity;
        activityDisplayMetrics.scaledDensity = targetScaleDensity;
        activityDisplayMetrics.densityDpi = targetDensityDpi;
    }

}
