package im.zego.enjoycommon.util;

import android.os.Handler;
import android.os.Looper;

import java.util.concurrent.ThreadFactory;
import java.util.concurrent.atomic.AtomicInteger;

public class ThreadHelper {

    public static final int CPU_COUNT = Runtime.getRuntime().availableProcessors();

    public static final Handler handler = new Handler(Looper.getMainLooper());

    /**
     * 线程类型
     */
    private static final byte TYPE_SINGLE = -1;
    private static final byte TYPE_CACHED = -2;
    private static final byte TYPE_IO = -3;


    /**
     * 当前是否在主线程中
     *
     * @return true:在主线程 false:不在
     */
    public static boolean isMainThread() {
        return Looper.myLooper() == Looper.getMainLooper();
    }

    /**
     * ui线程任务执行
     *
     * @param runnable
     */
    public static void runOnUiThread(Runnable runnable) {
        runOnUiThread(runnable, 0);
    }

    /**
     * ui线程任务执行
     *
     * @param runnable
     * @param delay    延时时间，毫秒
     */
    public static void runOnUiThread(Runnable runnable, long delay) {
        if (isMainThread() && delay <= 0) {
            runnable.run();
        } else {
            handler.postDelayed(runnable, delay);
        }
    }

    private static final class LocalThreadFactory implements ThreadFactory {
        private String threadName;
        private AtomicInteger statics = new AtomicInteger();

        public LocalThreadFactory(String threadName) {
            this.threadName = threadName;
        }

        @Override
        public Thread newThread(Runnable r) {

            return new Thread(r, threadName());
        }

        private String threadName() {
            statics.incrementAndGet();
            return threadName + "(" + statics.get() + ")";
        }
    }

}
